@@ -11,12 +11,22 @@
  */
 
 #include <stdio.h>
+
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#endif
+
 #include <stdlib.h>
 #include <string.h>
 
+#include "config.h"
 #include "filter.h"
 #include "frame.h"
+#include "dsputil.h"
+
+#ifdef MMX
+#include "i386/mmx.h"
+#endif
 
 //Regular filter
 #define LUMA_THRESHOLD_DEFAULT 15
@@ -49,144 +59,6 @@ typedef struct ThisFilter
 
 } ThisFilter;
 
-#define MM_MMX    0x0001 /* standard MMX */
-#define MM_3DNOW  0x0004 /* AMD 3DNOW */
-#define MM_MMXEXT 0x0002 /* SSE integer functions or AMD MMX ext */
-#define MM_SSE    0x0008 /* SSE functions */
-#define MM_SSE2   0x0010 /* PIV SSE2 functions */
-
-#ifdef i386
-
-//From linearblend
-#define cpuid(index,eax,ebx,ecx,edx)\
-    __asm __volatile\
-        ("movl %%ebx, %%esi\n\t"\
-         "cpuid\n\t"\
-         "xchgl %%ebx, %%esi"\
-         : "=a" (eax), "=S" (ebx),\
-           "=c" (ecx), "=d" (edx)\
-         : "0" (index));
-
-int mm_support(void) // From linearblend
-{
-  int rval;
-  int eax, ebx, ecx, edx;
-
-  __asm__ __volatile__ (
-			/* See if CPUID instruction is supported ... */
-			/* ... Get copies of EFLAGS into eax and ecx */
-			"pushf\n\t"
-			"popl %0\n\t"
-			"movl %0, %1\n\t"
-
-			/* ... Toggle the ID bit in one copy and store */
-			/*     to the EFLAGS reg */
-			"xorl $0x200000, %0\n\t"
-			"push %0\n\t"
-			"popf\n\t"
-
-			/* ... Get the (hopefully modified) EFLAGS */
-			"pushf\n\t"
-			"popl %0\n\t"
-			: "=a" (eax), "=c" (ecx)
-			:
-			: "cc"
-			);
-
-  if (eax == ecx)
-    return 0; /* CPUID not supported */
-
-  cpuid(0, eax, ebx, ecx, edx);
-    
-  if (ebx == 0x756e6547 &&
-      edx == 0x49656e69 &&
-      ecx == 0x6c65746e) {
-
-    /* intel */
-  inteltest:
-    cpuid(1, eax, ebx, ecx, edx);
-    if ((edx & 0x00800000) == 0)
-      return 0;
-    rval = MM_MMX;
-    if (edx & 0x02000000)
-      rval |= MM_MMXEXT | MM_SSE;
-    if (edx & 0x04000000)
-      rval |= MM_SSE2;
-    return rval;
-  } else if (ebx == 0x68747541 &&
-	     edx == 0x69746e65 &&
-	     ecx == 0x444d4163) {
-    /* AMD */
-    cpuid(0x80000000, eax, ebx, ecx, edx);
-    if ((unsigned)eax < 0x80000001)
-      goto inteltest;
-    cpuid(0x80000001, eax, ebx, ecx, edx);
-    if ((edx & 0x00800000) == 0)
-      return 0;
-    rval = MM_MMX;
-    if (edx & 0x80000000)
-      rval |= MM_3DNOW;
-    if (edx & 0x00400000)
-      rval |= MM_MMXEXT;
-    return rval;
-  } else if (ebx == 0x746e6543 &&
-	     edx == 0x48727561 &&
-	     ecx == 0x736c7561) {  /*  "CentaurHauls" */
-    /* VIA C3 */
-    cpuid(0x80000000, eax, ebx, ecx, edx);
-    if ((unsigned)eax < 0x80000001)
-      goto inteltest;
-    cpuid(0x80000001, eax, ebx, ecx, edx);
-    rval = 0;
-    if( edx & ( 1 << 31) )
-      rval |= MM_3DNOW;
-    if( edx & ( 1 << 23) )
-      rval |= MM_MMX;
-    if( edx & ( 1 << 24) )
-      rval |= MM_MMXEXT;
-    if (rval==0)
-      goto inteltest;
-    return rval;
-  } else if (ebx == 0x69727943 &&
-	     edx == 0x736e4978 &&
-	     ecx == 0x64616574) {
-    /* Cyrix Section */
-    /* See if extended CPUID level 80000001 is supported */
-    /* The value of CPUID/80000001 for the 6x86MX is undefined
-       according to the Cyrix CPU Detection Guide (Preliminary
-       Rev. 1.01 table 1), so we'll check the value of eax for
-       CPUID/0 to see if standard CPUID level 2 is supported.
-       According to the table, the only CPU which supports level
-       2 is also the only one which supports extended CPUID levels.
-    */
-    if (eax != 2)
-      goto inteltest;
-    cpuid(0x80000001, eax, ebx, ecx, edx);
-    if ((eax & 0x00800000) == 0)
-      return 0;
-    rval = MM_MMX;
-    if (eax & 0x01000000)
-      rval |= MM_MMXEXT;
-    return rval;
-  } else if (ebx == 0x756e6547 &&
-             edx == 0x54656e69 &&
-             ecx == 0x3638784d) {
-    /* Tranmeta Crusoe */
-    cpuid(0x80000000, eax, ebx, ecx, edx);
-    if ((unsigned)eax < 0x80000001)
-      return 0;
-    cpuid(0x80000001, eax, ebx, ecx, edx);
-    if ((edx & 0x00800000) == 0)
-      return 0;
-    return MM_MMX;
-  } else {
-    return 0;
-  }
-}
-#else // i386
-int mm_support(void) { return 0; }
-#endif
-
 int quickdnr(VideoFilter *f, VideoFrame *frame)
 {  
   ThisFilter *tf = (ThisFilter *)f; 
@@ -198,7 +70,7 @@ int quickdnr(VideoFilter *f, VideoFrame *frame)
     tf->first = 0;
   }
   for(y = 0;y < tf->Luma_size;y++) {
-    if(abs(tf->average[y] - frame->buf[y]) < tf->Luma_threshold1) {
+    if (abs(tf->average[y] - frame->buf[y]) < tf->Luma_threshold1) {
       tf->average[y] = (tf->average[y] + frame->buf[y]) >> 1;
       frame->buf[y] = tf->average[y];
     }
@@ -206,7 +78,7 @@ int quickdnr(VideoFilter *f, VideoFrame *frame)
   }
   
   for(y = tf->Luma_size;y < tf->UV_size;y++) {
-    if(abs(tf->average[y] - frame->buf[y]) < tf->Chroma_threshold1) {
+    if (abs(tf->average[y] - frame->buf[y]) < tf->Chroma_threshold1) {
       tf->average[y] = (tf->average[y] + frame->buf[y]) >> 1;
       frame->buf[y] = tf->average[y];
     }
@@ -230,8 +102,8 @@ int quickdnr2(VideoFilter *f, VideoFrame *frame)
 
   for(y = 0; y < tf->Luma_size; y++) {
     t = abs(tf->average[y] - frame->buf[y]);
-    if(t < tf->Luma_threshold1) {
-      if(t > tf->Luma_threshold2)
+    if (t < tf->Luma_threshold1) {
+      if (t > tf->Luma_threshold2)
 	tf->average[y] = (tf->average[y] + frame->buf[y]) >> 1;
       frame->buf[y] = tf->average[y];
     }
@@ -240,8 +112,8 @@ int quickdnr2(VideoFilter *f, VideoFrame *frame)
 
  for(y = tf->Luma_size; y < (tf->UV_size); y++) {
     t = abs(tf->average[y] - frame->buf[y]);
-    if(t < tf->Chroma_threshold1) {
-      if(t > tf->Chroma_threshold2)
+    if (t < tf->Chroma_threshold1) {
+      if (t > tf->Chroma_threshold2)
 	tf->average[y] = (tf->average[y] + frame->buf[y]) >> 1;
       frame->buf[y] = tf->average[y];
     }
@@ -253,7 +125,7 @@ int quickdnr2(VideoFilter *f, VideoFrame *frame)
  return 0;
 }
 
-#ifdef i386
+#ifdef MMX
 
 int quickdnrMMX(VideoFilter *f, VideoFrame *frame)
 {
@@ -271,9 +143,23 @@ int quickdnrMMX(VideoFilter *f, VideoFrame *frame)
     tf->first = 0;
   }
 
-  asm volatile("prefetch 64(%0)     \n\t" //Experimental values from athlon
-	       "prefetch 64(%1)     \n\t"
-	       "prefetch 64(%2)     \n\t"
+  /*
+    Commented out all the prefetches. These don't do anything when
+    you are processing an array with sequential accesses because the
+    processor automatically does a prefetchT0 in these cases. The
+    instruction is meant to be used to specify a different prefetch
+    cache level, or to prefetch non-sequental data.
+    
+    These prefetches are not available on all MMX processors so if
+    we wanted to use them we would need to test for a prefetch
+    capable processor before using them. -- dtk
+  */
+
+  asm volatile(/*
+               "prefetch 64(%0)     \n\t" //Experimental values from athlon
+               "prefetch 64(%1)     \n\t"
+               "prefetch 64(%2)     \n\t"
+               -- pointless & breaks on some processors */
 	       "movq (%0), %%mm4    \n\t"
 	       "movq (%1), %%mm5    \n\t"
 	       "movq (%2), %%mm6    \n\t"
@@ -281,8 +167,10 @@ int quickdnrMMX(VideoFilter *f, VideoFrame *frame)
 	       );
 
   for(y = 0;y < (tf->Luma_size);y += 8) { //Luma
-    asm volatile("prefetchw 384(%0)    \n\t" //Experimental values from athlon
+    asm volatile(/*
+                 "prefetchw 384(%0)    \n\t" //Experimental values from athlon
 		 "prefetch 384(%1)     \n\t"
+                 -- pointless & breaks on some processors */
 
 		 "movq (%0), %%mm0     \n\t" //av-p
 		 "movq (%1), %%mm1     \n\t" //buf
@@ -313,8 +201,10 @@ int quickdnrMMX(VideoFilter *f, VideoFrame *frame)
   }
 
   for(y = tf->Luma_size;y < tf->UV_size;y += 8) { //Chroma
-    asm volatile("prefetchw 384(%0)    \n\t" //Experimental values for athlon
+    asm volatile(/*
+                 "prefetchw 384(%0)    \n\t" //Experimental values for athlon
 		 "prefetch 384(%1)     \n\t"
+                 -- pointless & breaks on some processors */
 
 		 "movq (%0), %%mm0     \n\t" //av-p
 		 "movq (%1), %%mm1     \n\t" //buf
@@ -369,16 +259,20 @@ int quickdnr2MMX(VideoFilter *f, VideoFrame *frame)
     tf->first = 0;
   }
 
-  asm volatile("prefetch 64(%0)     \n\t" //Experimental values from athlon
-	       "prefetch 64(%1)     \n\t"
+  asm volatile(/*
+               "prefetch 64(%0)     \n\t" //Experimental values from athlon
+               "prefetch 64(%1)     \n\t"
+               -- pointless & breaks on some processors */
 	       "movq (%0), %%mm4    \n\t"
 	       "movq (%1), %%mm5    \n\t"
 	       : : "r" (&sign_convert), "r" (&tf->Luma_threshold_mask1)
 	       );
 
   for(y = 0;y < (tf->Luma_size);y += 8) { //Luma
-    asm volatile("prefetchw 384(%0)    \n\t" //Experimental values from athlon
-		 "prefetch 384(%1)     \n\t"
+    asm volatile(/*
+                 "prefetchw 384(%0)    \n\t" //Experimental values from athlon
+                 "prefetch 384(%1)     \n\t"
+                 -- pointless & breaks on some processors */
 
 		 "movq (%0), %%mm0     \n\t" //av-p
 		 "movq (%1), %%mm1     \n\t" //buf
@@ -430,14 +324,18 @@ int quickdnr2MMX(VideoFilter *f, VideoFrame *frame)
     av_p++;
   }
 
-  asm volatile("prefetch 64(%0)     \n\t" //Experimental values from athlon
+  asm volatile(/*
+               "prefetch 64(%0)     \n\t" //Experimental values from athlon
+               -- pointless & breaks on some processors */
 	       "movq (%1), %%mm5    \n\t"
 	       : : "r" (&sign_convert), "r" (&tf->Chroma_threshold_mask1)
 	       );
 
   for(y = tf->Luma_size;y < tf->UV_size;y += 8) { //Chroma
-    asm volatile("prefetchw 384(%0)    \n\t" //Experimental values for athlon
-		 "prefetch 384(%1)     \n\t"
+    asm volatile(/*
+                 "prefetchw 384(%0)    \n\t" //Experimental values from athlon
+                 "prefetch 384(%1)     \n\t"
+                 -- pointless & breaks on some processors */
 
 		 "movq (%0), %%mm0     \n\t" //av-p
 		 "movq (%1), %%mm1     \n\t" //buf
@@ -495,16 +393,7 @@ int quickdnr2MMX(VideoFilter *f, VideoFrame *frame)
 
   return 0;
 }
-#else // i386
-int quickdnrMMX(VideoFilter *f, VideoFrame *frame) {
-     (void)f; (void)frame;
-     return 0;
-};
-int quickdnr2MMX(VideoFilter *f, VideoFrame *frame) {
-     (void)f; (void)frame;
-     return 0;
-};
-#endif // i386
+#endif /* MMX */
 
 void cleanup(VideoFilter *vf)
 {
@@ -582,8 +471,9 @@ VideoFilter *new_filter(VideoFrameType inpixfmt, VideoFrameType outpixfmt,
     double_threshold = 1;
   }
 
-  if(mm_support() > MM_MMXEXT) {
-    if(double_threshold) filter->vf.filter = &quickdnr2MMX;
+#ifdef MMX
+  if (mm_support() > MM_MMXEXT) {
+    if (double_threshold) filter->vf.filter = &quickdnr2MMX;
     else filter->vf.filter = &quickdnrMMX;
     
     filter->Luma_threshold_mask1 = 0;
@@ -608,7 +498,9 @@ VideoFilter *new_filter(VideoFrameType inpixfmt, VideoFrameType outpixfmt,
     }
 
   }
-  else if(double_threshold) filter->vf.filter = &quickdnr2;
+  else
+#endif
+  if (double_threshold) filter->vf.filter = &quickdnr2;
   else filter->vf.filter = &quickdnr;
 
   filter->first = 1;
