@@ -7,7 +7,7 @@
  * This plays MythTV audio through the default output device on OS X.
  *
  * = REVISION
- * $Id: audiooutputca.cpp,v 1.2 2004/11/11 07:36:44 ijr Exp $
+ * $Id: audiooutputca.cpp 10219 2006-06-16 12:58:38Z danielk $
  *
  * = AUTHORS
  * Jeremiah Morris
@@ -21,6 +21,7 @@ using namespace std;
 
 #include "mythcontext.h"
 #include "audiooutputca.h"
+#include "config.h"
 
 // this holds Core Audio member variables
 struct CoreAudioData {
@@ -36,18 +37,23 @@ OSStatus AuCA_AURender(void *inRefCon,
                        AudioBufferList *ioData);
 
 
-AudioOutputCA::AudioOutputCA(QString audiodevice, int laudio_bits, 
-                             int laudio_channels, int laudio_samplerate,
-                             AudioOutputSource source, bool set_initial_vol)
-             : AudioOutputBase(audiodevice, laudio_bits,
-                               laudio_channels, laudio_samplerate,
-                               source, set_initial_vol)
+AudioOutputCA::AudioOutputCA(
+    QString laudio_main_device, QString           laudio_passthru_device,
+    int     laudio_bits,        int               laudio_channels,
+    int     laudio_samplerate,  AudioOutputSource lsource,
+    bool    lset_initial_vol,   bool              laudio_passthru)
+    : AudioOutputBase(laudio_main_device, laudio_passthru_device,
+                      laudio_bits,        laudio_channels,
+                      laudio_samplerate,  lsource,
+                      lset_initial_vol,   laudio_passthru),
+      coreaudio_data(new CoreAudioData()),
+      bufferedBytes(0)
 {
     // Create private data
-    coreaudio_data = new CoreAudioData();
     coreaudio_data->output_unit = NULL;
     
-    Reconfigure(laudio_bits, laudio_channels, laudio_samplerate);
+    Reconfigure(laudio_bits, laudio_channels,
+                laudio_samplerate, laudio_passthru);
 }
 
 AudioOutputCA::~AudioOutputCA()
@@ -107,8 +113,10 @@ bool AudioOutputCA::OpenDevice()
     bzero(&conv_in_desc, sizeof(AudioStreamBasicDescription));
     conv_in_desc.mSampleRate       = audio_samplerate;
     conv_in_desc.mFormatID         = kAudioFormatLinearPCM;
-    conv_in_desc.mFormatFlags      = kLinearPCMFormatFlagIsSignedInteger
-                                     | kLinearPCMFormatFlagIsBigEndian;
+    conv_in_desc.mFormatFlags      = kLinearPCMFormatFlagIsSignedInteger;
+#ifdef WORDS_BIGENDIAN
+    conv_in_desc.mFormatFlags     |= kLinearPCMFormatFlagIsBigEndian;
+#endif
     conv_in_desc.mBytesPerPacket   = audio_bytes_per_sample;
     conv_in_desc.mFramesPerPacket  = 1;
     conv_in_desc.mBytesPerFrame    = audio_bytes_per_sample;
@@ -172,6 +180,7 @@ bool AudioOutputCA::RenderAudio(unsigned char *aubuf,
 {
     if (pauseaudio || killaudio)
     {
+        audio_actually_paused = true;
         return false;
     }
     
