@@ -1,27 +1,42 @@
 #ifndef AUDIOOUTPUTDX
 #define AUDIOOUTPUTDX
 
+/* ACK! <windows.h> and <dsound.h> should only be in cpp's compiled
+ * in windows only. Some of the variables in AudioOutputDX need to
+ * be moved to a private class before removing these includes though.
+ */
+#include <windows.h> // HACK HACK HACK
+#include <dsound.h>  // HACK HACK HACK
+
+// Qt headers
 #include <qstring.h>
 
-using namespace std;
-
-#include "audiooutput.h"
-
-#include <windows.h>
-//#include <mmsystem.h>
-#include <dsound.h>
+// MythTV headers
+#include "audiooutputbase.h"
 
 class AudioOutputDX : public AudioOutputBase
 {
 public:
-    AudioOutputDX(QString audiodevice, int audio_bits, 
-                  int audio_channels, int audio_samplerate,
-                  AudioOutputSource source, bool set_initial_vol);
+    AudioOutputDX(QString laudio_main_device,
+                  QString laudio_passthru_device,
+                  int laudio_bits,
+                  int laudio_channels, int laudio_samplerate,
+                  AudioOutputSource lsource,
+                  bool lset_initial_vol, bool laudio_passthru);
     virtual ~AudioOutputDX();
 
+    /// BEGIN HACK HACK HACK HACK These need to actually be implemented!
+    bool OpenDevice(void) { return false; }
+    void CloseDevice(void) {}
+    void WriteAudio(unsigned char*, int) {}
+    virtual int getSpaceOnSoundcard(void) { return 0; }
+    virtual int getBufferedOnSoundcard(void) { return 0; }
+#warning Several methods in AudioOutputDX need to be implemented...
+    /// END HACK HACK HACK HACK
+	
     virtual void Reset(void);
-    virtual void Reconfigure(int audio_bits, 
-                         int audio_channels, int audio_samplerate);
+    virtual void Reconfigure(int audio_bits,       int audio_channels,
+                             int audio_samplerate, int audio_passthru);
     virtual void SetBlocking(bool blocking);
 
     virtual bool AddSamples(char *buffer, int samples, long long timecode);
@@ -55,7 +70,6 @@ public:
     
     bool awaiting_data;
  
-    QString audiodevice;
     int effdsp; // from the recorded stream
     int audio_bytes_per_sample;
     int audio_bits;
