@@ -59,9 +59,18 @@ class HttpComms : public QObject
                             int maxRetries = 3, int maxRedirects = 3, 
                             bool allowGzip = false, Credentials* webCred = NULL);
     
+    static QString postHttp(QUrl               &url, 
+                            QHttpRequestHeader *pAddlHdr          = NULL, 
+                            QIODevice          *pData             = NULL,
+                            int                 timeoutMS         = 10000, 
+                            int                 maxRetries        = 3, 
+                            int                 maxRedirects      = 3, 
+                            bool                allowGzip         = false,
+                            Credentials        *webCred           = NULL, 
+                            bool                isInQtEventThread = true );
     
     void request(QUrl &url, int timeoutms = -1, bool allowGzip = false);
-    void request(QUrl &url, QHttpRequestHeader &header, int timeoutms = -1);
+    void request(QUrl &url, QHttpRequestHeader &header, int timeoutms = -1, QIODevice *pData = NULL);
     
     void setCookie( const QString& cookie ) { m_cookie = cookie; }
     const QString& getCookie() const { return m_cookie; }
