@@ -682,11 +682,10 @@ void ManagedList::cursorDown(bool page)
         int newIndex = curGroup->getCurIndex();
         int itemCount = curGroup->getItemCount();
 
-        newIndex += (page ? itemCount : 1);
+        newIndex += (page ? listSize : 1);
 
         if (newIndex >= itemCount)
-            newIndex = newIndex - itemCount;
-
+            newIndex = (page ? itemCount - 1 : newIndex - itemCount);
 
         while(curGroup->getItem(newIndex)->getEnabled() == false)
         {
@@ -708,9 +707,9 @@ void ManagedList::cursorUp(bool page)
         int newIndex = curGroup->getCurIndex();
         int itemCount = curGroup->getItemCount();
 
-        newIndex -= (page ? itemCount : 1);
+        newIndex -= (page ? listSize : 1);
         if (newIndex < 0)
-            newIndex = itemCount + newIndex;
+            newIndex = (page ? 0 : itemCount + newIndex);
 
         while(curGroup->getItem(newIndex)->getEnabled() == false)
         {
