@@ -11,17 +11,13 @@ class MythCDROM : public MythMediaDevice
               bool AllowEject);
     virtual ~MythCDROM() {};
 
-    // Commented out functions are implemented in MythMediaDevice, so can't
-    // be abstract here.
-
-    //virtual MediaError testMedia(void) = 0;
-    virtual bool mediaChanged(void) = 0;
-    virtual bool checkOK(void) = 0;
+    virtual bool mediaChanged(void) { return false; }
+    virtual bool checkOK(void)      { return true; }
     virtual bool openDevice(void);
-    virtual MediaStatus checkMedia(void) = 0;
-    //virtual MediaError eject(void) = 0;
-    //virtual MediaError lock(void) = 0;
-    //virtual MediaError unlock(void) = 0;
+    virtual MediaStatus checkMedia(void)
+    {
+        return setStatus(MEDIASTAT_UNKNOWN, false);
+    }
 
     static MythCDROM* get(QObject* par, const char* devicePath, bool SuperMount,
                           bool AllowEject);
@@ -42,7 +38,7 @@ public:
     virtual bool mediaChanged(void);
     virtual bool checkOK(void);
     virtual MediaStatus checkMedia(void);
-    virtual MediaError eject(void);
+    virtual MediaError eject(bool open_close = true);
     virtual MediaError lock(void);
     virtual MediaError unlock(void);
 };
@@ -52,15 +48,12 @@ class MythCDROMFreeBSD: public MythCDROM
 {
 public:
     MythCDROMFreeBSD(QObject* par, const char* DevicePath, bool SuperMount,
-        bool AllowEject):
+                     bool AllowEject):
         MythCDROM(par, DevicePath, SuperMount, AllowEject) {
     }
 
     virtual MediaError testMedia(void);
-    virtual bool mediaChanged(void);
-    virtual bool checkOK(void);
-    virtual MediaStatus checkMedia(void);
-    virtual MediaError eject(void);
+    virtual MediaError eject(bool open_close = true);
     virtual MediaError lock(void);
     virtual MediaError unlock(void);
 };
