@@ -13,6 +13,8 @@ using namespace std;
 #include "mythcontext.h"
 #include "util.h"
 #include "mythdialogs.h"
+#include "virtualkeyboard.h"
+#include "libmythui/mythmainwindow.h"
 
 void MythComboBox::keyPressEvent(QKeyEvent *e)
 {
@@ -319,11 +321,24 @@ void MythSlider::focusOutEvent(QFocusEvent *e)
     QSlider::focusOutEvent(e);
 }
 
+MythLineEdit::~MythLineEdit()
+{
+    if (popup)
+        delete popup;
+}
+
+void MythLineEdit::Init()
+{
+    popup = NULL;
+    popupPosition = VK_POSBELOWEDIT;
+}
+
 void MythLineEdit::keyPressEvent(QKeyEvent *e)
 {
     bool handled = false;
     QStringList actions;
-    if (gContext->GetMainWindow()->TranslateKeyPress("qt", e, actions))
+    if ((!popup || !popup->isShown()) &&
+        gContext->GetMainWindow()->TranslateKeyPress("qt", e, actions, false))
     {
         for (unsigned int i = 0; i < actions.size() && !handled; i++)
         {
@@ -334,7 +349,22 @@ void MythLineEdit::keyPressEvent(QKeyEvent *e)
                 focusNextPrevChild(false);
             else if (action == "DOWN")
                 focusNextPrevChild(true);
-            else if (action == "SELECT" && e->text().isNull())
+            else if (action == "SELECT" && 
+                    (e->text().isNull() ||
+                    (e->key() == Qt::Key_Enter) ||
+                    (e->key() == Qt::Key_Return)))
+            {
+                if ((allowVirtualKeyboard) &&
+				    (gContext->GetNumSetting("UseVirtualKeyboard", 1) == 1))
+                {
+                    popup = new VirtualKeyboard(gContext->GetMainWindow(), this);
+                    gContext->GetMainWindow()->detach(popup);
+                    popup->exec();
+                    delete popup;
+                    popup = NULL;
+                }
+            }
+            else if (action == "SELECT" && e->text().isNull() )
                 e->ignore();
             else
                 handled = false;
@@ -370,9 +400,23 @@ void MythLineEdit::focusInEvent(QFocusEvent *e)
 void MythLineEdit::focusOutEvent(QFocusEvent *e)
 {
     this->unsetPalette();
+    if (popup && popup->isShown() && !popup->hasFocus())
+        popup->hide();
     QLineEdit::focusOutEvent(e);
 }
 
+void MythLineEdit::hideEvent(QHideEvent *e)
+{
+    if (popup && popup->isShown())
+        popup->hide();
+    QLineEdit::hideEvent(e);
+}
+
+void MythLineEdit::mouseDoubleClickEvent(QMouseEvent *e)
+{
+    QLineEdit::mouseDoubleClickEvent(e);
+}
+
 MythRemoteLineEdit::MythRemoteLineEdit(QWidget * parent, const char * name)
                   : QTextEdit(parent, name)
 {
@@ -452,6 +496,9 @@ void MythRemoteLineEdit::Init()
     setMaximumHeight(fontsize.height() * m_lines * 5 / 4);
 
     connect(cycle_timer, SIGNAL(timeout()), this, SLOT(endCycle()));
+    
+    popup = NULL;
+    popupPosition = VK_POSBELOWEDIT;
 }
 
 void MythRemoteLineEdit::startCycle(QString current_choice, QString set)
@@ -656,7 +703,9 @@ void MythRemoteLineEdit::keyPressEvent(QKeyEvent *e)
 {
     bool handled = false;
     QStringList actions;
-    if (gContext->GetMainWindow()->TranslateKeyPress("qt", e, actions))
+
+    if ((!popup || !popup->isShown()) &&
+          gContext->GetMainWindow()->TranslateKeyPress("qt", e, actions, false))
     {
         for (unsigned int i = 0; i < actions.size() && !handled; i++)
         {
@@ -679,6 +728,22 @@ void MythRemoteLineEdit::keyPressEvent(QKeyEvent *e)
                 QWidget::focusNextPrevChild(true);
                 emit tryingToLooseFocus(true);
             }
+            else if ((action == "SELECT") &&
+                     (!active_cycle) &&
+                     ((e->text().isNull()) ||
+                      (e->key() == Qt::Key_Enter) || 
+                      (e->key() == Qt::Key_Return)))
+            {
+                if (gContext->GetNumSetting("UseVirtualKeyboard", 1) == 1)
+                {
+                    popup = new VirtualKeyboard(gContext->GetMainWindow(), this);
+                    gContext->GetMainWindow()->detach(popup);
+                    popup->exec();
+                    delete popup;
+                    popup = NULL;
+                }
+ 
+            }
             else
                 handled = false;
         }
@@ -687,6 +752,14 @@ void MythRemoteLineEdit::keyPressEvent(QKeyEvent *e)
     if (handled)
         return;
 
+    if (popup && popup->isShown())
+    {
+        endCycle();
+        QTextEdit::keyPressEvent(e);
+        emit textChanged(this->text());
+        return;
+    }
+
     switch (e->key())
     {
         case Key_Enter:
@@ -860,16 +933,41 @@ void MythRemoteLineEdit::focusOutEvent(QFocusEvent *e)
 {
     this->unsetPalette();
 
+    if (popup && popup->isShown() && !popup->hasFocus())
+        popup->hide();
+
     emit lostFocus();
     QTextEdit::focusOutEvent(e);
 }
 
+
 MythRemoteLineEdit::~MythRemoteLineEdit()
 {
     if (cycle_timer)
     {
         delete cycle_timer;
     }
+
+    if (popup)
+        delete popup;
+}
+
+void MythRemoteLineEdit::insert(QString text)
+{
+    QTextEdit::insert(text);
+    emit textChanged(this->text());
+}
+
+void MythRemoteLineEdit::del()
+{
+    doKeyboardAction(QTextEdit::ActionDelete);
+    emit textChanged(this->text());
+}
+
+void MythRemoteLineEdit::backspace()
+{
+    doKeyboardAction(QTextEdit::ActionBackspace);
+    emit textChanged(this->text());
 }
 
 void MythTable::keyPressEvent(QKeyEvent *e)
@@ -934,9 +1032,13 @@ void MythPushButton::keyPressEvent(QKeyEvent *e)
 {
     bool handled = false;
     QStringList actions;
+    keyPressActions.clear();
+
     if (gContext->GetMainWindow()->TranslateKeyPress("qt", (QKeyEvent *)e,
                                                      actions))
     {
+        keyPressActions = actions;
+
         for (unsigned int i = 0; i < actions.size() && !handled; i++)
         {
             QString action = actions[i];
@@ -970,19 +1072,15 @@ void MythPushButton::keyPressEvent(QKeyEvent *e)
 void MythPushButton::keyReleaseEvent(QKeyEvent *e)
 {
     bool handled = false;
-    QStringList actions;
-    if (gContext->GetMainWindow()->TranslateKeyPress("qt", (QKeyEvent *)e,
-                                                     actions))
+    QStringList actions = keyPressActions;
+    for (unsigned int i = 0; i < actions.size() && !handled; i++)
     {
-        for (unsigned int i = 0; i < actions.size() && !handled; i++)
+        QString action = actions[i];
+        if (action == "SELECT")
         {
-            QString action = actions[i];
-            if (action == "SELECT")
-            {
-                QKeyEvent tempe(QEvent::KeyRelease, Key_Space, ' ', 0, " ");
-                QPushButton::keyReleaseEvent(&tempe);
-                handled = true;
-            }
+            QKeyEvent tempe(QEvent::KeyRelease, Key_Space, ' ', 0, " ");
+            QPushButton::keyReleaseEvent(&tempe);
+            handled = true;
         }
     }
 
@@ -1227,6 +1325,10 @@ void MythListBox::keyPressEvent(QKeyEvent* e)
             }
             else if (action == "MENU")
                 emit menuButtonPressed(currentItem());
+            else if (action == "EDIT")
+                emit editButtonPressed(currentItem());
+            else if (action == "DELETE")
+                emit deleteButtonPressed(currentItem());
             else if (action == "SELECT")
                 emit accepted(currentItem());
         }
@@ -1255,4 +1357,3 @@ void MythListBox::focusInEvent(QFocusEvent *e)
     emit changeHelpText(helptext);
     QListBox::focusInEvent(e);
 }
-
