@@ -30,21 +30,56 @@
 #include "mythdbcon.h"
 #include "DisplayRes.h"
 
+/** \class Configurable
+ *  \brief Configurable is the root of all the database aware widgets.
+ *
+ *   This is an abstract class and several methods must be implemented
+ *   in children. load(), save(), and byName(QString), are all abstract.
+ *   And while configWidget(ConfigurationGroup *, QWidget*, const char*)
+ *   has an implementation, all it does is print an error message and
+ *   return a NULL pointer.
+ */
+
 QWidget* Configurable::configWidget(ConfigurationGroup *cg, QWidget* parent,
                                     const char* widgetName) 
 {
     (void)cg;
     (void)parent;
     (void)widgetName;
-    cout << "BUG: Configurable is visible, but has no configWidget\n";
+    VERBOSE(VB_IMPORTANT, "BUG: Configurable is visible, but has no "
+            "configWidget");
     return NULL;
 }
 
-ConfigurationGroup::~ConfigurationGroup()  {
-    for(childList::iterator i = children.begin() ;
-        i != children.end() ;
-        ++i )
-        delete *i;
+/** \fn Configurable::enableOnSet(const QString &val)
+ *  \brief This slot allows you to enable this configurable when a
+ *         binary configurable is set to true.
+ *  \param val signal value, should be "0" to disable, other to disable.
+ */
+void Configurable::enableOnSet(const QString &val)
+{
+    setEnabled( val != "0" );
+}
+
+/** \fn Configurable::enableOnUnset(const QString &val)
+ *  \brief This slot allows you to enable this configurable when a
+ *         binary configurable is set to false.
+ *  \param val signal value, should be "0" to enable, other to disable.
+ */
+void Configurable::enableOnUnset(const QString &val)
+{
+    setEnabled( val == "0" );
+}
+
+ConfigurationGroup::~ConfigurationGroup()
+{
+    childList::iterator it = children.begin();
+    while (it != children.end())
+    {
+        if (*it)
+            delete *it;
+        ++it;
+    }
 }
 
 Setting* ConfigurationGroup::byName(QString name) {
@@ -74,6 +109,13 @@ void ConfigurationGroup::save() {
         (*i)->save();
 }
 
+void ConfigurationGroup::save(QString destination) {
+    for(childList::iterator i = children.begin() ;
+        i != children.end() ;
+        ++i )
+        (*i)->save(destination);
+}
+
 QWidget* VerticalConfigurationGroup::configWidget(ConfigurationGroup *cg, 
                                                   QWidget* parent,
                                                   const char* widgetName) 
@@ -84,49 +126,32 @@ QWidget* VerticalConfigurationGroup::configWidget(ConfigurationGroup *cg,
     if (!useframe)
         widget->setFrameShape(QFrame::NoFrame);
 
-    QVBoxLayout *layout = NULL;
-
     float wmult = 0, hmult = 0;
-
     gContext->GetScreenSettings(wmult, hmult);
 
+    int space  = (zeroSpace) ? 4 : -1;
+    int margin = (int) ((uselabel) ? (28 * hmult) : (10 * hmult));
+    margin = (zeroMargin) ? 4 : margin;
+
+    QVBoxLayout *layout = new QVBoxLayout(widget, margin, space);
+
     if (uselabel)
-    {
-        int space = -1;
-        int margin = (int)(28 * hmult);
-        if (zeroSpace)
-            space = 4;
-        if (zeroMargin)
-            margin = 4;
-
-        layout = new QVBoxLayout(widget, margin, space);
-        // This makes weird and bad things happen in qt -mdz 2002/12/28
-        //widget->setInsideMargin(20);
         widget->setTitle(getLabel());
-    }
-    else
-    {
-        int space = -1;
-        int margin = (int)(10 * hmult);
-        if (zeroSpace)
-            space = 4;
-        if (zeroMargin)
-            margin = 4;
-        layout = new QVBoxLayout(widget, margin, space);
-    }
 
-    for(unsigned i = 0 ; i < children.size() ; ++i)
+    for (uint i = 0; i < children.size(); i++)
+    {
         if (children[i]->isVisible())
         {
             QWidget *child = children[i]->configWidget(cg, widget, NULL);
             layout->add(child);
             children[i]->setEnabled(children[i]->isEnabled());
         }
+    }
       
     if (cg)
     {
-        connect(this, SIGNAL(changeHelpText(QString)), cg,
-                SIGNAL(changeHelpText(QString)));
+        connect(this, SIGNAL(changeHelpText(QString)),
+                cg,   SIGNAL(changeHelpText(QString)));
     } 
 
     return widget;
@@ -315,6 +340,13 @@ void StackedConfigurationGroup::save() {
         children[top]->save();
 }
 
+void StackedConfigurationGroup::save(QString destination) {
+    if (saveAll)
+        ConfigurationGroup::save(destination);
+    else if (top < children.size())
+        children[top]->save(destination);
+}
+
 void TriggeredConfigurationGroup::setTrigger(Configurable* _trigger) {
     trigger = _trigger;
     // Make sure the stack is after the trigger
@@ -390,6 +422,18 @@ void SelectSetting::setValue(const QString& newValue)  {
         addSelection(newValue, newValue, true);
 }
 
+void SelectSetting::setValue(int which)
+{
+    if ((unsigned)which > values.size()-1 || which < 0) {
+        VERBOSE(VB_IMPORTANT, 
+                "SelectSetting::setValue(): invalid index " << which);
+        return;
+    }
+    current = which;
+    isSet = true;
+    Setting::setValue(values[current]);
+}
+
 QWidget* LabelSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
                                     const char* widgetName) {
     (void)cg;
@@ -450,7 +494,8 @@ QWidget* LineEditSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
         connect(edit, SIGNAL(changeHelpText(QString)), cg, 
                 SIGNAL(changeHelpText(QString)));
 
-    edit->setRW(rw);
+    setRW(rw);
+    SetPasswordEcho(password_echo);
 
     return widget;
 }
@@ -475,6 +520,13 @@ void LineEditSetting::setVisible(bool b)
     }
 }
 
+void LineEditSetting::SetPasswordEcho(bool b)
+{
+    password_echo = b;
+    if (edit)
+        edit->setEchoMode(b ? QLineEdit::Password : QLineEdit::Normal);
+}
+
 QWidget* SliderSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
                                      const char* widgetName) {
     QHBox* widget;
@@ -553,6 +605,8 @@ QWidget* SpinBoxSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
     if (1 < step)
         spinbox->setLineStep(step);
     spinbox->setValue(intValue());
+    if (!svtext.isEmpty())
+        spinbox->setSpecialValueText(svtext);
 
     connect(spinbox, SIGNAL(valueChanged(int)), this, SLOT(setValue(int)));
     connect(this, SIGNAL(valueChanged(int)), spinbox, SLOT(setValue(int)));
@@ -673,6 +727,26 @@ void ComboBoxSetting::setVisible(bool b)
     }
 }
 
+void ComboBoxSetting::setValue(QString newValue)
+{
+    if (!rw)
+    {
+        VERBOSE(VB_IMPORTANT, "ComboBoxSetting::setValue(QString): "
+                "BUG: attempted to set value of read-only ComboBox");
+        return;
+    }
+    Setting::setValue(newValue);
+    if (widget)
+        widget->setCurrentItem(current);
+};
+
+void ComboBoxSetting::setValue(int which)
+{
+    if (widget)
+        widget->setCurrentItem(which);
+    SelectSetting::setValue(which);
+};
+
 void HostRefreshRateComboBox::ChangeResolution(const QString& resolution)
 {
     clearSelections();
@@ -893,13 +967,78 @@ MythDialog* ConfigurationWizard::dialogWidget(MythMainWindow *parent,
     return wizard;
 }
 
+JumpPane::JumpPane(const QStringList &labels, const QStringList &helptext) :
+    ConfigurationGroup(true, false, true, true),
+    VerticalConfigurationGroup(true, false, true, true)
+{
+    //setLabel(tr("Jump To Buttons"));
+    for (uint i = 0; i < labels.size(); i++)
+    {
+        TransButtonSetting *button =
+            new TransButtonSetting(QString::number(i));
+        button->setLabel(labels[i]);
+        button->setHelpText(helptext[i]);
+        connect(button, SIGNAL(pressed(QString)),
+                this,   SIGNAL(pressed(QString)));
+        addChild(button);
+    }
+}
+
+MythDialog *JumpConfigurationWizard::dialogWidget(MythMainWindow *parent,
+                                                  const char *widgetName)
+{
+    MythJumpWizard *wizard = new MythJumpWizard(parent, widgetName);
+    dialog = wizard;
+
+    connect(this,   SIGNAL(changeHelpText(QString)),
+            wizard, SLOT(  setHelpText(   QString)));
+
+    childWidgets.clear();
+    QStringList labels, helptext;
+    for (uint i = 0; i < children.size(); i++)
+    {
+        if (children[i]->isVisible())
+        {
+            childWidgets.push_back(children[i]->configWidget(this, parent));
+            labels.push_back(children[i]->getLabel());
+            helptext.push_back(children[i]->getHelpText());
+        }
+    }
+
+    JumpPane *jumppane = new JumpPane(labels, helptext);
+    QWidget  *widget   = jumppane->configWidget(this, parent);
+    wizard->addPage(widget, "");
+    wizard->setFinishEnabled(widget, true);
+    connect(jumppane, SIGNAL(pressed( QString)),
+            this,     SLOT(  showPage(QString)));
+
+    for (uint i = 0; i < childWidgets.size(); i++)
+    {
+        wizard->addPage(childWidgets[i], labels[i]);
+        wizard->setFinishEnabled(childWidgets[i], true);
+    }
+
+    return wizard;
+}
+
+void JumpConfigurationWizard::showPage(QString page)
+{
+    uint pagenum = page.toUInt();
+    if (pagenum >= childWidgets.size() || !dialog)
+        return;
+    ((MythJumpWizard*)(dialog))->showPage(childWidgets[pagenum]);
+}
+
 void SimpleDBStorage::load() 
 {
-    QString querystr = QString("SELECT %1 FROM %2 WHERE %3;")
-        .arg(column).arg(table).arg(whereClause());
+    MSqlBindings bindings;
+    QString querystr = QString("SELECT " + column + " FROM "
+           + table + " WHERE " + whereClause(bindings) + ";");
     
     MSqlQuery query(MSqlQuery::InitCon());
-    query.exec(querystr);
+    query.prepare(querystr);
+    query.bindValues(bindings);
+    query.exec();
 
     if (query.isActive() && query.size() > 0) 
     {
@@ -914,42 +1053,61 @@ void SimpleDBStorage::load()
     }
 }
 
-void SimpleDBStorage::save() 
+void SimpleDBStorage::save(QString table) 
 {
     if (!isChanged())
         return;
 
-    QString querystr = QString("SELECT * FROM %1 WHERE %2;")
-        .arg(table).arg(whereClause());
+    MSqlBindings bindings;
+    QString querystr = QString("SELECT * FROM " + table + " WHERE "
+            + whereClause(bindings) + ";");
 
     MSqlQuery query(MSqlQuery::InitCon());
-    query.exec(querystr);
+    query.prepare(querystr);
+    query.bindValues(bindings);
+    query.exec();
 
     if (query.isActive() && query.size() > 0) {
         // Row already exists
         // Don"t change this QString. See the CVS logs rev 1.91.
-        querystr = QString("UPDATE " + table + " SET " + setClause() + 
-                           " WHERE " + whereClause() + ";");
-        // cerr << querystr << endl;
-        query.exec(querystr);
+        MSqlBindings bindings;
+
+        querystr = QString("UPDATE " + table + " SET " + setClause(bindings) + 
+                           " WHERE " + whereClause(bindings) + ";");
+
+        query.prepare(querystr);
+        query.bindValues(bindings);
+        query.exec();
+
         if (!query.isActive())
             MythContext::DBError("simpledbstorage update", querystr);
     } else {
         // Row does not exist yet
-        querystr = QString("INSERT INTO %1 SET %2;")
-            .arg(table).arg(setClause());
-        query.exec(querystr);
+        MSqlBindings bindings;
+
+        querystr = QString("INSERT INTO " + table + " SET "
+                + setClause(bindings) + ";");
+
+        query.prepare(querystr);
+        query.bindValues(bindings);
+        query.exec();
+
         if (!query.isActive())
             MythContext::DBError("simpledbstorage update", querystr);
     }
 }
 
-void AutoIncrementStorage::save() {
+void SimpleDBStorage::save() 
+{
+    save(table);
+}
+
+void AutoIncrementStorage::save(QString table) {
     if (intValue() == 0) 
     {
         // Generate a new, unique ID
-        QString querystr = QString("INSERT INTO %1 (%2) VALUES (0);")
-                                .arg(table).arg(column);
+        QString querystr = QString("INSERT INTO " + table + " (" 
+                + column + ") VALUES (0);");
 
         MSqlQuery query(MSqlQuery::InitCon());
         query.exec(querystr);
@@ -959,18 +1117,15 @@ void AutoIncrementStorage::save() {
             MythContext::DBError("inserting row", query);
             return;
         }
-        query.exec("SELECT LAST_INSERT_ID();");
-        if (!query.isActive() || query.size() < 1) 
-        {
-            MythContext::DBError("selecting last insert id", query);
-            return;
-        }
-
-        query.next();
-        setValue(query.value(0).toInt());
+        setValue(query.lastInsertId().toInt());
     }
 }
 
+void AutoIncrementStorage::save() 
+{
+    save(table);
+}
+
 void ListBoxSetting::setEnabled(bool b)
 {
     Configurable::setEnabled(b);
@@ -1008,10 +1163,14 @@ QWidget* ListBoxSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
             this, SIGNAL(accepted(int)));
     connect(widget, SIGNAL(menuButtonPressed(int)),
             this, SIGNAL(menuButtonPressed(int)));
+    connect(widget, SIGNAL(editButtonPressed(int)),
+            this, SIGNAL(editButtonPressed(int)));
+    connect(widget, SIGNAL(deleteButtonPressed(int)),
+            this, SIGNAL(deleteButtonPressed(int)));
     connect(this, SIGNAL(valueChanged(const QString&)),
             widget, SLOT(setCurrentItem(const QString&)));
-    connect(widget, SIGNAL(highlighted(const QString&)),
-            this, SLOT(setValueByLabel(const QString&)));
+    connect(widget, SIGNAL(highlighted(int)),
+            this, SLOT(setValueByIndex(int)));
 
     if (cg)
         connect(widget, SIGNAL(changeHelpText(QString)), cg,
@@ -1030,13 +1189,10 @@ void ListBoxSetting::setSelectionMode(MythListBox::SelectionMode mode)
        widget->setSelectionMode(selectionMode);
 }
 
-void ListBoxSetting::setValueByLabel(const QString& label) {
-    for(unsigned i = 0 ; i < labels.size() ; ++i)
-        if (labels[i] == label) {
-            setValue(values[i]);
-            return;
-        }
-    cerr << "BUG: ListBoxSetting::setValueByLabel called for unknown label " << label << endl;
+void ListBoxSetting::setValueByIndex(int index)
+{
+    if (((uint)index) < values.size())
+        setValue(values[index]);
 }
 
 void ImageSelectSetting::addImageSelection(const QString& label,
@@ -1114,8 +1270,8 @@ QWidget* ImageSelectSetting::configWidget(ConfigurationGroup *cg,
         widget->setCurrentItem(current);
     else
         current = 0;
-   
-    if (images.size() != 0 && images[current])
+
+    if (images.size() != 0 && current < images.size() && images[current])
     { 
         QImage temp = *(images[current]);
         temp = temp.smoothScale((int)(184 * m_hmult), (int)(138 * m_hmult), 
@@ -1175,6 +1331,7 @@ QWidget* ButtonSetting::configWidget(ConfigurationGroup* cg, QWidget* parent,
     //button->setHelpText(getHelpText());
 
     connect(button, SIGNAL(pressed()), this, SIGNAL(pressed()));
+    connect(button, SIGNAL(pressed()), this, SLOT(SendPressedString()));
 
     if (cg)
         connect(button, SIGNAL(changeHelpText(QString)),
@@ -1183,6 +1340,11 @@ QWidget* ButtonSetting::configWidget(ConfigurationGroup* cg, QWidget* parent,
     return button;
 }
 
+void ButtonSetting::SendPressedString(void)
+{
+    emit pressed(name);
+}
+
 void ButtonSetting::setEnabled(bool fEnabled)
 {
     Configurable::setEnabled(fEnabled);
@@ -1266,26 +1428,67 @@ int ConfigurationPopupDialog::exec(bool saveOnAccept)
     return ret;
 }
 
-QString HostSetting::whereClause(void)
+QString SimpleDBStorage::setClause(MSqlBindings& bindings)
 {
-    return QString("value = '%1' AND hostname = '%2'")
-                   .arg(getName()).arg(gContext->GetHostName());
+    QString tagname(":SET" + column.upper());
+    QString clause(column + " = " + tagname);
+
+    bindings.insert(tagname, getValue().utf8());
+
+    return clause;
 }
 
-QString HostSetting::setClause(void)
+QString HostSetting::whereClause(MSqlBindings& bindings)
 {
-    return QString("value = '%1', data = '%2', hostname = '%3'")
-                   .arg(getName()).arg(getValue().utf8())
-                   .arg(gContext->GetHostName());
+    /* Returns a where clause of the form:
+     * "value = :VALUE AND hostname = :HOSTNAME"
+     * The necessary bindings are added to the MSQLBindings&
+     */
+    QString valueTag(":WHEREVALUE");
+    QString hostnameTag(":WHEREHOSTNAME");
+
+    QString clause("value = " + valueTag + " AND hostname = " + hostnameTag);
+
+    bindings.insert(valueTag, getName());
+    bindings.insert(hostnameTag, gContext->GetHostName());
+
+    return clause;
 }
 
-QString GlobalSetting::whereClause(void)
+QString HostSetting::setClause(MSqlBindings& bindings)
 {
-    return QString("value = '%1'").arg(getName().utf8());
+    QString valueTag(":SETVALUE");
+    QString dataTag(":SETDATA");
+    QString hostnameTag(":SETHOSTNAME");
+    QString clause("value = " + valueTag + ", data = " + dataTag
+            + ", hostname = " + hostnameTag);
+
+    bindings.insert(valueTag, getName());
+    bindings.insert(dataTag, getValue().utf8());
+    bindings.insert(hostnameTag, gContext->GetHostName());
+
+    return clause;
 }
 
-QString GlobalSetting::setClause(void)
+QString GlobalSetting::whereClause(MSqlBindings& bindings)
 {
-    return QString("value = '%1', data = '%2'")
-                   .arg(getName()).arg(getValue());
+    QString valueTag(":WHEREVALUE");
+    QString clause("value = " + valueTag);
+
+    bindings.insert(valueTag, getName());
+
+    return clause;
+}
+
+QString GlobalSetting::setClause(MSqlBindings& bindings)
+{
+    QString valueTag(":SETVALUE");
+    QString dataTag(":SETDATA");
+
+    QString clause("value = " + valueTag + ", data = " + dataTag);
+
+    bindings.insert(valueTag, getName());
+    bindings.insert(dataTag, getValue().utf8());
+
+    return clause;
 }
