@@ -9,10 +9,11 @@
 
 #include "mythwidgets.h"
 #include "mythdialogs.h"
+#include "mythdbcon.h"
 
 using namespace std;
 
-class QSqlDatabase;
+//class QSqlDatabase;
 class QWidget;
 class ConfigurationGroup;
 class QDir;
@@ -28,8 +29,9 @@ public:
     virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent,
                                   const char* widgetName = 0);
 
-    virtual void load(QSqlDatabase* db) = 0;
-    virtual void save(QSqlDatabase* db) = 0;
+    virtual void load() = 0;
+    virtual void save() = 0;
+    virtual void save(QString /*destination*/) { }
 
     // A name for looking up the setting
     void setName(QString str) {
@@ -54,6 +56,10 @@ public:
     virtual void setEnabled(bool b) { enabled = b; }
     bool isEnabled() { return enabled; }
 
+public slots:
+    virtual void enableOnSet(const QString &val);
+    virtual void enableOnUnset(const QString &val);
+
 protected:
     bool labelAboveWidget; 
     bool enabled;
@@ -103,8 +109,12 @@ class ConfigurationGroup: virtual public Configurable
 {
     Q_OBJECT
   public:
-    ConfigurationGroup(bool luselabel = true, bool luseframe = true) 
-             { uselabel = luselabel; useframe = luseframe; }
+    ConfigurationGroup(bool luselabel = true, bool luseframe = true,
+                       bool lzeroMargin = false, bool lzeroSpace = false) :
+        uselabel(luselabel),     useframe(luseframe),
+        zeroMargin(lzeroMargin), zeroSpace(lzeroSpace)
+    {
+    }
     virtual ~ConfigurationGroup();
 
 
@@ -114,13 +124,21 @@ class ConfigurationGroup: virtual public Configurable
 
     virtual Setting* byName(QString name);
 
-    virtual void load(QSqlDatabase* db);
+    virtual void load();
 
-    virtual void save(QSqlDatabase* db);
+    virtual void save();
+    virtual void save(QString destination);
 
     void setUseLabel(bool useit) { uselabel = useit; }
     void setUseFrame(bool useit) { useframe = useit; }
 
+    void setOptions(bool luselabel = true, bool luseframe = true,
+                    bool lzeroMargin = false, bool lzeroSpace = false)
+    {
+        uselabel = luselabel; useframe = luseframe;
+        zeroMargin = lzeroMargin; zeroSpace = lzeroSpace;
+    }
+
   signals:
     void changeHelpText(QString);
     
@@ -129,37 +147,69 @@ class ConfigurationGroup: virtual public Configurable
     childList children;
     bool uselabel;
     bool useframe;
+    bool zeroMargin;
+    bool zeroSpace;
 };
 
-class VerticalConfigurationGroup: virtual public ConfigurationGroup {
+class VerticalConfigurationGroup : virtual public ConfigurationGroup
+{
+  public:
+    VerticalConfigurationGroup(
+        bool luselabel   = true,  bool luseframe  = true,
+        bool lzeroMargin = false, bool lzeroSpace = false) :
+        ConfigurationGroup(luselabel, luseframe, lzeroMargin, lzeroSpace)
+    {
+    }
+
+    virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent,
+                                  const char* widgetName = NULL);
+};
+
+class HorizontalConfigurationGroup : virtual public ConfigurationGroup
+{
+  public:
+    HorizontalConfigurationGroup(
+        bool luselabel   = true,  bool luseframe  = true,
+        bool lzeroMargin = false, bool lzeroSpace = false) :
+        ConfigurationGroup(luselabel, luseframe, lzeroMargin, lzeroSpace)
+    {
+    }
+
+    virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent,
+                                  const char* widgetName = NULL);
+};
+
+class GridConfigurationGroup: virtual public ConfigurationGroup {
  public:
-    VerticalConfigurationGroup(bool uselabel = true, bool useframe = true) 
-                : ConfigurationGroup(uselabel, useframe) { }
-
-    virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent,
-                                  const char* widgetName = 0);
-};
-
-class HorizontalConfigurationGroup: virtual public ConfigurationGroup {
- public:
-    HorizontalConfigurationGroup(bool uselabel = true, bool useframe = true) 
-                : ConfigurationGroup(uselabel, useframe) { }
+    GridConfigurationGroup(uint col,
+                           bool uselabel   = true,  bool useframe  = true,
+                           bool zeroMargin = false, bool zeroSpace = false) :
+        ConfigurationGroup(uselabel, useframe, zeroMargin, zeroSpace), 
+        columns(col)
+    {
+    }
 
     virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent,
                                   const char* widgetName = 0);
+ private:
+    uint columns;
 };
 
 class StackedConfigurationGroup: virtual public ConfigurationGroup {
     Q_OBJECT
 public:
-    StackedConfigurationGroup(bool uselabel = true)
-                : ConfigurationGroup(uselabel) { top = 0; saveAll = true; };
+    StackedConfigurationGroup(bool uselabel = true) :
+        ConfigurationGroup(uselabel, true, false, false),
+        top(0), saveAll(true)
+    {
+    }
 
     virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent,
                                   const char* widgetName = 0);
 
     void raise(Configurable* child);
-    virtual void save(QSqlDatabase* db);
+    virtual void save();
+    virtual void save(QString destination);
 
     // save all children, or only the top?
     void setSaveAll(bool b) { saveAll = b; };
@@ -188,23 +238,31 @@ signals:
 
 class ConfigurationDialog: virtual public Configurable {
 public:
+    ConfigurationDialog() : 
+        Configurable(), dialog(NULL) {};
+
     // Make a modal dialog containing configWidget
     virtual MythDialog* dialogWidget(MythMainWindow *parent,
                                      const char* widgetName = 0);
 
     // Show a dialogWidget, and save if accepted
-    int exec(QSqlDatabase* db, bool saveOnExec = true, bool doLoad = true);
+    int exec(bool saveOnExec = true, bool doLoad = true);
 
 protected:
     MythDialog *dialog;
 };
 
-// A wizard is a group with one child per page
+/** \class ConfigurationWizard
+ *  \brief A wizard is a group with one child per page.
+ */
 class ConfigurationWizard: public ConfigurationDialog,
-                           public ConfigurationGroup {
-public:
+                           public ConfigurationGroup
+{
+  public:
+    ConfigurationWizard() : ConfigurationGroup(true, true, false, false) {}
+
     virtual MythDialog* dialogWidget(MythMainWindow *parent,
-                                     const char* widgetName=0);
+                                     const char *widgetName = NULL);
 };
 
 // Read-only display of a setting
@@ -218,20 +276,29 @@ public:
 
 class LineEditSetting: virtual public Setting {
 protected:
-    LineEditSetting(bool readwrite = true) : edit(NULL) { rw = readwrite; };
+    LineEditSetting(bool readwrite = true) :
+        edit(NULL), rw(readwrite),  password_echo(false) { }
 public:
     virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent, 
                                   const char* widgetName = 0);
 
-    void setRW(bool readwrite = true) { rw = readwrite; edit->setRW(rw); };
-    void setRO() { rw = false; edit->setRO(); };
+    void setRW(bool readwrite = true)
+    {
+        rw = readwrite;
+        if (edit)
+            edit->setRW(rw);
+    }
+
+    void setRO(void) { setRW(false); }
 
     virtual void setEnabled(bool b);
     virtual void setVisible(bool b);
+    virtual void SetPasswordEcho(bool b);
 
 private:
     MythLineEdit* edit;
     bool rw;
+    bool password_echo;
 };
 
 // TODO: set things up so that setting the value as a string emits
@@ -275,11 +342,15 @@ public:
 
 class SpinBoxSetting: public BoundedIntegerSetting {
 protected:
-    SpinBoxSetting(int min, int max, int step, bool allow_single_step = false):
+    SpinBoxSetting(int min, int max, int step, 
+                   bool allow_single_step = false,
+                   QString special_value_text = ""):
         BoundedIntegerSetting(min, max, step),
-	sstep(allow_single_step) {};
+	sstep(allow_single_step),
+        svtext(special_value_text) {};
 
     bool sstep;
+    QString svtext;
     
 public:
     virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent, 
@@ -306,14 +377,7 @@ signals:
 public slots:
 
     virtual void setValue(const QString& newValue);
-
-    virtual void setValue(int which) {
-        if ((unsigned)which > values.size()-1) {
-            cout << "SelectSetting::setValue(): invalid index " << which << endl;
-            return;
-        }
-        setValue(values[which]);
-    };
+    virtual void setValue(int which);
 
     virtual QString getSelectionLabel(void) const {
         if (!isSet)
@@ -356,21 +420,15 @@ class ComboBoxSetting: public SelectSetting {
     Q_OBJECT;
 
 protected:
-    ComboBoxSetting(bool _rw = false) {
+    ComboBoxSetting(bool _rw = false, int _step = 1) {
         rw = _rw;
+        step = _step;
         widget = NULL;
     };
-public:
-    virtual void setValue(QString newValue) {
-        if (!rw)
-            cout << "BUG: attempted to set value of read-only ComboBox as string\n";
-        else
-            Setting::setValue(newValue);
-    };
 
-    virtual void setValue(int which) {
-        SelectSetting::setValue(which);
-    };
+public:
+    virtual void setValue(QString newValue);
+    virtual void setValue(int which);
 
     virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent, 
                                   const char* widgetName = 0);
@@ -394,6 +452,9 @@ protected slots:
 private:
     bool rw;
     MythComboBox *widget;
+
+protected:
+    int step;
 };
 
 class ListBoxSetting: public SelectSetting {
@@ -413,9 +474,11 @@ public:
 signals:
     void accepted(int);
     void menuButtonPressed(int);
+    void editButtonPressed(int);
+    void deleteButtonPressed(int);
 
 protected slots:
-    void setValueByLabel(const QString& label);
+    void setValueByIndex(int index);
     void addSelection(const QString& label,
                       QString value=QString::null,
                       bool select=false) {
@@ -487,11 +550,11 @@ protected:
 class TriggeredConfigurationGroup: virtual public ConfigurationGroup {
     Q_OBJECT
 public:
-    TriggeredConfigurationGroup(bool uselabel = true) 
-           : ConfigurationGroup(uselabel)
+    TriggeredConfigurationGroup(bool uselabel = true) :
+        ConfigurationGroup(uselabel, true, false, false),
+        configStack(NULL), trigger(NULL)
     {
-        trigger = configStack = NULL;
-    };
+    }
 
     void setTrigger(Configurable* _trigger);
 
@@ -509,9 +572,16 @@ protected:
     map<QString,Configurable*> triggerMap;
 };
     
-class TabbedConfigurationGroup: virtual public ConfigurationGroup {
+class TabbedConfigurationGroup: virtual public ConfigurationGroup
+{
     Q_OBJECT
-public:
+
+  public:
+    TabbedConfigurationGroup() :
+        ConfigurationGroup(true, true, false, false)
+    {
+    }
+
     virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent,
                                   const char* widgetName = 0);
 };
@@ -545,9 +615,9 @@ public:
         setLabel("Channel");
     };
 
-    static void fillSelections(QSqlDatabase* db, SelectSetting* setting);
-    virtual void fillSelections(QSqlDatabase* db) {
-        fillSelections(db, this);
+    static void fillSelections(SelectSetting* setting);
+    virtual void fillSelections() {
+        fillSelections(this);
     };
 };
 
@@ -582,8 +652,9 @@ public:
     DBStorage(QString _table, QString _column):
         table(_table), column(_column) {};
 
-    virtual void load(QSqlDatabase* db) = 0;
-    virtual void save(QSqlDatabase* db) = 0;
+    virtual void load() = 0;
+    virtual void save() = 0;
+    virtual void save(QString /*destination*/) { }
 
 protected:
     QString getColumn(void) const { return column; };
@@ -600,21 +671,21 @@ public:
 
     virtual ~SimpleDBStorage() {};
 
-    virtual void load(QSqlDatabase* db);
-    virtual void save(QSqlDatabase* db);
+    virtual void load();
+    virtual void save();
+    virtual void save(QString destination);
 
 protected:
 
-    virtual QString whereClause(void) = 0;
-    virtual QString setClause(void) {
-        return QString("%1 = '%2'").arg(column).arg(getValue());
-    };
+    virtual QString whereClause(MSqlBindings&) = 0;
+    virtual QString setClause(MSqlBindings& bindings);
 };
 
 class TransientStorage: virtual public Setting {
 public:
-    virtual void load(QSqlDatabase* db) { (void)db; }
-    virtual void save(QSqlDatabase* db) { (void)db; }
+    virtual void load() {  }
+    virtual void save() {  }
+    virtual void save(QString) {  }
 };
 
 class AutoIncrementStorage: virtual public IntegerSetting, public DBStorage {
@@ -624,14 +695,16 @@ public:
         setValue(0);
     };
 
-    virtual void load(QSqlDatabase* db) { (void)db; };
-    virtual void save(QSqlDatabase* db);
+    virtual void load() { };
+    virtual void save();
+    virtual void save(QString destination);
 };
 
 class ButtonSetting: virtual public Setting {
     Q_OBJECT
 public:
-    ButtonSetting() : button(NULL) {}
+    ButtonSetting(QString _name = "button") :
+        name(_name), button(NULL) {}
     virtual QWidget* configWidget(ConfigurationGroup* cg, QWidget* parent,
                                   const char* widgetName=0);
 
@@ -639,13 +712,14 @@ public:
 
 signals:
     void pressed();
-protected:
-    MythPushButton *button;
-};
+    void pressed(QString name);
 
-class TransButtonSetting: public ButtonSetting, public TransientStorage {
-public:
-    TransButtonSetting() {};
+protected slots:
+    void SendPressedString();
+
+protected:
+    QString name;
+    MythPushButton *button;
 };
 
 class ConfigPopupDialogWidget: public MythPopupBox {
@@ -667,7 +741,7 @@ public:
 
     virtual MythDialog* dialogWidget(MythMainWindow* parent,
                                      const char* widgetName=0);
-    int exec(QSqlDatabase* db, bool saveOnAccept=true);
+    int exec(bool saveOnAccept=true);
 
 public slots:
     void accept() { if (dialog) dialog->accept(); };
@@ -691,26 +765,17 @@ private:
     int totalSteps;
 };
 
-class TransLabelSetting: public LabelSetting, public TransientStorage {
-public:
-    TransLabelSetting() {};
-};
-
-class TransCheckBoxSetting: public CheckBoxSetting, public TransientStorage {
-public:
-    TransCheckBoxSetting() {};
-};
-
 class HostSetting: public SimpleDBStorage, virtual public Configurable {
 public:
     HostSetting(QString name):
         SimpleDBStorage("settings", "data") {
         setName(name);
     };
+    virtual ~HostSetting() { ; }
 
 protected:
-    virtual QString whereClause(void);
-    virtual QString setClause(void);
+    virtual QString whereClause(MSqlBindings& bindings);
+    virtual QString setClause(MSqlBindings& bindings);
 };
 
 class GlobalSetting: public SimpleDBStorage, virtual public Configurable {
@@ -721,10 +786,61 @@ public:
     };
 
 protected:
-    virtual QString whereClause(void);
-    virtual QString setClause(void);
+    virtual QString whereClause(MSqlBindings& bindings);
+    virtual QString setClause(MSqlBindings& bindings);
 };
 
+///////////////////////////////////////////////////////////////////////////////
+
+class TransButtonSetting :
+    public ButtonSetting, public TransientStorage
+{
+  public:
+    TransButtonSetting(QString name = "button") : ButtonSetting(name) { }
+};
+
+class TransLabelSetting :
+    public LabelSetting, public TransientStorage
+{
+  public:
+    TransLabelSetting() { }
+};
+
+class TransLineEditSetting :
+    public LineEditSetting, public TransientStorage
+{
+  public:
+    TransLineEditSetting(bool rw = true) : LineEditSetting(rw) { }
+};
+
+class TransCheckBoxSetting :
+    public CheckBoxSetting, public TransientStorage
+{
+  public:
+    TransCheckBoxSetting() { }
+};
+
+class TransComboBoxSetting :
+    public ComboBoxSetting, public TransientStorage
+{
+  public:
+    TransComboBoxSetting(bool rw = true, int _step = 1) :
+        ComboBoxSetting(rw, _step) { }
+};
+
+class TransSpinBoxSetting :
+    public SpinBoxSetting, public TransientStorage
+{
+  public:
+    TransSpinBoxSetting(int min, int max, int step,
+                        bool allow_single_step = false,
+                        QString special_value_text = "") :
+        SpinBoxSetting(min, max, step, allow_single_step,
+                       special_value_text) { }
+};
+
+///////////////////////////////////////////////////////////////////////////////
+
 class HostSlider: public SliderSetting, public HostSetting {
   public:
     HostSlider(const QString &name, int min, int max, int step) :
@@ -744,6 +860,7 @@ class HostCheckBox: public CheckBoxSetting, public HostSetting {
   public:
     HostCheckBox(const QString &name) :
         HostSetting(name) { }
+    virtual ~HostCheckBox() { ; }
 };
 
 class HostComboBox: public ComboBoxSetting, public HostSetting {
@@ -751,6 +868,42 @@ class HostComboBox: public ComboBoxSetting, public HostSetting {
     HostComboBox(const QString &name, bool rw = false) :
         ComboBoxSetting(rw),
         HostSetting(name) { }
+    virtual ~HostComboBox() { ; }
+};
+
+class HostRefreshRateComboBox: virtual public HostComboBox
+{
+    Q_OBJECT
+  public:
+    HostRefreshRateComboBox(const QString &name, bool rw = false) :
+        HostComboBox(name, rw) { ; }
+    virtual ~HostRefreshRateComboBox() { ; }
+  public slots:
+    virtual void ChangeResolution(const QString& resolution);
+  private:
+    static const vector<short> GetRefreshRates(const QString &resolution);
+};
+
+class HostTimeBox: public ComboBoxSetting, public HostSetting {
+  public:
+    HostTimeBox(const QString &name, const QString &defaultTime = "00:00",
+                const int interval = 1) :
+        ComboBoxSetting(false, 30 / interval),
+        HostSetting(name)
+    {
+        int hour;
+        int minute;
+        QString timeStr;
+
+        for (hour = 0; hour < 24; hour++)
+        {
+            for (minute = 0; minute < 60; minute += interval)
+            {
+                timeStr = timeStr.sprintf("%02d:%02d", hour, minute);
+                addSelection(timeStr, timeStr, timeStr == defaultTime);
+            }
+        }
+    }
 };
 
 class HostLineEdit: public LineEditSetting, public HostSetting {
@@ -807,5 +960,54 @@ class GlobalImageSelect: public ImageSelectSetting, public GlobalSetting {
         GlobalSetting(name) { }
 };
 
+class GlobalTimeBox: public ComboBoxSetting, public GlobalSetting {
+  public:
+    GlobalTimeBox(const QString &name, const QString &defaultTime = "00:00",
+                  const int interval = 1) :
+        ComboBoxSetting(false, 30 / interval),
+        GlobalSetting(name)
+    {
+        int hour;
+        int minute;
+        QString timeStr;
+
+        for (hour = 0; hour < 24; hour++)
+        {
+            for (minute = 0; minute < 60; minute += interval)
+            {
+                timeStr = timeStr.sprintf("%02d:%02d", hour, minute);
+                addSelection(timeStr, timeStr, timeStr == defaultTime);
+            }
+        }
+    }
+};
+
+/** \class JumpConfigurationWizard
+ *  \brief A jump wizard is a group with one child per page, and jump buttons
+ */
+class JumpConfigurationWizard : public ConfigurationWizard
+{
+    Q_OBJECT
+
+  public:
+    virtual MythDialog* dialogWidget(MythMainWindow *parent,
+                                     const char *widgetName = NULL);
+  protected slots:
+    void showPage(QString);
+
+  protected:
+    vector<QWidget*> childWidgets;
+};
+
+class JumpPane : public VerticalConfigurationGroup
+{
+    Q_OBJECT
+
+  public:
+    JumpPane(const QStringList &labels, const QStringList &helptext);
+
+  signals:
+    void pressed(QString);
+};
 
 #endif
