@@ -22,6 +22,7 @@
 #include <iostream>
 
 #include "mythcontext.h"
+#include "lcddevice.h"
 
 #include "uilistbtntype.h"
 
@@ -143,7 +144,8 @@ UIListTreeType::UIListTreeType(const QString &name, const QRect &area,
 
     m_active = NULL;
     m_inactive = NULL;
-
+    takes_focus = true;
+ 
     SetItemRegColor(Qt::black,QColor(80,80,80),100);
     SetItemSelColor(QColor(82,202,56),QColor(52,152,56),255);
 
@@ -275,6 +277,7 @@ void UIListTreeType::CreateLevel(int level)
                                       m_itemSelAlpha);
             newlevel->SetSpacing(m_spacing);
             newlevel->SetMargin(m_margin);
+            newlevel->SetParentListTree(this);
 
             listLevels.append(newlevel);
         }
@@ -286,12 +289,19 @@ UIListGenericTree *UIListTreeType::GetCurrentPosition(void)
     return currentpos;
 }
 
-void UIListTreeType::Draw(QPainter *p, int order, int blah)
+void UIListTreeType::Draw(QPainter *p, int order, int context)
 {
     if (hidden)
     {
         return;
     }
+    
+    if (m_context != -1 && m_context != context)
+        return;
+    
+    if (m_order != order)
+        return;
+
     QPtrListIterator<UIListBtnType> it(listLevels);
     UIListBtnType *child;
 
@@ -323,14 +333,17 @@ void UIListTreeType::Draw(QPainter *p, int order, int blah)
 
         if (child->GetArea().right() + offset > m_totalarea.left())
         {
-            child->Draw(p, order, blah, list_tree_active);
+            child->Draw(p, order, context, list_tree_active);
         }
         ++it;
     }
 }
 
-void UIListTreeType::DrawRegion(QPainter *p, QRect &area, int order, int blah)
+void UIListTreeType::DrawRegion(QPainter *p, QRect &area, int order, int context)
 {
+    if (m_context != -1 && m_context != context)
+        return;
+
     QPtrListIterator<UIListBtnType> it(listLevels);
     UIListBtnType *child;
 
@@ -362,7 +375,7 @@ void UIListTreeType::DrawRegion(QPainter *p, QRect &area, int order, int blah)
             drawRect == area)
         {
             child->SetDrawOffset(0 - child->GetArea().x());
-            child->Draw(p, order, blah, list_tree_active);
+            child->Draw(p, order, context, list_tree_active);
             child->SetDrawOffset(offset);
         }
 
@@ -724,6 +737,12 @@ bool UIListTreeType::tryToSetCurrent(QStringList route)
         else
         {
             keep_going = false;
+
+            //
+            //  Try to at least move to same level that the route was at
+            //
+            
+            MoveRight(false);
         }
         ++it;
     }
@@ -731,6 +750,30 @@ bool UIListTreeType::tryToSetCurrent(QStringList route)
     return keep_going;
 }
 
+bool UIListTreeType::incSearchStart(void)
+{
+    bool res = currentlevel->incSearchStart();
+
+    if (res)
+    {
+        SetCurrentPosition();
+        RedrawCurrent();
+    }
+    return (res);
+}
+
+bool UIListTreeType::incSearchNext(void)
+{
+    bool res = currentlevel->incSearchNext();
+
+    if (res)
+    {
+        SetCurrentPosition();
+        RedrawCurrent();
+    }
+    return (res);
+}
+
 int UIListTreeType::getDepth()
 {
     return curlevel;
@@ -791,19 +834,32 @@ void UIListTreeType::moveAwayFrom(UIListGenericTree *node)
 
 int UIListTreeType::getNumbItemsVisible()
 {
-    if(!currentlevel)
+    if (!currentlevel)
     {
         return 0;
     }
     return (int) currentlevel->GetNumbItemsVisible();
 }
 
+bool UIListTreeType::takeFocus()
+{
+    setActive(true);
+    return UIType::takeFocus();
+}
+
+void UIListTreeType::looseFocus()
+{
+    setActive(false);
+    UIType::looseFocus();
+}
+
 //////////////////////////////////////////////////////////////////////////////
 
 UIListBtnType::UIListBtnType(const QString& name, const QRect& area,
                              int order, bool showArrow, bool showScrollArrows)
              : UIType(name)
 {
+    m_parentListTree   = NULL;
     m_order            = order;
     m_rect             = area;
 
@@ -812,6 +868,7 @@ UIListBtnType::UIListBtnType(const QString& name, const QRect& area,
 
     m_active           = false;
     m_visible          = true;
+    takes_focus        = true;
     m_showUpArrow      = false;
     m_showDnArrow      = false;
 
@@ -823,6 +880,8 @@ UIListBtnType::UIListBtnType(const QString& name, const QRect& area,
     m_selPosition = 0;
     m_topPosition = 0;
     m_itemCount = 0;
+    m_incSearch = "";
+    m_bIncSearchContains = false;
 
     m_initialized     = false;
     m_clearing        = false;
@@ -1029,12 +1088,45 @@ void UIListBtnType::SetItemCurrent(UIListBtnTypeItem* item)
         m_selPosition = 0;
     }
 
-    m_itemCount = m_selPosition;
-    m_topItem = item;
     m_selItem = item;
+    m_topItem = m_selItem;
+    m_topPosition = m_selPosition;
     (*m_topIterator) = (*m_selIterator);
 
-    if (m_showScrollArrows && m_itemCount > (int)m_itemsVisible)
+    // centre the selected item in the list
+    int count = m_itemsVisible / 2;
+
+    while (count && m_topPosition > 0) 
+    {
+        --(*m_topIterator);
+        --m_topPosition;
+        --count;
+    }
+
+    // backup if we have scrolled past the end of the list
+    if (m_topPosition + (int)m_itemsVisible > m_itemCount)
+    {
+        while (m_topPosition > 0 && m_topPosition + (int)m_itemsVisible > m_itemCount) 
+        {
+            --(*m_topIterator);
+            --m_topPosition;
+        }
+    }
+
+    if (m_topIterator->current())
+        m_topItem = m_topIterator->current();
+    else
+    {
+        m_topItem = m_topIterator->toFirst();
+        m_topPosition = 0;
+    }
+
+    if (m_topItem != m_itemList.first())
+        m_showUpArrow = true;
+    else
+        m_showUpArrow = false;
+
+    if (m_topPosition + (int)m_itemsVisible < m_itemCount)
         m_showDnArrow = true;
     else
         m_showDnArrow = false;
@@ -1392,21 +1484,125 @@ bool UIListBtnType::MoveItemUpDown(UIListBtnTypeItem *item, bool flag)
     return true;
 }
 
+bool UIListBtnType::incSearchStart(void)
+{
+    MythPopupBox *popup = new MythPopupBox(gContext->GetMainWindow(),
+                                           "incserach_popup");
 
-void UIListBtnType::Draw(QPainter *p, int order, int blah)
+    QLabel *caption = popup->addLabel(tr("Search"), MythPopupBox::Large);
+    caption->setAlignment(Qt::AlignCenter);
+
+    MythComboBox *modeCombo = new MythComboBox(false, popup, "mode_combo" );
+    modeCombo->insertItem(tr("Starts with text"));
+    modeCombo->insertItem(tr("Contains text"));
+    popup->addWidget(modeCombo);
+
+    MythLineEdit *searchEdit = new MythLineEdit(false, popup, "mode_combo");
+    searchEdit->setText(m_incSearch);
+    popup->addWidget(searchEdit);
+    searchEdit->setFocus();
+
+    popup->addButton(tr("Search"));
+    popup->addButton(tr("Cancel"));
+
+    int res = popup->ExecPopup();
+
+    if (res == 0)
+    {
+        m_incSearch = searchEdit->text();
+        m_bIncSearchContains = (modeCombo->currentItem() == 1);
+        incSearchNext();
+    }
+
+    delete popup;
+
+    return (res == 0);
+}
+
+bool UIListBtnType::incSearchNext(void)
+{
+    if (!m_selItem)
+    {
+        return false;
+    }
+
+    //
+    //  Move the active node to the node whose string value
+    //  starts or contains the search text
+    //
+
+    QPtrListIterator<UIListBtnTypeItem> it = (*m_selIterator);
+    ++it;
+
+    while (it.current())
+    {
+        if (m_bIncSearchContains)
+        {
+            if (it.current()->text().find(m_incSearch, 0, false) != -1)
+                break;
+        }
+        else
+        {
+            if (it.current()->text().startsWith(m_incSearch, false))
+                break;
+        }
+
+        ++it;
+    }
+
+    // if it is NULL, we reached the end of the list. wrap to the
+    // beginning and try again
+    if (!it.current())
+    {
+        it.toFirst();
+
+        while (it.current())
+        {
+            // we're back at the current_node, which means there are no
+            // matching nodes
+            if (it.current() == m_selItem)
+            {
+                break;
+            }
+
+            if (m_bIncSearchContains)
+            {
+                if (it.current()->text().find(m_incSearch, 0, false) != -1)
+                    break;
+            }
+            else
+            {
+                if (it.current()->text().startsWith(m_incSearch, false))
+                    break;
+            }
+
+            ++it;
+        }
+    }
+
+    if (it.current())
+    {
+        SetItemCurrent(it.current());
+        return true;
+    }
+
+    return false;
+}
+
+void UIListBtnType::Draw(QPainter *p, int order, int context)
 {
     //
     //  Just call the other Draw() function. Tried to accomplish the same
     //  goal with default parameters, but that broke MythGallery (?)
     //
 
-    Draw(p, order, blah, true);
+    Draw(p, order, context, true);
 }
 
 
-void UIListBtnType::Draw(QPainter *p, int order, int, bool active_on)
+void UIListBtnType::Draw(QPainter *p, int order, int context, bool active_on)
 {
-    if (!m_visible)
+    if (!m_visible || hidden)
         return;
 
     if (!m_initialized)
@@ -1415,8 +1611,79 @@ void UIListBtnType::Draw(QPainter *p, int order, int, bool active_on)
     if (m_order != order)
         return;
 
+    if (m_context != -1 && m_context != context)
+        return;
+
+    //  Put something on the LCD device (if one exists)
+    if (class LCD *lcddev = LCD::Get())
+    {
+        if (m_active)
+        {
+            // add max of lcd height menu items either side of the selected item
+            // let the lcdserver figure out which ones to display
+
+            QPtrList<LCDMenuItem> menuItems;
+            menuItems.setAutoDelete(true);
+
+            QPtrListIterator<UIListBtnTypeItem> it = (*m_selIterator);
+            uint count = 0;
+
+            // move back up the list a little
+            while (it.current() && count < lcddev->getLCDHeight())
+            {
+                --it;
+                ++count;
+            }
+
+            if (!it.current())
+                it.toFirst();
+            count = 0;
+            while (it.current() && count < lcddev->getLCDHeight() * 2)
+            {
+                UIListBtnTypeItem *curItem = it.current();
+                QString msg = curItem->text();
+                bool selected;
+                CHECKED_STATE checkState = NOTCHECKABLE;
+                if (curItem->checkable())
+                {
+                    if (curItem->state() == UIListBtnTypeItem::HalfChecked || 
+                        curItem->state() == UIListBtnTypeItem::FullChecked)
+                        checkState = CHECKED;
+                    else
+                        checkState = UNCHECKED;
+                }
+
+                if (curItem == m_selItem)
+                    selected = true;
+                else
+                    selected = false;
+
+                menuItems.append(new LCDMenuItem(selected, checkState, msg));
+                ++it;
+                ++count;
+            }
+
+            QString title = "";
+
+            if (m_parentListTree && m_parentListTree->getDepth() > 0)
+                title = "<< ";
+            else
+                title = "   ";
+
+            if ((m_selItem && m_selItem->getDrawArrow()) || m_showArrow)
+                title += " >>";
+            else
+                title += "   ";
+
+            if (!menuItems.isEmpty())
+            {
+                lcddev->switchToMenu(&menuItems, title);
+            }
+        }
+    }
+
     fontProp* font = m_active ? m_fontActive : m_fontInactive;
-    
+
     if (!active_on)
     {
         font = m_fontInactive;
@@ -1633,18 +1900,32 @@ void UIListBtnType::LoadPixmap(QPixmap& pix, const QString& fileName)
     }
 }
 
+void UIListBtnType::calculateScreenArea()
+{
+    QRect r = m_rect; 
+    r.moveBy(m_parent->GetAreaRect().left(),
+             m_parent->GetAreaRect().top());
+    screen_area = r;
+}
+
+bool UIListBtnType::takeFocus()
+{
+    SetActive(true);
+    return UIType::takeFocus();
+}
+
+void UIListBtnType::looseFocus()
+{
+    SetActive(false);
+    UIType::looseFocus();
+}
+
 //////////////////////////////////////////////////////////////////////////////
 
 UIListBtnTypeItem::UIListBtnTypeItem(UIListBtnType* lbtype, const QString& text,
                                      QPixmap *pixmap, bool checkable,
                                      CheckState state, bool showArrow)
 {
-    if (!lbtype) {
-        std::cerr << "UIListBtnTypeItem: trying to creating item without parent"
-                  << std::endl;
-        exit(-45);
-    }
-    
     m_parent    = lbtype;
     m_text      = text;
     m_pixmap    = pixmap;
