@@ -450,7 +450,7 @@ bool DiSEqCDevTree::Execute(const DiSEqCDevSettings &settings,
     ApplyVoltage(settings, tuning);
 
     // turn off tone burst first if commands need to be sent
-    if (m_root->IsCommandNeeded(settings))
+    if (m_root->IsCommandNeeded(settings, tuning))
     {
         SetTone(false);
         usleep(DISEQC_SHORT_WAIT);
@@ -969,41 +969,40 @@ bool DiSEqCDevSwitch::Execute(const DiSEqCDevSettings &settings,
     if (pos < 0)
         return false;
 
-    // determine if switch command needs to be sent based on last pos
-    if ((m_last_pos == (uint)pos) && m_children[pos])
-        return m_children[pos]->Execute(settings, tuning);
-
     // perform switching
-    switch (m_type)
+    if (ShouldSwitch(settings, tuning))
     {
-        case kTypeTone:
-            success = ExecuteTone(settings, tuning, pos);
-            break;
-        case kTypeDiSEqCCommitted:
-        case kTypeDiSEqCUncommitted:
-            success = ExecuteDiseqc(settings, tuning, pos);
-            break;
-        case kTypeLegacySW21:
-        case kTypeLegacySW42:
-        case kTypeLegacySW64:
-            success = ExecuteLegacy(settings, tuning, pos);
-            break;
-        default:
-            success = false;
-            VERBOSE(VB_IMPORTANT, LOC_ERR +
-                    QString("Unknown switch type (%1)")
-                    .arg((uint)m_type));
-            break;
-    }
+        switch (m_type)
+        {
+            case kTypeTone:
+                success = ExecuteTone(settings, tuning, pos);
+                break;
+            case kTypeDiSEqCCommitted:
+            case kTypeDiSEqCUncommitted:
+                success = ExecuteDiseqc(settings, tuning, pos);
+                break;
+            case kTypeLegacySW21:
+            case kTypeLegacySW42:
+            case kTypeLegacySW64:
+                success = ExecuteLegacy(settings, tuning, pos);
+                break;
+            default:
+                success = false;
+                VERBOSE(VB_IMPORTANT, LOC_ERR +
+                        QString("Unknown switch type (%1)")
+                        .arg((uint)m_type));
+                break;
+        }
 
-    // if a child device will be sending a diseqc command, wait 100ms
-    if (m_children[pos]->IsCommandNeeded(settings))
-    {
-        VERBOSE(VB_CHANNEL, LOC + "Waiting for switch");
-        usleep(DISEQC_LONG_WAIT);
-    }
+        // if a child device will be sending a diseqc command, wait 100ms
+        if (m_children[pos]->IsCommandNeeded(settings, tuning))
+        {
+            VERBOSE(VB_CHANNEL, LOC + "Waiting for switch");
+            usleep(DISEQC_LONG_WAIT);
+        }
 
-    m_last_pos = pos;
+        m_last_pos = pos;
+    }
 
     // chain to child if the switch was successful
     if (success)
@@ -1015,6 +1014,8 @@ bool DiSEqCDevSwitch::Execute(const DiSEqCDevSettings &settings,
 void DiSEqCDevSwitch::Reset(void)
 {
     m_last_pos = (uint) -1;
+    m_last_high_band = (uint) -1;
+    m_last_horizontal = (uint) -1;
     dvbdev_vec_t::iterator it = m_children.begin();
     for (; it != m_children.end(); ++it)
     {
@@ -1023,19 +1024,15 @@ void DiSEqCDevSwitch::Reset(void)
     }
 }
 
-bool DiSEqCDevSwitch::IsCommandNeeded(const DiSEqCDevSettings &settings) const
+bool DiSEqCDevSwitch::IsCommandNeeded(const DiSEqCDevSettings &settings,
+                                      const DVBTuning         &tuning) const
 {
-    // sanity check switch position
     int pos = GetPosition(settings);
     if (pos < 0)
         return false;
 
-    // if position is changing, a command is definitely needed
-    if ((uint)pos != m_last_pos)
-        return true;
-
-    // otherwise, the child that will be selected may need a command
-    return m_children[pos]->IsCommandNeeded(settings);
+    return (ShouldSwitch(settings, tuning) ||
+            m_children[pos]->IsCommandNeeded(settings, tuning));
 }
 
 DiSEqCDevDevice *DiSEqCDevSwitch::GetSelectedChild(const DiSEqCDevSettings &settings) const
@@ -1106,7 +1103,7 @@ bool DiSEqCDevSwitch::Load(void)
     // populate switch parameters from db
     MSqlQuery query(MSqlQuery::InitCon());
     query.prepare(
-        "SELECT subtype, switch_ports "
+        "SELECT subtype, switch_ports, cmd_repeat "
         "FROM diseqc_tree "
         "WHERE diseqcid = :DEVID");
     query.bindValue(":DEVID", GetDeviceID());
@@ -1120,6 +1117,7 @@ bool DiSEqCDevSwitch::Load(void)
     {
         m_type = SwitchTypeFromString(query.value(0).toString());
         m_num_ports = query.value(1).toUInt();
+        m_repeat = query.value(2).toUInt();
         m_children.resize(m_num_ports);
         for (uint i = 0; i < m_num_ports; i++)
             m_children[i] = NULL;
@@ -1169,7 +1167,8 @@ bool DiSEqCDevSwitch::Store(void) const
             "    type         = 'switch', "
             "    description  = :DESC, "
             "    subtype      = :TYPE, "
-            "    switch_ports = :PORTS "
+            "    switch_ports = :PORTS, "
+            "    cmd_repeat   = :REPEAT "
             "WHERE diseqcid = :DEVID");
     }
     else
@@ -1177,10 +1176,12 @@ bool DiSEqCDevSwitch::Store(void) const
         query.prepare(
             "INSERT INTO diseqc_tree"
             " ( parentid,      ordinal,         type, "
-            "   description,   subtype,         switch_ports) "
+            "   description,   subtype,         switch_ports, "
+            "   cmd_repeat )"
             "VALUES "
             " (:PARENT,       :ORDINAL,         'switch', "
-            "  :DESC,         :TYPE,            :PORTS)");
+            "  :DESC,         :TYPE,            :PORTS, "
+            "  :REPEAT )");
     }
 
     if (m_parent)
@@ -1190,6 +1191,7 @@ bool DiSEqCDevSwitch::Store(void) const
     query.bindValue(":DESC",    GetDescription());
     query.bindValue(":TYPE",    type);
     query.bindValue(":PORTS",   m_num_ports);
+    query.bindValue(":REPEAT",  m_repeat);
     query.bindValue(":DEVID",   GetDeviceID());
 
     if (!query.exec())
@@ -1348,6 +1350,46 @@ bool DiSEqCDevSwitch::ExecuteTone(const DiSEqCDevSettings &/*settings*/,
     return false;
 }
 
+bool DiSEqCDevSwitch::ShouldSwitch(const DiSEqCDevSettings &settings,
+                                   const DVBTuning &tuning) const
+{
+    int pos = GetPosition(settings);
+    if (pos < 0)
+        return false;
+
+    // committed switch should change for band and polarity as well
+    if (kTypeDiSEqCCommitted == m_type)
+    {
+        // retrieve LNB info
+        bool high_band  = false;
+        bool horizontal = false;
+        DiSEqCDevLNB *lnb  = m_tree.FindLNB(settings);
+        if (lnb)
+        {
+            high_band   = lnb->IsHighBand(tuning);
+            horizontal  = lnb->IsHorizontal(tuning);
+        }
+
+        if(high_band != m_last_high_band ||
+           horizontal != m_last_horizontal)
+            return true;
+    }
+    else if (kTypeLegacySW42 == m_type ||
+             kTypeLegacySW64 == m_type)
+    {
+        // retrieve LNB info
+        bool horizontal = false;
+        DiSEqCDevLNB *lnb  = m_tree.FindLNB(settings);
+        if (lnb)
+            horizontal  = lnb->IsHorizontal(tuning);
+
+        if (horizontal != m_last_horizontal)
+            return true;
+    }
+
+    return m_last_pos != (uint)pos;
+}
+
 bool DiSEqCDevSwitch::ExecuteDiseqc(const DiSEqCDevSettings &settings,
                                     const DVBTuning &tuning,
                                     uint pos)
@@ -1385,7 +1427,13 @@ bool DiSEqCDevSwitch::ExecuteDiseqc(const DiSEqCDevSettings &settings,
     VERBOSE(VB_CHANNEL, LOC + "Changing to DiSEqC switch port " +
             QString("%1/%2").arg(pos + 1).arg(m_num_ports));
 
-    return m_tree.SendCommand(DISEQC_ADR_SW_ALL, cmd, m_repeat, 1, &data);
+    bool ret = m_tree.SendCommand(DISEQC_ADR_SW_ALL, cmd, m_repeat, 1, &data);
+    if(ret)
+    {
+        m_last_high_band = high_band;
+        m_last_horizontal = horizontal;
+    }
+    return ret;
 }
 
 int DiSEqCDevSwitch::GetPosition(const DiSEqCDevSettings &settings) const
@@ -1490,7 +1538,8 @@ void DiSEqCDevRotor::Reset(void)
         m_child->Reset();
 }
 
-bool DiSEqCDevRotor::IsCommandNeeded(const DiSEqCDevSettings &settings) const
+bool DiSEqCDevRotor::IsCommandNeeded(const DiSEqCDevSettings &settings,
+                                     const DVBTuning         &tuning) const
 {
     double position = settings.GetValue(GetDeviceID());
 
@@ -1498,7 +1547,7 @@ bool DiSEqCDevRotor::IsCommandNeeded(const DiSEqCDevSettings &settings) const
         return true;
 
     if (m_child)
-        return m_child->IsCommandNeeded(settings);
+        return m_child->IsCommandNeeded(settings, tuning);
 
     return false;
 }
@@ -1528,15 +1577,20 @@ bool DiSEqCDevRotor::SetChild(uint ordinal, DiSEqCDevDevice *device)
     return true;
 }
 
-uint DiSEqCDevRotor::GetVoltage(const DiSEqCDevSettings &settings,
-                                const DVBTuning         &tuning) const
+bool DiSEqCDevRotor::IsMoving(const DiSEqCDevSettings &settings) const
 {
     double position = settings.GetValue(GetDeviceID());
     double completed = GetProgress();
     bool   moving   = (completed < 1.0) || (position != m_last_position);
 
+    return (m_last_pos_known && moving);
+}
+
+uint DiSEqCDevRotor::GetVoltage(const DiSEqCDevSettings &settings,
+                                const DVBTuning         &tuning) const
+{
     // override voltage if the last position is known and the rotor is moving
-    if (m_last_pos_known && moving)
+    if (IsMoving(settings))
     {
         VERBOSE(VB_CHANNEL, LOC +
                 "Overriding voltage to 18V for faster rotor movement");
@@ -1555,7 +1609,8 @@ bool DiSEqCDevRotor::Load(void)
     MSqlQuery query(MSqlQuery::InitCon());
     query.prepare(
         "SELECT subtype,         rotor_positions, "
-        "       rotor_hi_speed,  rotor_lo_speed "
+        "       rotor_hi_speed,  rotor_lo_speed, "
+        "       cmd_repeat "
         "FROM diseqc_tree "
         "WHERE diseqcid = :DEVID");
     query.bindValue(":DEVID", GetDeviceID());
@@ -1570,6 +1625,7 @@ bool DiSEqCDevRotor::Load(void)
         m_type     = RotorTypeFromString(query.value(0).toString());
         m_speed_hi = query.value(2).toDouble();
         m_speed_lo = query.value(3).toDouble();
+        m_repeat   = query.value(4).toUInt();
 
         // form of "angle1=index1:angle2=index2:..."
         QString positions = query.value(1).toString();
@@ -1639,7 +1695,8 @@ bool DiSEqCDevRotor::Store(void) const
             "    subtype         = :TYPE,    "
             "    rotor_hi_speed  = :HISPEED, "
             "    rotor_lo_speed  = :LOSPEED, "
-            "    rotor_positions = :POSMAP   "
+            "    rotor_positions = :POSMAP,  "
+            "    cmd_repeat      = :REPEAT   "
             "WHERE diseqcid = :DEVID");
     }
     else
@@ -1648,11 +1705,11 @@ bool DiSEqCDevRotor::Store(void) const
             "INSERT INTO diseqc_tree "
             " ( parentid,       ordinal,         type,   "
             "   description,    subtype,         rotor_hi_speed, "
-            "   rotor_lo_speed, rotor_positions ) "
+            "   rotor_lo_speed, rotor_positions, cmd_repeat ) "
             "VALUES "
             " (:PARENT,         :ORDINAL,        'rotor',  "
             "  :DESC,           :TYPE,           :HISPEED, "
-            "  :LOSPEED,        :POSMAP )");
+            "  :LOSPEED,        :POSMAP,         :REPEAT )");
     }
 
     if (m_parent)
@@ -1664,6 +1721,7 @@ bool DiSEqCDevRotor::Store(void) const
     query.bindValue(":HISPEED", m_speed_hi);
     query.bindValue(":LOSPEED", m_speed_lo);
     query.bindValue(":POSMAP",  posmap);
+    query.bindValue(":REPEAT",  m_repeat);
     query.bindValue(":DEVID",   GetDeviceID());
 
     if (!query.exec())
@@ -1883,7 +1941,8 @@ bool DiSEqCDevLNB::Load(void)
     MSqlQuery query(MSqlQuery::InitCon());
     query.prepare(
         "SELECT subtype,         lnb_lof_switch, "
-        "       lnb_lof_hi,      lnb_lof_lo "
+        "       lnb_lof_hi,      lnb_lof_lo, "
+        "       cmd_repeat "
         "FROM diseqc_tree "
         "WHERE diseqcid = :DEVID");
     query.bindValue(":DEVID", GetDeviceID());
@@ -1899,6 +1958,7 @@ bool DiSEqCDevLNB::Load(void)
         m_lof_switch = query.value(1).toInt();
         m_lof_hi     = query.value(2).toInt();
         m_lof_lo     = query.value(3).toInt();
+        m_repeat     = query.value(4).toUInt();
     }
 
     return true;
@@ -1921,7 +1981,8 @@ bool DiSEqCDevLNB::Store(void) const
             "    subtype         = :TYPE,    "
             "    lnb_lof_switch  = :LOFSW,   "
             "    lnb_lof_lo      = :LOFLO,   "
-            "    lnb_lof_hi      = :LOFHI    "
+            "    lnb_lof_hi      = :LOFHI,   "
+            "    cmd_repeat      = :REPEAT   "
             "WHERE diseqcid = :DEVID");
     }
     else
@@ -1930,11 +1991,11 @@ bool DiSEqCDevLNB::Store(void) const
             "INSERT INTO diseqc_tree"
             " ( parentid,      ordinal,         type, "
             "   description,   subtype,         lnb_lof_switch, "
-            "   lnb_lof_lo,    lnb_lof_hi ) "
+            "   lnb_lof_lo,    lnb_lof_hi,      cmd_repeat ) "
             "VALUES "
             " (:PARENT,       :ORDINAL,         'lnb', "
             "  :DESC,         :TYPE,            :LOFSW, "
-            "  :LOFLO,        :LOFHI ) ");
+            "  :LOFLO,        :LOFHI,           :REPEAT ) ");
     }
 
     if (m_parent)
@@ -1946,6 +2007,7 @@ bool DiSEqCDevLNB::Store(void) const
     query.bindValue(":LOFSW",   m_lof_switch);
     query.bindValue(":LOFLO",   m_lof_lo);
     query.bindValue(":LOFHI",   m_lof_hi);
+    query.bindValue(":REPEAT",  m_repeat);
     query.bindValue(":DEVID",   GetDeviceID());
 
     // update dev_id
