@@ -19,28 +19,33 @@
 using namespace std;
 
 #include "mythcontext.h"
+#include "mythdbcon.h"
 #include "guidegrid.h"
 #include "infostructs.h"
 #include "programinfo.h"
 #include "oldsettings.h"
-#include "tv.h"
+#include "tv_play.h"
+#include "tv_rec.h"
 #include "progfind.h"
 #include "proglist.h"
+#include "customedit.h"
 #include "util.h"
 #include "remoteutil.h"
+#include "channelutil.h"
 
-QString RunProgramGuide(QString startchannel, bool thread, TV *player,
-                        bool allowsecondaryepg)
+bool RunProgramGuide(uint &chanid, QString &channum,
+                     bool thread, TV *player,
+                     bool allowsecondaryepg)
 {
-    QString chanstr;
-   
+    bool channel_changed = false;
+
     if (thread)
         qApp->lock();
 
-    if (startchannel == QString::null)
-        startchannel = "";
+    gContext->addCurrentLocation("GuideGrid");
 
-    GuideGrid *gg = new GuideGrid(gContext->GetMainWindow(), startchannel, 
+    GuideGrid *gg = new GuideGrid(gContext->GetMainWindow(),
+                                  chanid, channum,
                                   player, allowsecondaryepg, "guidegrid");
 
     gg->Show();
@@ -55,21 +60,30 @@ QString RunProgramGuide(QString startchannel, bool thread, TV *player,
     else
         gg->exec();
 
-    chanstr = gg->getLastChannel();
+    if (chanid != gg->GetChanID())
+    {
+        chanid  = gg->GetChanID();
+        channum = gg->GetChanNum();
+        channel_changed = true;
+    }
 
     if (thread)
         qApp->lock();
 
     delete gg;
 
+    gContext->removeCurrentLocation();
+
     if (thread)
         qApp->unlock();
 
-    return chanstr;
+    return channel_changed;
 }
 
-GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
-                     bool allowsecondaryepg, const char *name)
+GuideGrid::GuideGrid(MythMainWindow *parent,
+                     uint chanid, QString channum,
+                     TV *player, bool allowsecondaryepg,
+                     const char *name)
          : MythDialog(parent, name)
 {
     desiredDisplayChans = DISPLAY_CHANS = 6;
@@ -78,7 +92,6 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
     m_currentStartChannel = 0;
 
     m_player = player;
-    m_db = QSqlDatabase::database();
 
     m_context = 0;
 
@@ -92,8 +105,6 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
     curInfoRect = QRect(0, 0, 0, 0);
     videoRect = QRect(0, 0, 0, 0);
 
-    MythContext::KickDatabase(m_db);
-
     jumpToChannelEnabled = gContext->GetNumSetting("EPGEnableJumpToChannel", 0);
     jumpToChannelActive = false;
     jumpToChannelHasRect = false; // by default, we assume there is no specific region for jumpToChannel in the theme
@@ -169,7 +180,7 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
             container->SetDrawFontShadow(false);
     }
 
-    channelOrdering = gContext->GetSetting("ChannelOrdering", "channum + 0");
+    channelOrdering = gContext->GetSetting("ChannelOrdering", "channum");
     dateformat = gContext->GetSetting("ShortDateFormat", "ddd d");
     unknownTitle = gContext->GetSetting("UnknownTitle", "Unknown");
     unknownCategory = gContext->GetSetting("UnknownCategory", "Unknown");
@@ -214,31 +225,32 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
     int secsoffset = -((m_originalStartTime.time().minute() % 30) * 60 +
                         m_originalStartTime.time().second());
     m_currentStartTime = m_originalStartTime.addSecs(secsoffset);
-    m_startChanStr = channel;
+    startChanID  = chanid;
+    startChanNum = channum;
 
     m_currentRow = (int)(desiredDisplayChans / 2);
     m_currentCol = 0;
 
-    for (int y = 0; y < DISPLAY_CHANS; y++)
+    for (int y = 0; y < MAX_DISPLAY_CHANS; y++)
         m_programs[y] = NULL;
 
-    for (int x = 0; x < DISPLAY_TIMES; x++)
+    for (int x = 0; x < MAX_DISPLAY_TIMES; x++)
     {
         m_timeInfos[x] = NULL;
-        for (int y = 0; y < DISPLAY_CHANS; y++)
+        for (int y = 0; y < MAX_DISPLAY_CHANS; y++)
             m_programInfos[y][x] = NULL;
     }
 
-    //QTime clock = QTime::currentTime();
+    //MythTimer clock = QTime::currentTime();
     //clock.start();
     fillTimeInfos();
     //int filltime = clock.elapsed();
     //clock.restart();
-    fillChannelInfos(maxchannel);
+    fillChannelInfos();
+    maxchannel = max((int)m_channelInfos.size() - 1, 0);
     setStartChannel((int)(m_currentStartChannel) - 
                     (int)(desiredDisplayChans / 2));
-    if (DISPLAY_CHANS > maxchannel)
-        DISPLAY_CHANS = maxchannel;
+    DISPLAY_CHANS = min(DISPLAY_CHANS, maxchannel + 1);
 
     //int fillchannels = clock.elapsed();
     //clock.restart();
@@ -265,13 +277,13 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
 GuideGrid::~GuideGrid()
 {
     gContext->removeListener(this);
-    for (int x = 0; x < DISPLAY_TIMES; x++)
+    for (int x = 0; x < MAX_DISPLAY_TIMES; x++)
     {
         if (m_timeInfos[x])
             delete m_timeInfos[x];
     }
 
-    for (int y = 0; y < DISPLAY_CHANS; y++)
+    for (int y = 0; y < MAX_DISPLAY_CHANS; y++)
     {
         if (m_programs[y])
             delete m_programs[y];
@@ -326,6 +338,22 @@ void GuideGrid::keyPressEvent(QKeyEvent *e)
     }
     else
     {
+        // We want to handle jump to channel before everything else
+        // The reason is because the number keys could be mapped to
+        // other things. If this is the case, then the jump to channel
+        // will not work correctly.
+        if (jumpToChannelEnabled) 
+        {
+            int digit;
+            if (jumpToChannelGetInputDigit(actions, digit)) 
+            {
+                jumpToChannelDigitPress(digit);
+                // Set handled = true here so that it won't process any
+                // more actions.
+                handled = true;
+            }
+        }        
+	      
         for (unsigned int i = 0; i < actions.size() && !handled; i++)
         {
             QString action = actions[i];
@@ -351,9 +379,6 @@ void GuideGrid::keyPressEvent(QKeyEvent *e)
                 dayLeft();
             else if (action == "DAYRIGHT")
                 dayRight();
-            else if (jumpToChannelEnabled &&
-                     ((action[0] >= '0') && (action[0] <= '9')))
-                jumpToChannelDigitPress(action.toInt());
             else if (jumpToChannelEnabled && jumpToChannelActive &&
                      (action == "ESCAPE"))
                 jumpToChannelCancel();
@@ -390,6 +415,8 @@ void GuideGrid::keyPressEvent(QKeyEvent *e)
             }
             else if (action == "INFO")
                 editScheduled();
+            else if (action == "CUSTOMEDIT")
+                customEdit();
             else if (action == "UPCOMING")
                 upcoming();
             else if (action == "DETAILS")
@@ -485,15 +512,22 @@ void GuideGrid::parseContainer(QDomElement &element)
         videoRect = area;
 }
 
-QString GuideGrid::getLastChannel(void)
+QString GuideGrid::GetChanNum(void)
 {
-    unsigned int chanNum = m_currentRow + m_currentStartChannel;
-    if (chanNum >= m_channelInfos.size())
-        chanNum -= m_channelInfos.size();
+    if (m_channelInfos.empty() || !selectState)
+        return "";
 
-    if (selectState)
-        return m_channelInfos[chanNum].chanstr;
-    return 0;
+    uint idx = (m_currentRow + m_currentStartChannel) % m_channelInfos.size();
+    return m_channelInfos[idx].chanstr;
+}
+
+uint GuideGrid::GetChanID(void)
+{
+    if (m_channelInfos.empty() || !selectState)
+        return 0;
+
+    uint idx = (m_currentRow + m_currentStartChannel) % m_channelInfos.size();
+    return m_channelInfos[idx].chanid;
 }
 
 void GuideGrid::timeout()
@@ -524,76 +558,62 @@ void GuideGrid::timeout()
     repaint(curInfoRect, false);
 }
 
-void GuideGrid::fillChannelInfos(int &maxchannel, bool gotostartchannel)
+void GuideGrid::fillChannelInfos(bool gotostartchannel)
 {
     m_channelInfos.clear();
 
-    QString queryfav;
-    QSqlQuery query;
-
-    QString queryall = "SELECT channel.channum, channel.callsign, "
-                       "channel.icon, channel.chanid, favorites.favid, "
-                       "channel.name FROM channel LEFT JOIN favorites ON "
-                       "favorites.chanid = channel.chanid WHERE visible = 1 "
-                       "GROUP BY channum, callsign "
-                       "ORDER BY " + channelOrdering + ";";
+    DBChanList channels = ChannelUtil::GetChannels(0, true,
+                                                   "channum, callsign");
+    ChannelUtil::SortChannels(channels, channelOrdering, true);
 
     if (showFavorites)
     {
-        queryfav = "SELECT channel.channum, channel.callsign, "
-                   "channel.icon, channel.chanid, favorites.favid, "
-                   "channel.name FROM favorites, channel WHERE "
-                   "channel.chanid = favorites.chanid and visible = 1 "
-                   "ORDER BY " + channelOrdering + ";";
-
-        query.exec(queryfav);   
-
-        // If we don't have any favorites, then just show regular listings.
-        if (!query.isActive() || query.numRowsAffected() == 0)
+        DBChanList tmp;
+        for (uint i = 0; i < channels.size(); i++)
         {
-            showFavorites = (!showFavorites);
-            query.exec(queryall);
+            if (channels[i].favorite)
+                tmp.push_back(channels[i]);
         }
+
+        if (!tmp.empty())
+            channels = tmp;
     }
-    else
+
+    bool startingset = false;
+    for (uint i = 0; i < channels.size(); i++)
     {
-        query.exec(queryall);
+        ChannelInfo val;
+        val.chanstr  = channels[i].channum;
+        val.chanid   = channels[i].chanid;
+        val.callsign = channels[i].callsign;
+        val.favid    = channels[i].favorite;
+        val.channame = channels[i].name;
+        val.iconpath = channels[i].icon;
+        val.iconload = false;
+
+        // set starting channel index if it hasn't been set
+        bool match   = gotostartchannel && !startingset;
+        match &= (startChanID)  ? val.chanid  == (int)startChanID : true;
+        match &= (!startChanID) ? val.chanstr == startChanNum     : true;
+        if (match)
+        {
+            m_currentStartChannel = m_channelInfos.size();
+            startingset = true;
+        }
+
+        // add the new channel to the list
+        m_channelInfos.push_back(val);
     }
  
-    bool set = false;
-    maxchannel = 0;
-    
-    if (query.isActive() && query.numRowsAffected() > 0)
+    // set starting channel index to 0 if it hasn't been set
+    if (gotostartchannel)
+        m_currentStartChannel = (startingset) ? m_currentStartChannel : 0;
+
+    if (m_channelInfos.empty())
     {
-        while (query.next())
-        {
-            ChannelInfo val;
-            val.chanstr = query.value(0).toString();
-            if (val.chanstr != "")
-            {
-                val.callsign = QString::fromUtf8(query.value(1).toString());
-                val.iconpath = query.value(2).toString();
-                val.chanstr = query.value(0).toString();
-                val.chanid = query.value(3).toInt();
-                val.favid = query.value(4).toInt();
-                val.channame = QString::fromUtf8(query.value(5).toString());
-                val.icon = NULL;
-        
-                if (gotostartchannel && val.chanstr == m_startChanStr && !set)
-                {
-                    m_currentStartChannel = m_channelInfos.size();
-                    set = true;
-                }
-                
-                m_channelInfos.push_back(val);
-                maxchannel++;
-            }
-        }
-    }
-    else
-    {
-        cerr << "You don't have any channels defined in the database\n";
-        cerr << "This isn't going to work.\n";
+        VERBOSE(VB_IMPORTANT, "GuideGrid: "
+                "\n\t\t\tYou don't have any channels defined in the database."
+                "\n\t\t\tGuide grid will have nothing to show you.");
     }
 }
 
@@ -690,22 +710,30 @@ void GuideGrid::fillProgramRowInfos(unsigned int row)
         m_programInfos[row][x] = NULL;
     }
 
+    if (m_channelInfos.size() == 0)
+        return;
+
     int chanNum = row + m_currentStartChannel;
-    if (chanNum >= (int)m_channelInfos.size())
-        chanNum -= (int)m_channelInfos.size();
+    if (chanNum >= (int) m_channelInfos.size())
+        chanNum -= (int) m_channelInfos.size();
+    if (chanNum >= (int) m_channelInfos.size())
+        return;
+
     if (chanNum < 0)
         chanNum = 0;
 
     m_programs[row] = proglist = new ProgramList();
 
-    QString querystr = QString(
-        "WHERE program.chanid = %1 AND program.endtime >= %2 "
-        "    AND program.starttime <= %3 ")
-        .arg(m_channelInfos[chanNum].chanid)
-        .arg(m_currentStartTime.toString("yyyyMMddhhmm00"))
-        .arg(m_currentEndTime.toString("yyyyMMddhhmm00"));
+    MSqlBindings bindings;
+    QString querystr = "WHERE program.chanid = :CHANID "
+                       "  AND program.endtime >= :STARTTS "
+                       "  AND program.starttime <= :ENDTS "
+                       "  AND program.manualid = 0 ";
+    bindings[":CHANID"] = m_channelInfos[chanNum].chanid;
+    bindings[":STARTTS"] = m_currentStartTime.toString("yyyy-MM-ddThh:mm:00");
+    bindings[":ENDTS"] = m_currentEndTime.toString("yyyy-MM-ddThh:mm:00");
 
-    proglist->FromProgram(m_db, querystr, m_recList);
+    proglist->FromProgram(querystr, bindings, m_recList);
 
     QDateTime ts = m_currentStartTime;
 
@@ -879,7 +907,8 @@ void GuideGrid::fillProgramRowInfos(unsigned int row)
                 }
 
                 int recStat;
-                if (proginfo->recstatus == rsConflict)
+                if (proginfo->recstatus == rsConflict ||
+                    proginfo->recstatus == rsOffLine)
                     recStat = 2;
                 if (proginfo->recstatus <= rsWillRecord)
                     recStat = 1;
@@ -1063,6 +1092,9 @@ void GuideGrid::paintChannels(QPainter *p)
     if (infocontainer)
         itype = (UIImageType *)container->GetType("icon");
 
+    if (type && type->GetNums() != DISPLAY_CHANS)
+        type->SetSize(DISPLAY_CHANS);
+
     ChannelInfo *chinfo = &(m_channelInfos[m_currentStartChannel]);
 
     for (unsigned int y = 0; y < (unsigned int)DISPLAY_CHANS; y++)
@@ -1070,22 +1102,29 @@ void GuideGrid::paintChannels(QPainter *p)
         unsigned int chanNumber = y + m_currentStartChannel;
         if (chanNumber >= m_channelInfos.size())
             chanNumber -= m_channelInfos.size();
-  
+        if (chanNumber >= m_channelInfos.size())
+            break;  
+
         chinfo = &(m_channelInfos[chanNumber]);
         if ((y == (unsigned int)2 && scrolltype != 1) || 
             ((signed int)y == m_currentRow && scrolltype == 1))
         {
-            if (chinfo->iconpath != "none" && chinfo->iconpath != "")
+            if (!chinfo->iconpath.isEmpty() && chinfo->iconpath != "none")
             {
                 int iconsize = 0;
                 if (itype)
-                    iconsize = itype->GetSize();
+                    iconsize = itype->GetSize().width();
                 else if (type)
                     iconsize = type->GetSize();
-                if (!chinfo->icon)
+                if (!chinfo->iconload)
                     chinfo->LoadIcon(iconsize);
-                if (chinfo->icon && itype)
-                    itype->SetImage(*(chinfo->icon));
+                if (chinfo->iconload)
+                {
+                    if (itype)
+                        itype->SetImage(chinfo->icon);
+                }
+                else
+                    chinfo->iconpath = QString::null;
             }
         }
 
@@ -1098,14 +1137,19 @@ void GuideGrid::paintChannels(QPainter *p)
 
         if (type)
         {
-            if (chinfo->iconpath != "none" && chinfo->iconpath != "")
+            if (!chinfo->iconpath.isEmpty() && chinfo->iconpath != "none")
             {
                 int iconsize = 0;
                 iconsize = type->GetSize();
-                if (!chinfo->icon)
+                if (!chinfo->iconload)
                     chinfo->LoadIcon(iconsize);
-                if (chinfo->icon)
-                    type->SetIcon(y, *(chinfo->icon));
+                if (chinfo->iconload)
+                    type->SetIcon(y, chinfo->icon);
+                else
+                {
+                    chinfo->iconpath = QString::null;
+                    type->ResetImage(y);
+                }
             }
             else
             {
@@ -1180,6 +1224,9 @@ void GuideGrid::paintPrograms(QPainter *p)
 
 void GuideGrid::paintInfo(QPainter *p)
 {
+    if (m_currentRow < 0 || m_currentCol < 0)
+        return;
+
     ProgramInfo *pginfo = m_programInfos[m_currentRow][m_currentCol];
     if (!pginfo)
         return;
@@ -1193,12 +1240,14 @@ void GuideGrid::paintInfo(QPainter *p)
     int chanNum = m_currentRow + m_currentStartChannel;
     if (chanNum >= (int)m_channelInfos.size())
         chanNum -= (int)m_channelInfos.size();
+    if (chanNum >= (int)m_channelInfos.size())
+        return;
     if (chanNum < 0)
         chanNum = 0;
 
     ChannelInfo *chinfo = &(m_channelInfos[chanNum]);
 
-    pginfo->ToMap(m_db, infoMap);
+    pginfo->ToMap(infoMap);
 
     LayerSet *container = theme->GetSet("program_info");
     if (container)
@@ -1206,12 +1255,6 @@ void GuideGrid::paintInfo(QPainter *p)
         container->ClearAllText();
         container->SetText(infoMap);
 
-        UITextType *type;
-
-        type = (UITextType *)container->GetType("misicon");
-        if (type)
-            type->SetText(chinfo->callsign);
-
         UIImageType *itype = (UIImageType *)container->GetType("icon");
         if (itype)
         {
@@ -1219,6 +1262,13 @@ void GuideGrid::paintInfo(QPainter *p)
             itype->LoadImage();
         }
 
+        if (!itype || chinfo->iconpath.isEmpty() || chinfo->iconpath == "none")
+        {
+            UITextType *type = (UITextType *)container->GetType("misicon");
+            if (type)
+                type->SetText(chinfo->callsign);
+        }
+
         container->Draw(&tmp, 1, m_context);
         container->Draw(&tmp, 2, m_context);
         container->Draw(&tmp, 3, m_context);
@@ -1246,9 +1296,9 @@ void GuideGrid::generateListings()
 
     int maxchannel = 0;
     DISPLAY_CHANS = desiredDisplayChans;
-    fillChannelInfos(maxchannel);
-    if (DISPLAY_CHANS > maxchannel)
-        DISPLAY_CHANS = maxchannel;
+    fillChannelInfos();
+    maxchannel = max((int)m_channelInfos.size() - 1, 0);
+    DISPLAY_CHANS = min(DISPLAY_CHANS, maxchannel + 1);
 
     m_recList.FromScheduler();
     fillProgramInfos();
@@ -1257,13 +1307,14 @@ void GuideGrid::generateListings()
 
 void GuideGrid::toggleChannelFavorite()
 {
-    QString thequery;
-    QSqlQuery query;
+    MSqlQuery query(MSqlQuery::InitCon());
 
     // Get current channel id, and make sure it exists...
     int chanNum = m_currentRow + m_currentStartChannel;
     if (chanNum >= (int)m_channelInfos.size())
         chanNum -= (int)m_channelInfos.size();
+    if (chanNum >= (int)m_channelInfos.size())
+        return;
     if (chanNum < 0)
         chanNum = 0;
 
@@ -1272,18 +1323,16 @@ void GuideGrid::toggleChannelFavorite()
 
     if (favid > 0) 
     {
-        thequery = QString("DELETE FROM favorites WHERE favid = '%1'")
-                           .arg(favid);
-
-        query.exec(thequery);
+        query.prepare("DELETE FROM favorites WHERE favid = :FAVID ;");
+        query.bindValue(":FAVID", favid);
+        query.exec(); 
     }
     else
     {
         // We have no favorites record...Add one to toggle...
-        thequery = QString("INSERT INTO favorites (chanid) VALUES ('%1')")
-                           .arg(chanid);
-
-        query.exec(thequery);
+        query.prepare("INSERT INTO favorites (chanid) VALUES (:FAVID);");
+        query.bindValue(":FAVID", chanid);
+        query.exec(); 
     }
 
     if (showFavorites)
@@ -1292,9 +1341,9 @@ void GuideGrid::toggleChannelFavorite()
     {
         int maxchannel = 0;
         DISPLAY_CHANS = desiredDisplayChans;
-        fillChannelInfos(maxchannel, false);
-        if (DISPLAY_CHANS > maxchannel)
-            DISPLAY_CHANS = maxchannel;
+        fillChannelInfos(false);
+        maxchannel = max((int)m_channelInfos.size() - 1, 0);
+        DISPLAY_CHANS = min(DISPLAY_CHANS, maxchannel + 1);
 
         repaint(channelRect, false);
     }
@@ -1602,7 +1651,7 @@ void GuideGrid::quickRecord()
     if (pginfo->title == unknownTitle)
         return;
 
-    pginfo->ToggleRecord(m_db);
+    pginfo->ToggleRecord();
 
     m_recList.FromScheduler();
     fillProgramInfos();
@@ -1624,7 +1673,7 @@ void GuideGrid::editRecording()
     setFocusPolicy(QWidget::NoFocus);
 
     ProgramInfo *temppginfo = new ProgramInfo(*pginfo);
-    temppginfo->EditRecording(m_db);
+    temppginfo->EditRecording();
     delete temppginfo;
 
     setFocusPolicy(storeFocus);
@@ -1651,7 +1700,7 @@ void GuideGrid::editScheduled()
     setFocusPolicy(QWidget::NoFocus);
 
     ProgramInfo *temppginfo = new ProgramInfo(*pginfo);
-    temppginfo->EditScheduled(m_db);
+    temppginfo->EditScheduled();
     delete temppginfo;
 
     setFocusPolicy(storeFocus);
@@ -1665,6 +1714,19 @@ void GuideGrid::editScheduled()
     repaint(fullRect, false);
 }
 
+void GuideGrid::customEdit()
+{
+    ProgramInfo *pginfo = m_programInfos[m_currentRow][m_currentCol];
+
+    if (!pginfo)
+        return;
+
+    CustomEdit *ce = new CustomEdit(gContext->GetMainWindow(),
+                                    "customedit", pginfo);
+    ce->exec();
+    delete ce;
+}
+
 void GuideGrid::upcoming()
 {
     ProgramInfo *pginfo = m_programInfos[m_currentRow][m_currentCol];
@@ -1675,8 +1737,7 @@ void GuideGrid::upcoming()
     if (pginfo->title == unknownTitle)
         return;
 
-    ProgLister *pl = new ProgLister(plTitle, pginfo->title,
-                                   QSqlDatabase::database(),
+    ProgLister *pl = new ProgLister(plTitle, pginfo->title, "",
                                    gContext->GetMainWindow(), "proglist");
     pl->exec();
     delete pl;
@@ -1692,21 +1753,19 @@ void GuideGrid::details()
     if (pginfo->title == unknownTitle)
         return;
 
-    pginfo->showDetails(QSqlDatabase::database());
+    pginfo->showDetails();
 }
 
 void GuideGrid::channelUpdate(void)
 {
-    if (!m_player)
+    if (m_channelInfos.empty())
         return;
 
-    int chanNum = m_currentRow + m_currentStartChannel;
-    if (chanNum >= (int)m_channelInfos.size())
-        chanNum -= (int)m_channelInfos.size();
-    if (chanNum < 0)
-        chanNum = 0;
+    uint idx = (m_currentRow + m_currentStartChannel) % m_channelInfos.size();
+    ChannelInfo info = m_channelInfos[idx];
 
-    m_player->EPGChannelUpdate(m_channelInfos[chanNum].chanstr);
+    if (m_player)
+        m_player->EPGChannelUpdate(info.chanid, info.chanstr);
 }
 
 //
@@ -1757,9 +1816,10 @@ void GuideGrid::jumpToChannelDeleteLastDigit()
 {
     jumpToChannel /= 10;
     
-    if (jumpToChannel == 0) {
+    if (jumpToChannel == 0) 
         jumpToChannelCancel();
-    }
+    else 
+        jumpToChannelShowSelection();
 }
 
 void GuideGrid::jumpToChannelShowSelection()
@@ -1816,3 +1876,20 @@ void GuideGrid::jumpToChannelTimeout()
     jumpToChannelCommit();
 }
 
+// function returns true if it is able to find a mapping to a digit and false
+// otherwise.
+bool GuideGrid::jumpToChannelGetInputDigit(QStringList &actions, int &digit)
+{
+    for (unsigned int i = 0; i < actions.size(); ++i) 
+    {
+        QString action = actions[i];
+        if (action[0] >= '0' && action[0] <= '9') 
+        {
+            digit = action.toInt();
+            return true;
+        }
+    }
+    
+    // There were no actions that resolved to a digit
+    return false;
+}
