@@ -51,22 +51,29 @@
      bbinfo from Brent Beyeler, beyeler@home.com
 */
 
-#include <iostream>
-#include <pthread.h>
+// C includes
 #include <cstdio>
 #include <cstdlib>
 #include <cerrno>
+#include <ctime>
+#include <fcntl.h>
+#include <pthread.h>
+#include <unistd.h>
+
+// C system includes
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <fcntl.h>
-#include <unistd.h>
 #include <sys/ioctl.h>
 #include <sys/time.h>
-#include <ctime>
-#include "videodev_myth.h"
 
+// C++ includes
+#include <iostream>
 using namespace std;
 
+// MythTV system include
+#include "videodev_myth.h"
+
+// MythTV includes
 #include "hdtvrecorder.h"
 #include "RingBuffer.h"
 #include "mythcontext.h"
@@ -74,8 +81,11 @@ using namespace std;
 #include "programinfo.h"
 #include "channel.h"
 #include "mpegtables.h"
+#include "atsctables.h"
 #include "atscstreamdata.h"
+#include "tv_rec.h"
 
+// AVLib/FFMPEG includes
 extern "C" {
 #include "../libavcodec/avcodec.h"
 #include "../libavformat/avformat.h"
@@ -101,10 +111,12 @@ extern "C" {
         };
 #endif
 
-HDTVRecorder::HDTVRecorder()
-    : DTVRecorder(), _atsc_stream_data(0), _resync_count(0)
+HDTVRecorder::HDTVRecorder(TVRec *rec)
+    : DTVRecorder(rec),
+      _atsc_stream_data(NULL),
+      _resync_count(0)
 {
-    _atsc_stream_data = new ATSCStreamData(-1, DEFAULT_SUBCHANNEL);
+    SetStreamData(new ATSCStreamData(-1, DEFAULT_SUBCHANNEL));
 
     _buffer_size = TSPacket::SIZE * 1500;
     if ((_buffer = new unsigned char[_buffer_size])) {
@@ -121,18 +133,29 @@ HDTVRecorder::HDTVRecorder()
     loop = random() % (report_loops / 2);
 }
 
-HDTVRecorder::~HDTVRecorder()
+void HDTVRecorder::TeardownAll(void)
 {
-    if (_stream_fd >= 0)
-        close(_stream_fd);
-    if (_atsc_stream_data)
-        delete _atsc_stream_data;
-    if (_buffer)
-        delete[] _buffer;
-
     // Make SURE that the ringbuffer thread is cleaned up
     StopRecording();
 
+    if (_stream_fd >= 0)
+    {
+        close(_stream_fd);
+        _stream_fd = -1;
+    }
+    
+    SetStreamData(NULL);
+
+    if (_buffer)
+    {
+        delete[] _buffer;
+        _buffer = NULL;
+    }
+}
+
+HDTVRecorder::~HDTVRecorder()
+{
+    TeardownAll();
     pthread_mutex_destroy(&ringbuf.lock);
     pthread_mutex_destroy(&ringbuf.lock_stats);
 }
@@ -140,12 +163,11 @@ HDTVRecorder::~HDTVRecorder()
 void HDTVRecorder::SetOptionsFromProfile(RecordingProfile *profile,
                                          const QString &videodev,
                                          const QString &audiodev,
-                                         const QString &vbidev, int ispip)
+                                         const QString &vbidev)
 {
     (void)audiodev;
     (void)vbidev;
     (void)profile;
-    (void)ispip;
 
     SetOption("videodevice", videodev);
     SetOption("tvformat", gContext->GetSetting("TVFormat"));
@@ -183,6 +205,30 @@ bool HDTVRecorder::Open()
     return (_stream_fd>0);
 }
 
+void HDTVRecorder::SetStreamData(MPEGStreamData *xdata)
+{
+    ATSCStreamData *data = dynamic_cast<ATSCStreamData*>(xdata);
+
+    if (data == _atsc_stream_data)
+        return;
+
+    ATSCStreamData *old_data = _atsc_stream_data;
+    _atsc_stream_data = data;
+    if (old_data)
+        delete old_data;
+
+    if (data)
+    {
+        data->AddMPEGSPListener(this);
+        data->AddATSCMainListener(this);
+    }
+}
+
+MPEGStreamData *HDTVRecorder::GetStreamData(void)
+{
+    return _atsc_stream_data;
+}
+
 bool readchan(int chanfd, unsigned char* buffer, int dlen) {
     int len = read(chanfd, buffer, dlen); // read next byte
     if (dlen != len)
@@ -250,6 +296,10 @@ void HDTVRecorder::fill_ringbuffer(void)
     size_t    read_size;
     bool      run, request_pause, paused;
 
+    pthread_mutex_lock(&ringbuf.lock);
+    ringbuf.run = true;
+    pthread_mutex_unlock(&ringbuf.lock);
+
     for (;;)
     {
         pthread_mutex_lock(&ringbuf.lock);
@@ -269,6 +319,8 @@ void HDTVRecorder::fill_ringbuffer(void)
             pthread_mutex_unlock(&ringbuf.lock);
 
             pauseWait.wakeAll();
+            if (tvrec)
+                tvrec->RecorderPaused();
 
             usleep(1000);
             continue;
@@ -367,8 +419,6 @@ void HDTVRecorder::fill_ringbuffer(void)
 
     close(_stream_fd);
     _stream_fd = -1;
-
-    pthread_exit(reinterpret_cast<void *>(0));
 }
 
 /* read count bytes from ring into buffer */
@@ -392,7 +442,7 @@ int HDTVRecorder::ringbuf_read(unsigned char *buffer, size_t count)
     {
         usleep(50000);
 
-        if (_request_pause || dev_error || dev_eof)
+        if (request_pause || dev_error || dev_eof)
             return 0;
 
         pthread_mutex_lock(&ringbuf.lock);
@@ -511,7 +561,6 @@ void HDTVRecorder::StartRecording(void)
     ringbuf.avg_cnt = 0;
     ringbuf.request_pause = false;
     ringbuf.paused = false;
-    ringbuf.run = true;
     ringbuf.error = false;
     ringbuf.eof = false;
 
@@ -539,7 +588,7 @@ void HDTVRecorder::StartRecording(void)
         pause = ringbuf.paused;
         pthread_mutex_unlock(&ringbuf.lock);
 
-        if (_request_pause)
+        if (request_pause)
         {
             pthread_mutex_lock(&ringbuf.lock);
             ringbuf.request_pause = true;
@@ -560,6 +609,7 @@ void HDTVRecorder::StartRecording(void)
 
         if (dev_error)
         {
+            VERBOSE(VB_IMPORTANT, "HDTV: device error detected");
             _error = true;
             break;
         }
@@ -574,9 +624,8 @@ void HDTVRecorder::StartRecording(void)
 
         len += remainder;
         remainder = ProcessData(_buffer, len);
-        if (remainder > 0) // leftover bytes
-            memmove(_buffer, &(_buffer[_buffer_size - remainder]),
-                    remainder);
+        if (remainder > 0 && (len > remainder)) // leftover bytes
+            memmove(_buffer, &(_buffer[len - remainder]), remainder);
     }
 
     FinishRecording();
@@ -585,28 +634,27 @@ void HDTVRecorder::StartRecording(void)
 
 void HDTVRecorder::StopRecording(void)
 {
-    bool            run;
-    int             idx;
+    TVRec *rec = tvrec;
+    tvrec = NULL; // don't notify of pause..
 
-    _request_pause = true;
-    for (idx = 0; idx < 400; ++idx)
+    bool ok = true;
+    if (!IsPaused())
     {
-        if (GetPause())
-            break;
-        pauseWait.wait(500);
+        Pause();
+        ok = WaitForPause(250);
     }
 
     _request_recording = false;
 
     pthread_mutex_lock(&ringbuf.lock);
-    run = ringbuf.run;
+    bool run = ringbuf.run;
     ringbuf.run = false;
     pthread_mutex_unlock(&ringbuf.lock);
 
     if (run)
         pthread_join(ringbuf.thread, NULL);
 
-    if (idx == 400)
+    if (!ok)
     {
         // Better to have a memory leak, then a segfault?
         VERBOSE(VB_IMPORTANT, "DTV ringbuffer not cleaned up!\n");
@@ -616,6 +664,7 @@ void HDTVRecorder::StopRecording(void)
         delete[] ringbuf.buffer;
         ringbuf.buffer = 0;
     }
+    tvrec = rec;
 }
 
 void HDTVRecorder::Pause(bool /*clear*/)
@@ -623,37 +672,29 @@ void HDTVRecorder::Pause(bool /*clear*/)
     pthread_mutex_lock(&ringbuf.lock);
     ringbuf.paused = false;
     pthread_mutex_unlock(&ringbuf.lock);
-    _request_pause = true;
+    request_pause = true;
 }
 
-bool HDTVRecorder::GetPause(void)
+bool HDTVRecorder::IsPaused(void) const
 {
-    bool paused;
-
     pthread_mutex_lock(&ringbuf.lock);
-    paused = ringbuf.paused;
+    bool paused = ringbuf.paused;
     pthread_mutex_unlock(&ringbuf.lock);
 
     return paused;
 }
 
-void HDTVRecorder::WaitForPause(void)
-{
-    if (!GetPause())
-        if (!pauseWait.wait(1000))
-            VERBOSE(VB_IMPORTANT,
-                    QString("Waited too long for recorder to pause"));
-}
-
-int HDTVRecorder::ResyncStream(unsigned char *buffer, int curr_pos, int len)
+int HDTVRecorder::ResyncStream(const unsigned char *buffer,
+                               uint curr_pos, uint len)
 {
     // Search for two sync bytes 188 bytes apart, 
-    int pos = curr_pos;
-    int nextpos = pos + TSPacket::SIZE;
+    uint pos = curr_pos;
+    uint nextpos = pos + TSPacket::SIZE;
     if (nextpos >= len)
         return -1; // not enough bytes; caller should try again
     
-    while (buffer[pos] != SYNC_BYTE || buffer[nextpos] != SYNC_BYTE) {
+    while (buffer[pos] != SYNC_BYTE || buffer[nextpos] != SYNC_BYTE)
+    {
         pos++;
         nextpos++;
         if (nextpos == len)
@@ -663,146 +704,122 @@ int HDTVRecorder::ResyncStream(unsigned char *buffer, int curr_pos, int len)
     return pos;
 }
 
-void HDTVRecorder::WritePAT() {
-    if (StreamData()->PAT()) {
-        ProgramAssociationTable* pat = StreamData()->PAT();
-        int next = (pat->tsheader()->ContinuityCounter()+1)&0xf;
-        pat->tsheader()->SetContinuityCounter(next);
-        ringBuffer->Write(pat->tsheader()->data(), TSPacket::SIZE);
+void HDTVRecorder::HandleSingleProgramPAT(ProgramAssociationTable *pat)
+{
+    if (!pat)
+        return;
+
+    int next = (pat->tsheader()->ContinuityCounter()+1)&0xf;
+    pat->tsheader()->SetContinuityCounter(next);
+    BufferedWrite(*(reinterpret_cast<TSPacket*>(pat->tsheader())));
+}
+
+void HDTVRecorder::HandleSingleProgramPMT(ProgramMapTable* pmt)
+{
+    if (!pmt)
+        return;
+
+    int next = (pmt->tsheader()->ContinuityCounter()+1)&0xf;
+    pmt->tsheader()->SetContinuityCounter(next);
+    BufferedWrite(*(reinterpret_cast<TSPacket*>(pmt->tsheader())));
+}
+
+/** \fn HDTVRecorder::HandleMGT(const MasterGuideTable*)
+ *  \brief Handles Master Guide Table, by enabling the 
+ *         scanning of all PIDs listed.
+ */
+void HDTVRecorder::HandleMGT(const MasterGuideTable *mgt)
+{
+    for (unsigned int i=0; i<mgt->TableCount(); i++)
+        GetStreamData()->AddListeningPID(mgt->TablePID(i));
+}
+
+/** \fn HDTVRecorder::HandleVCT(uint, const VirtualChannelTable*)
+ *  \brief Handles Virtual Channel Tables by finding the program
+ *         number to use.
+ *  \bug Assumes there is only one VCT, may break on Cable.
+ */
+void HDTVRecorder::HandleVCT(uint /*tsid*/, const VirtualChannelTable *vct)
+{
+    if (vct->ChannelCount() < 1)
+    {
+        VERBOSE(VB_IMPORTANT,
+                "HDTVRecorder::HandleVCT: table has no channels");
+        return;
+    }
+
+    bool found = false;    
+    VERBOSE(VB_RECORD, QString("Desired channel %1_%2")
+            .arg(GetATSCStreamData()->DesiredMajorChannel())
+            .arg(GetATSCStreamData()->DesiredMinorChannel()));
+    for (uint i = 0; i < vct->ChannelCount(); i++)
+    {
+        int maj = GetATSCStreamData()->DesiredMajorChannel();
+        int min = GetATSCStreamData()->DesiredMinorChannel();
+        if ((maj == -1 || vct->MajorChannel(i) == (uint)maj) &&
+            (vct->MinorChannel(i) == (uint)min))
+        {
+            uint pnum = (uint) GetATSCStreamData()->DesiredProgram();
+            if (vct->ProgramNumber(i) != pnum)
+            {
+                VERBOSE(VB_RECORD, 
+                        QString("Resetting desired program from %1"
+                                " to %2")
+                        .arg(GetATSCStreamData()->DesiredProgram())
+                        .arg(vct->ProgramNumber(i)));
+                // Do a (partial?) reset here if old desired
+                // program is not 0?
+                GetATSCStreamData()->SetDesiredProgram(vct->ProgramNumber(i));
+            }
+            found = true;
+        }
+    }
+    if (!found)
+    {
+        VERBOSE(VB_IMPORTANT, 
+                QString("Desired channel %1_%2 not found;"
+                        " using %3_%4 instead.")
+                .arg(GetATSCStreamData()->DesiredMajorChannel())
+                .arg(GetATSCStreamData()->DesiredMinorChannel())
+                .arg(vct->MajorChannel(0))
+                .arg(vct->MinorChannel(0)));
+        VERBOSE(VB_IMPORTANT, vct->toString());
+        GetATSCStreamData()->SetDesiredProgram(vct->ProgramNumber(0));
     }
 }
 
-#if WHACK_A_BUG_VIDEO
-static int WABV_base_pid     = 0x100;
-#define WABV_WAIT 60
-static int WABV_wait_a_while = WABV_WAIT;
-bool WABV_started = false;
-#endif
-
-#if WHACK_A_BUG_AUDIO
-static int WABA_base_pid     = 0x200;
-#define WABA_WAIT 60
-static int WABA_wait_a_while = WABA_WAIT;
-bool WABA_started = false;
-#endif
-
-void HDTVRecorder::WritePMT() {
-    if (StreamData()->PMT()) {
-        ProgramMapTable* pmt = StreamData()->PMT();
-        int next = (pmt->tsheader()->ContinuityCounter()+1)&0xf;
-        pmt->tsheader()->SetContinuityCounter(next);
-
-#if WHACK_A_BUG_VIDEO
-        WABV_wait_a_while--;
-        if (WABV_wait_a_while<=0) {
-            WABV_started = true;
-            WABV_wait_a_while = WABV_WAIT;
-            WABV_base_pid = (((WABV_base_pid-0x100)+1)%32)+0x100;
-            if (StreamID::MPEG2Video != StreamData()->PMT()->StreamType(0))
-            {
-                VERBOSE(VB_IMPORTANT, "HDTVRecorder::WritePMT(): Error,"
-                        "Whack a Bug can not rewrite PMT, wrong stream type");
-            }
-            else
-            {
-                VERBOSE(VB_IMPORTANT, QString("Whack a Bug: new video pid %1").
-                        arg(WABV_base_pid));
-                // rewrite video pid
-                const uint old_video_pid=StreamData()->PMT()->StreamPID(0);
-                StreamData()->PMT()->SetStreamPID(0, WABV_base_pid);
-                if (StreamData()->PMT()->PCRPID() == old_video_pid)
-                    StreamData()->PMT()->SetPCRPID(WABV_base_pid);
-                StreamData()->PMT()->SetCRC(StreamData()->PMT()->CalcCRC());
-                VERBOSE(VB_IMPORTANT, StreamData()->PMT()->toString());
-            }
-        }
-#endif
-#if WHACK_A_BUG_AUDIO
-        WABA_wait_a_while--;
-        if (WABA_wait_a_while<=0) {
-            WABA_started = true;
-            WABA_wait_a_while = WABA_WAIT;
-            WABA_base_pid = (((WABA_base_pid-0x200)+1)%32)+0x200;
-            VERBOSE(VB_IMPORTANT, QString("Whack a Bug: new audio BASE pid %1").arg(WABA_base_pid));
-            // rewrite audio pids
-            for (uint i=0; i<StreamData()->PMT()->StreamCount(); i++) {
-                if (StreamID::MPEG2Audio == StreamData()->PMT()->StreamType(i) ||
-                    StreamID::MPEG2Audio == StreamData()->PMT()->StreamType(i)) {
-                    const uint old_audio_pid = StreamData()->PMT()->StreamPID(i);
-                    const uint new_audio_pid = WABA_base_pid + old_audio_pid;
-                    StreamData()->PMT()->SetStreamPID(i, new_audio_pid);
-                    if (StreamData()->PMT()->PCRPID() == old_audio_pid)
-                        StreamData()->PMT()->SetPCRPID(new_audio_pid);
-                    StreamData()->PMT()->SetCRC(StreamData()->PMT()->CalcCRC());
-                    VERBOSE(VB_IMPORTANT, StreamData()->PMT()->toString());
-                }
-            }
-        }
-#endif
-
-        ringBuffer->Write(pmt->tsheader()->data(), TSPacket::SIZE);
-    }
-}
-
-void HDTVRecorder::HandleVideo(const TSPacket* tspacket)
-{
-    FindKeyframes(tspacket);
-    // decoder needs video, of course (just this PID)
-    // delay until first GOP to avoid decoder crash on res change
-    if (_wait_for_keyframe && !_keyframe_seen)
-        return;
-
-#if WHACK_A_BUG_VIDEO
-    if (WABV_started)
-        ((TSPacket*)(tspacket))->SetPID(WABV_base_pid);
-#endif
-
-    ringBuffer->Write(tspacket->data(), TSPacket::SIZE);
-}
-
-void HDTVRecorder::HandleAudio(const TSPacket* tspacket)
-{
-    // decoder needs audio, of course (just this PID)
-    if (_wait_for_keyframe && !_keyframe_seen)
-        return;
-
-#if WHACK_A_BUG_AUDIO
-    if (WABA_started)
-        ((TSPacket*)(tspacket))->SetPID(WABA_base_pid+tspacket->PID());
-#endif
-
-    ringBuffer->Write(tspacket->data(), TSPacket::SIZE);
-}
-
-bool HDTVRecorder::ProcessTSPacket(const TSPacket& tspacket)
+bool HDTVRecorder::ProcessTSPacket(const TSPacket &tspacket)
 {
     bool ok = !tspacket.TransportError();
     if (ok && !tspacket.ScramplingControl())
     {
         if (tspacket.HasAdaptationField())
-            StreamData()->HandleAdaptationFieldControl(&tspacket);
+            GetStreamData()->HandleAdaptationFieldControl(&tspacket);
         if (tspacket.HasPayload())
         {
             const unsigned int lpid = tspacket.PID();
             // Pass or reject frames based on PID, and parse info from them
-            if (lpid == StreamData()->VideoPID())
-                HandleVideo(&tspacket);
-            else if (StreamData()->IsAudioPID(lpid))
-                HandleAudio(&tspacket);
-            else if (StreamData()->IsListeningPID(lpid))
-                StreamData()->HandleTables(&tspacket, this);
-            else if (StreamData()->IsWritingPID(lpid))
-                ringBuffer->Write(tspacket.data(), TSPacket::SIZE);
-            else if (StreamData()->VersionMGT()>=0)
+            if (lpid == GetStreamData()->VideoPIDSingleProgram())
+            {
+                _buffer_packets = !FindMPEG2Keyframes(&tspacket);
+                BufferedWrite(tspacket);
+            }
+            else if (GetStreamData()->IsAudioPID(lpid))
+                BufferedWrite(tspacket);
+            else if (GetStreamData()->IsListeningPID(lpid))
+                GetStreamData()->HandleTSTables(&tspacket);
+            else if (GetStreamData()->IsWritingPID(lpid))
+                BufferedWrite(tspacket);
+            else if (GetATSCStreamData()->VersionMGT()>=0)
                 _ts_stats.IncrPIDCount(lpid);
         }
     }
     return ok;
 }
 
-int HDTVRecorder::ProcessData(unsigned char *buffer, int len)
+int HDTVRecorder::ProcessData(const unsigned char *buffer, uint len)
 {
-    int pos = 0;
+    uint pos = 0;
 
     while (pos + 187 < len) // while we have a whole packet left
     {
@@ -823,13 +840,17 @@ int HDTVRecorder::ProcessData(unsigned char *buffer, int len)
         }
 
         const TSPacket *pkt = reinterpret_cast<const TSPacket*>(&buffer[pos]);
-        if (ProcessTSPacket(*pkt)) {
+        if (ProcessTSPacket(*pkt))
+        {
             pos += TSPacket::SIZE; // Advance to next TS packet
             _ts_stats.IncrTSPacketCount();
             if (0 == _ts_stats.TSPacketCount()%1000000)
                 VERBOSE(VB_RECORD, _ts_stats.toString());
-        } else // Let it resync in case of dropped bytes
-            buffer[pos] = SYNC_BYTE+1;
+        }
+        else
+        { // Let it resync in case of dropped bytes
+            pos++;
+        }
     }
 
     return len - pos;
@@ -837,8 +858,10 @@ int HDTVRecorder::ProcessData(unsigned char *buffer, int len)
 
 void HDTVRecorder::Reset(void)
 {
+    VERBOSE(VB_RECORD, "HDTVRecorder::Reset(void)");
     DTVRecorder::Reset();
 
+    _error = false;
     _resync_count = 0;
     _ts_stats.Reset();
 
@@ -849,8 +872,11 @@ void HDTVRecorder::Reset(void)
 
     if (_stream_fd >= 0) 
     {
-        Pause(true);
-        WaitForPause();
+        if (!IsPaused())
+        {
+            Pause();
+            WaitForPause();
+        }
         int ret = close(_stream_fd);
         if (ret < 0) 
         {
@@ -882,58 +908,4 @@ void HDTVRecorder::Reset(void)
         }
         Unpause();
     }
-
-    StreamData()->Reset();
-}
-
-void HDTVRecorder::ChannelNameChanged(const QString& new_chan)
-{
-    if (!_atsc_stream_data && !_buffer) 
-        return;
-
-    _wait_for_keyframe = _wait_for_keyframe_option;
-
-#if FAKE_VIDEO
-    RecorderBase::ChannelNameChanged("51");
-#else
-    DTVRecorder::ChannelNameChanged(new_chan);
-#endif
-
-    // look up freqid
- 
-    MSqlQuery query(MSqlQuery::InitCon());
-    QString thequery = QString("SELECT freqid "
-                               "FROM channel WHERE channum = \"%1\";")
-                              .arg(curChannelName);
-    query.prepare(thequery);
-
-    if (!query.exec() || !query.isActive())
-    {
-        MythContext::DBError("fetchtuningparamschanid", query);
-    }
-
-    if (query.size() <= 0)
-    {
-        return;
-    }
-    query.next();
-    QString freqid(query.value(0).toString());
-    VERBOSE(VB_RECORD, QString("Setting frequency for startRecording freqid: %1").arg(freqid));
-
-    int desired_channel = -1;
-    int desired_subchannel = DEFAULT_SUBCHANNEL;
-    int pos = freqid.find('-');
-    if (pos != -1) 
-    {
-        desired_channel = -1;
-        desired_subchannel = atoi(freqid.mid(pos+1).ascii());
-    }
-    else
-    {
-        VERBOSE(VB_IMPORTANT, QString("Error: Desired subchannel not "
-                                      "specified in freqid \"%1\", Using %2.")
-                                    .arg(freqid).arg(desired_subchannel));
-    }
-
-    StreamData()->Reset(desired_channel, desired_subchannel);
 }
