@@ -22,7 +22,9 @@ using namespace std;
 #include <qregexp.h>
 
 // avlib headers
+extern "C" {
 #include "../libavcodec/avcodec.h"
+}
 
 // MythTV headers
 #include "mpegrecorder.h"
@@ -456,11 +458,11 @@ bool MpegRecorder::SetV4L2DeviceOptions(int chanfd)
     ext_ctrl[3].id    = V4L2_CID_MPEG_AUDIO_L2_BITRATE;
     ext_ctrl[3].value = audbitrate - 1;
 
-    ext_ctrl[4].id    = V4L2_CID_MPEG_VIDEO_BITRATE;
-    ext_ctrl[4].value = (bitrate = min(bitrate, maxbitrate)) * 1000;
+    ext_ctrl[4].id    = V4L2_CID_MPEG_VIDEO_BITRATE_PEAK;
+    ext_ctrl[4].value = maxbitrate * 1000;
 
-    ext_ctrl[5].id    = V4L2_CID_MPEG_VIDEO_BITRATE_PEAK;
-    ext_ctrl[5].value = maxbitrate * 1000;
+    ext_ctrl[5].id    = V4L2_CID_MPEG_VIDEO_BITRATE;
+    ext_ctrl[5].value = (bitrate = min(bitrate, maxbitrate)) * 1000;
 
     ext_ctrl[6].id    = V4L2_CID_MPEG_STREAM_TYPE;
     ext_ctrl[6].value = V4L2_MPEG_STREAM_TYPE_MPEG2_PS;
@@ -875,7 +877,9 @@ void MpegRecorder::SavePositionMap(bool force)
     {
         curRecording->SetPositionMapDelta(positionMapDelta,
                                           MARK_GOP_START);
-        curRecording->SetFilesize(lastpackheaderpos);
+        // Stop setting the filesize here until we get the contention issue
+        // between with this thread and the scheduler worked out.
+        //curRecording->SetFilesize(lastpackheaderpos);
         positionMapDelta.clear();
     }
 }
