@@ -13,7 +13,7 @@ using namespace std;
 typedef QMap<long long, long long> frm_pos_map_t;
 typedef QMap<long long, int> frm_dir_map_t;
 
-#define NUMPROGRAMLINES 41
+#define NUMPROGRAMLINES 42
 
 typedef enum {
     MARK_UNSET = -10,
@@ -59,6 +59,7 @@ enum FlagMask {
 };
 
 enum RecStatusType {
+    rsFailed = -9,
     rsTunerBusy = -8,
     rsLowDiskSpace = -7,
     rsCancelled = -6,
@@ -157,8 +158,8 @@ class ProgramInfo
 
     // Quick gets
     bool SetRecordBasename(QString basename);
-    QString GetRecordBasename(void) const;
-    QString GetRecordFilename(const QString &prefix) const;
+    QString GetRecordBasename(bool fromDB = false) const;
+    QString GetRecordFilename(const QString &prefix, bool fromDB = false) const;
     QString GetPlaybackURL(QString playbackHost = "") const;
     QString MakeUniqueKey(void) const;
     int CalculateLength(void) const;
