@@ -4,7 +4,7 @@
  *
  * = DESCRIPTION
  * Basic video for Mac OS X, using an unholy amalgamation of QuickTime,
- * QuickDraw, and Quartz/Core Graphics.
+ * QuickDraw, Quartz/Core Graphics, and undocumented DVD playback APIs.
  *
  * = POSSIBLE ENHANCEMENTS
  * - Expand choices for the possibility of multiple displays
@@ -17,7 +17,7 @@
  *   a second time, may cause a crash (backtraces appreciated)
  * 
  * = REVISION
- * $Id: videoout_quartz.cpp,v 1.11 2005/04/07 15:33:50 danielk Exp $
+ * $Id: videoout_quartz.cpp 13053 2007-03-15 22:34:28Z nigel $
  *
  * = AUTHORS
  * Nigel Pearson, Jeremiah Morris
@@ -49,12 +49,21 @@ using namespace std;
 #include "filtermanager.h"
 #include "videoout_quartz.h"
 
+#include "util-osx.h"
+
 #import <CoreGraphics/CGBase.h>
 #import <CoreGraphics/CGDisplayConfiguration.h>
 #import <CoreGraphics/CGImage.h>
 #import <Carbon/Carbon.h>
 #import <QuickTime/QuickTime.h>
 
+#include "osd.h"
+#include "osdsurface.h"
+#include "mythconfig.h"
+#ifdef CONFIG_MAC_ACCEL
+#include "videoout_accel_utils.h"
+#endif
+
 class VideoOutputQuartzView;
 
 /*
@@ -85,6 +94,7 @@ struct QuartzData
 
     // Information about the display:
     WindowRef          window;            // MythTV window
+    Rect               windowBounds;      // dimensions, to restore size later
     CGDirectDisplayID  screen;            // screen containing main window
     float              refreshRate;       // for screen above
 
@@ -107,48 +117,6 @@ struct QuartzData
     VideoOutputQuartzView * embeddedView;    // special embedded widget
 };
 
-/*
- * These utility functions are used for querying parameters
- * from a CFDictionary.
- */
-// static int32_t getCFint32(CFDictionaryRef dict, CFStringRef key)
-// {
-//     CFNumberRef  ref = (CFNumberRef) CFDictionaryGetValue(dict, key);
-// 
-//     if (ref)
-//     {
-//         int32_t  val;
-// 
-//         if ( CFNumberGetValue(ref, kCFNumberSInt32Type, &val) )
-//             return val;
-//         else
-//             puts("getCFint32() - Failed to get 32bit int from value");
-//     }
-//     else
-//         puts("getCFint32() - Failed to get value");
-// 
-//     return 0;
-// }
-
-static double getCFdouble(CFDictionaryRef dict, CFStringRef key)
-{
-    CFNumberRef  ref = (CFNumberRef) CFDictionaryGetValue(dict, key);
-
-    if (ref)
-    {
-        double val;
-
-        if (CFNumberGetValue(ref, kCFNumberDoubleType, &val))
-            return val;
-        else
-            puts("getCFdouble() - Failed to get float from value");
-    }
-    else
-        puts("getCFdouble() - Failed to get value");
-
-    return 0.0;
-}
-
 /*
  * This abstract class is used for implementing output viewports.
  * See the subclasses below.
@@ -164,8 +132,9 @@ class VideoOutputQuartzView
     virtual void SetFrameSkip(int numskip);
     virtual void Show(void);
 
-    virtual void InputChanged(int width, int height, float aspect);
-    virtual void AspectChanged(float aspect);
+    virtual void InputChanged(int width, int height, float aspect,
+                              MythCodecID av_codec_id);
+    virtual void VideoAspectRatioChanged(float aspect);
     virtual void Zoom(int direction);
 
     virtual void EmbedChanged(bool embedded);
@@ -207,7 +176,7 @@ VideoOutputQuartzView::VideoOutputQuartzView(QuartzData *pData)
 
     thePort = NULL;
     desiredWidth = desiredHeight = desiredXoff = desiredYoff = 0;
-    theCodec = NULL;
+    theCodec = 0;
     theMask = NULL;
 
     frameSkip = 1;
@@ -303,7 +272,7 @@ void VideoOutputQuartzView::End(void)
     if (theCodec)
     {
         CDSequenceEnd(theCodec);
-        theCodec = NULL;
+        theCodec = 0;
         if (theMask)
         {
             DisposeRgn(theMask);
@@ -433,6 +402,15 @@ void VideoOutputQuartzView::Transform(void)
         TranslateMatrix(&matrix, X2Fix((w - sw) / 2.0), X2Fix((h - sh) / 2.0));
     }
 
+// apply the basic sizing to AccelUtils
+#ifdef CONFIG_MAC_ACCEL
+    AccelUtils *accel = AccelUtils::singleton();
+    if (accel)
+      accel->MoveResize(0, 0, parentData->srcWidth, parentData->srcHeight,
+                        (int)((w - sw) / 2.0), (int)((h - sh) / 2.0),
+                        sw, sh);
+#endif
+
     // apply over/underscan
     int hscan = gContext->GetNumSetting("HorizScanPercentage", 5);
     int vscan = gContext->GetNumSetting("VertScanPercentage", 5);
@@ -563,12 +541,14 @@ void VideoOutputQuartzView::Show(void)
     viewLock.lock();
     if (theCodec && thePort && parentData->pixmap)
     {
+      CodecFlags outFlags;
+
       // tell QuickTime to draw the current frame
       if (DecompressSequenceFrameWhen(theCodec,
                                       (Ptr)parentData->pixmap,
                                       parentData->pixmapSize,
                                       0,
-                                      NULL,
+                                      &outFlags,
                                       NULL,
                                       NULL))
       {
@@ -578,18 +558,20 @@ void VideoOutputQuartzView::Show(void)
     viewLock.unlock();
 }
 
-void VideoOutputQuartzView::InputChanged(int width, int height, float aspect)
+void VideoOutputQuartzView::InputChanged(int width, int height, float aspect,
+                                         MythCodecID av_codec_id)
 {
     (void)width;
     (void)height;
     (void)aspect;
+    (void)av_codec_id;
 
     // need to redo codec, but not the port
     End();
     Begin();
 }
 
-void VideoOutputQuartzView::AspectChanged(float aspect)
+void VideoOutputQuartzView::VideoAspectRatioChanged(float aspect)
 {
     (void)aspect;
 
@@ -1189,7 +1171,7 @@ VideoOutputQuartz::VideoOutputQuartz(void)
 {
     Started = 0; 
 
-    pauseFrame.buf = NULL;
+    init(&pauseFrame, FMT_YV12, NULL, 0, 0, 0, 0);
 
     data = new QuartzData();
     data->views.setAutoDelete(true);
@@ -1202,23 +1184,21 @@ VideoOutputQuartz::~VideoOutputQuartz()
     delete data;
 }
 
-void VideoOutputQuartz::AspectChanged(float aspect)
+void VideoOutputQuartz::VideoAspectRatioChanged(float aspect)
 {
     VERBOSE(VB_PLAYBACK,
-            QString("VideoOutputQuartz::AspectChanged(aspect=%1) [was %2]")
-                   .arg(aspect).arg(data->srcAspect));
+            QString("VideoOutputQuartz::VideoAspectRatioChanged"
+                    "(aspect=%1) [was %2]")
+            .arg(aspect).arg(data->srcAspect));
 
-    VideoOutput::AspectChanged(aspect);
-    MoveResize();
+    VideoOutput::VideoAspectRatioChanged(aspect);
 
     data->srcAspect = aspect;
-    data->srcMode = letterbox;
-    for (VideoOutputQuartzView *view = data->views.first();
-         view;
-         view = data->views.next())
-    {
-        view->AspectChanged(aspect);
-    }
+    data->srcMode   = db_letterbox;
+
+    VideoOutputQuartzView *view = NULL;
+    for (view = data->views.first(); view; view = data->views.next())
+        view->VideoAspectRatioChanged(aspect);
 }
 
 void VideoOutputQuartz::Zoom(int direction)
@@ -1228,9 +1208,9 @@ void VideoOutputQuartz::Zoom(int direction)
 
     VideoOutput::Zoom(direction);
     MoveResize();
-    data->ZoomedIn = ZoomedIn;
-    data->ZoomedUp = ZoomedUp;
-    data->ZoomedRight = ZoomedRight;
+    data->ZoomedIn    = mz_scale;
+    data->ZoomedUp    = mz_move.y();
+    data->ZoomedRight = mz_move.x();
 
     for (VideoOutputQuartzView *view = data->views.first();
          view;
@@ -1240,20 +1220,21 @@ void VideoOutputQuartz::Zoom(int direction)
     }
 }
 
-void VideoOutputQuartz::InputChanged(int width, int height, float aspect)
+void VideoOutputQuartz::InputChanged(int width, int height, float aspect,
+                                     MythCodecID av_codec_id)
 {
     VERBOSE(VB_PLAYBACK,
             QString("VideoOutputQuartz::InputChanged(width=%1, height=%2, aspect=%3")
                    .arg(width).arg(height).arg(aspect));
 
-    VideoOutput::InputChanged(width, height, aspect);
+    VideoOutput::InputChanged(width, height, aspect, av_codec_id);
 
     DeleteQuartzBuffers();
 
     data->srcWidth  = width;
     data->srcHeight = height;
     data->srcAspect = aspect;
-    data->srcMode   = letterbox;
+    data->srcMode   = db_letterbox;
 
     CreateQuartzBuffers();
 
@@ -1261,7 +1242,7 @@ void VideoOutputQuartz::InputChanged(int width, int height, float aspect)
          view;
          view = data->views.next())
     {
-        view->InputChanged(width, height, aspect);
+        view->InputChanged(width, height, aspect, av_codec_id);
     }
 
     MoveResize();    
@@ -1273,7 +1254,7 @@ int VideoOutputQuartz::GetRefreshRate(void)
             QString("VideoOutputQuartz::GetRefreshRate() [returning %1]")
                    .arg((int)data->refreshRate));
 
-    return (int)data->refreshRate;
+    return (int) (1000000 / data->refreshRate);
 }
 
 bool VideoOutputQuartz::Init(int width, int height, float aspect,
@@ -1292,16 +1273,16 @@ bool VideoOutputQuartz::Init(int width, int height, float aspect,
                    .arg(winh)
                    .arg(embedid));
 
-    VideoOutput::InitBuffers(kNumBuffers, true, kNeedFreeFrames, 
-                             kPrebufferFramesNormal, kPrebufferFramesSmall, 
-                             kKeepPrebuffer);
+    vbuffers.Init(kNumBuffers, true, kNeedFreeFrames, 
+                  kPrebufferFramesNormal, kPrebufferFramesSmall, 
+                  kKeepPrebuffer);
     VideoOutput::Init(width, height, aspect, winid,
                       winx, winy, winw, winh, embedid);
 
     data->srcWidth  = width;
     data->srcHeight = height;
     data->srcAspect = aspect;
-    data->srcMode   = letterbox;
+    data->srcMode   = db_letterbox;
     
     data->ZoomedIn = 0;
     data->ZoomedUp = 0;
@@ -1326,15 +1307,15 @@ bool VideoOutputQuartz::Init(int width, int height, float aspect,
     // dimensions and aspect ratio, but for simplicity we will just use the
     // display that contains the MythTV window.
 
-    Rect windowBounds;
-    if (GetWindowBounds(data->window, kWindowContentRgn, &windowBounds))
+    if (GetWindowBounds(data->window,
+                        kWindowStructureRgn, &(data->windowBounds)))
     {
         puts("GetWindowBounds failed");
         return false;
     }
     CGPoint pt;
-    pt.x = windowBounds.left;
-    pt.y = windowBounds.top;
+    pt.x = data->windowBounds.left;
+    pt.y = data->windowBounds.top;
     CGDisplayCount ct;
     data->screen = NULL;
     if (CGGetDisplaysWithPoint(pt, 1, &data->screen, &ct))
@@ -1346,9 +1327,10 @@ bool VideoOutputQuartz::Init(int width, int height, float aspect,
     // Find the refresh rate of our screen
     CFDictionaryRef m;
     m = CGDisplayCurrentMode(data->screen);
-    data->refreshRate = getCFdouble(m, kCGDisplayRefreshRate);
-    if (data->refreshRate == 0.0)	// LCD display?
-        data->refreshRate = 150;
+    data->refreshRate = get_float_CF(m, kCGDisplayRefreshRate);
+    if (data->refreshRate == 0.0)    // LCD display?
+        data->refreshRate = 150.0;   // Divisible by 25Hz and 30Hz
+                                     // to minimise AV sync waiting
 
     // Global configuration options
     data->scaleUpVideo = gContext->GetNumSetting("MacScaleUp", 1);
@@ -1357,7 +1339,7 @@ bool VideoOutputQuartz::Init(int width, int height, float aspect,
     data->correctGamma = gContext->GetNumSetting("MacGammaCorrect", 0);
     
     if (gContext->GetNumSetting("MacYuvConversion", 1))
-        data->yuvConverter = yuv2vuy_init_altivec();
+        data->yuvConverter = get_yuv2vuy_conv();
     else
         data->yuvConverter = NULL;
 
@@ -1393,6 +1375,13 @@ bool VideoOutputQuartz::Init(int width, int height, float aspect,
             tmp->SetFrameSkip(gContext->GetNumSetting("MacMainSkip", 0));
             data->views.append(tmp);
         }
+        else
+        {   
+            // If video in the main window is not enabled,
+            // hide (shrink) it so it is not in the way 
+            VERBOSE(VB_PLAYBACK, QString("Shrinking Main Window to 1x1"));
+            SizeWindow(data->window, 1, 1, true); 
+        } 
         if (gContext->GetNumSetting("MacFloatEnabled", 0))
         {
             float opacity =
@@ -1433,29 +1422,19 @@ bool VideoOutputQuartz::Init(int width, int height, float aspect,
 
 bool VideoOutputQuartz::CreateQuartzBuffers(void)
 {
-    for (int i = 0; i < numbuffers + 1; i++)
-    {
-        vbuffers[i].width  = XJ_width;
-        vbuffers[i].height = XJ_height;
-        vbuffers[i].bpp    = 12;
-        vbuffers[i].size   = XJ_width * XJ_height * vbuffers[i].bpp / 8;
-        vbuffers[i].codec  = FMT_YV12;
-
-        vbuffers[i].buf = new unsigned char[vbuffers[i].size + 64];
-    }
-
-    // Set up pause and scratch frames
-    scratchFrame = &(vbuffers[kNumBuffers]);
+    vbuffers.CreateBuffers(video_dim.width(), video_dim.height());
 
+    // Set up pause frame
     if (pauseFrame.buf)
         delete [] pauseFrame.buf;
 
-    pauseFrame.height = scratchFrame->height;
-    pauseFrame.width  = scratchFrame->width;
-    pauseFrame.bpp    = scratchFrame->bpp;
-    pauseFrame.size   = scratchFrame->size;
-    pauseFrame.buf    = new unsigned char[pauseFrame.size];
-    pauseFrame.frameNumber = scratchFrame->frameNumber;
+    VideoFrame *scratch = vbuffers.GetScratchFrame();
+
+    init(&pauseFrame, FMT_YV12, new unsigned char[scratch->size], 
+         scratch->width, scratch->height, scratch->bpp, scratch->size);
+
+    pauseFrame.frameNumber = scratch->frameNumber;
+
 
     // Set up pixel storage and image description for source
     data->pixelLock.lock();
@@ -1474,7 +1453,11 @@ bool VideoOutputQuartz::CreateQuartzBuffers(void)
     ImageDescription *desc = *data->imgDesc;
 
     desc->idSize = sizeof(ImageDescription);
+#ifdef WORDS_BIGENDIAN
     desc->cType = kYUV420CodecType;
+#else
+    desc->cType = kComponentVideoCodecType;  // Wrong, but prevents Intel crash
+#endif
     desc->version = 1;
     desc->revisionLevel = 0;
     desc->spatialQuality = codecNormalQuality;
@@ -1507,22 +1490,21 @@ bool VideoOutputQuartz::CreateQuartzBuffers(void)
     else
     {
         // YUV420 uses a descriptive header
-        data->pixelSize = (width * height * 3) / 2;
-        data->pixmapSize = sizeof(PlanarPixmapInfoYUV420) + data->pixelSize;
+        uint hdrSize = sizeof(PlanarPixmapInfoYUV420);
+
+        data->pixelSize  = scratch->size;
+        data->pixmapSize = hdrSize + data->pixelSize;
         data->pixmap = (PlanarPixmapInfoYUV420 *) new char[data->pixmapSize];
-        data->pixelData = &(data->pixmap[1]);
-    
-        long offset = sizeof(PlanarPixmapInfoYUV420);
-        data->pixmap->componentInfoY.offset = offset;
-        data->pixmap->componentInfoY.rowBytes = width;
-    
-        offset += width * height;
-        data->pixmap->componentInfoCb.offset = offset;
-        data->pixmap->componentInfoCb.rowBytes = width / 2;
-    
-        offset += (width * height) / 4;
-        data->pixmap->componentInfoCr.offset = offset;
-        data->pixmap->componentInfoCr.rowBytes = width / 2;
+
+        // Jump past the header:
+        data->pixelData = (char *)data->pixmap + hdrSize;
+
+        data->pixmap->componentInfoY.offset    = scratch->offsets[0] + hdrSize;
+        data->pixmap->componentInfoY.rowBytes  = scratch->pitches[0];
+        data->pixmap->componentInfoCb.offset   = scratch->offsets[1] + hdrSize;
+        data->pixmap->componentInfoCb.rowBytes = scratch->pitches[1];
+        data->pixmap->componentInfoCr.offset   = scratch->offsets[2] + hdrSize;
+        data->pixmap->componentInfoCr.rowBytes = scratch->pitches[2];
     }
     
     data->pixelLock.unlock();
@@ -1536,6 +1518,18 @@ void VideoOutputQuartz::Exit(void)
     {
         Started = false;
 
+        // Restore main window
+        // (assuming it was shrunk i.e. we were not in full screen mode)
+        if (data->windowedMode)
+        { 
+            VERBOSE(VB_PLAYBACK,
+                    QString("Restoring Main Window to %1x%2")  
+                    .arg(data->windowBounds.right - data->windowBounds.left)
+                    .arg(data->windowBounds.bottom - data->windowBounds.top));
+            SetWindowBounds(data->window, kWindowStructureRgn,
+                            &(data->windowBounds));
+        } 
+
         data->views.clear();
         DeleteQuartzBuffers();
     }
@@ -1560,13 +1554,12 @@ void VideoOutputQuartz::DeleteQuartzBuffers()
     data->pixelLock.unlock();
 
     if (pauseFrame.buf)
-        delete [] pauseFrame.buf;
-
-    for (int i = 0; i < numbuffers + 1; i++)
     {
-        delete [] vbuffers[i].buf;
-        vbuffers[i].buf = NULL;
+        delete [] pauseFrame.buf;
+        init(&pauseFrame, FMT_YV12, NULL, 0, 0, 0, 0);
     }
+
+    vbuffers.DeleteBuffers();
 }
 
 void VideoOutputQuartz::EmbedInWidget(WId wid, int x, int y, int w, int h)
@@ -1639,16 +1632,29 @@ void VideoOutputQuartz::PrepareFrame(VideoFrame *buffer, FrameScanType t)
 {
     (void)t;
 
-    if (!buffer)
-        buffer = scratchFrame;
+#ifdef CONFIG_MAC_ACCEL
+    AccelUtils *accel = AccelUtils::singleton();
+    if (accel && buffer)
+        accel->DecodeFrame(buffer);
+#endif // CONFIG_MAC_ACCEL
 
-    framesPlayed = buffer->frameNumber + 1;
+    if (buffer)
+        framesPlayed = buffer->frameNumber + 1;
 }
 
 void VideoOutputQuartz::Show(FrameScanType t)
 {
     (void)t;
 
+#ifdef CONFIG_MAC_ACCEL
+    AccelUtils *accel = AccelUtils::singleton();
+    if (accel)
+    {
+        accel->ShowFrame();
+        return;
+    }
+#endif // CONFIG_MAC_ACCEL
+
     data->pixelLock.lock();
     for (VideoOutputQuartzView *view = data->views.first();
          view;
@@ -1665,10 +1671,18 @@ void VideoOutputQuartz::DrawUnusedRects(bool)
 
 void VideoOutputQuartz::UpdatePauseFrame(void)
 {
-    VideoFrame *pauseb = scratchFrame;
-    if (usedVideoBuffers.count() > 0)
-        pauseb = usedVideoBuffers.head();
-    memcpy(pauseFrame.buf, pauseb->buf, pauseb->size);
+    if (!pauseFrame.buf)
+    {
+        puts("VideoOutputQuartz::UpdatePauseFrame() - no buffers?");
+        return;
+    }
+
+    VideoFrame *pauseb = vbuffers.GetScratchFrame();
+    VideoFrame *pauseu = vbuffers.head(kVideoBuffer_used);
+    if (pauseu)
+        memcpy(pauseFrame.buf, pauseu->buf, pauseu->size);
+    else
+        memcpy(pauseFrame.buf, pauseb->buf, pauseb->size);
 }
 
 void VideoOutputQuartz::ProcessFrame(VideoFrame *frame, OSD *osd,
@@ -1677,10 +1691,31 @@ void VideoOutputQuartz::ProcessFrame(VideoFrame *frame, OSD *osd,
 {
     if (!frame)
     {
-        frame = scratchFrame;
-        CopyFrame(scratchFrame, &pauseFrame);
+        frame = vbuffers.GetScratchFrame();
+        CopyFrame(vbuffers.GetScratchFrame(), &pauseFrame);
     }
 
+#ifdef CONFIG_MAC_ACCEL
+    AccelUtils *accel = AccelUtils::singleton();
+    if (accel)
+    {
+        if (osd && osd->Visible())
+        {
+            OSDSurface *surface = osd->Display();
+            if (surface && surface->Changed())
+            {
+              accel->DrawOSD(surface->y, surface->u, surface->v,
+                             surface->alpha);
+            }
+        }
+        else
+        {
+          accel->DrawOSD(NULL, NULL, NULL, NULL);
+        }
+        return;   // no need to process frame, it won't be used
+    }
+#endif // CONFIG_MAC_ACCEL
+
     if (filterList)
         filterList->ProcessFrame(frame);
 
@@ -1707,13 +1742,14 @@ void VideoOutputQuartz::ProcessFrame(VideoFrame *frame, OSD *osd,
     {
         if (data->yuvConverter)
         {
-            int frameSize = frame->width * frame->height;
             data->yuvConverter((uint8_t *)(data->pixelData),
-                               frame->buf,
-                               &frame->buf[frameSize],
-                               &frame->buf[frameSize * 5 / 4],
+                               frame->buf + frame->offsets[0], // Y
+                               frame->buf + frame->offsets[1], // U
+                               frame->buf + frame->offsets[2], // V
                                frame->width, frame->height,
                                (frame->width % 2), (frame->width % 2), 0);
+            // FIXME - These values (stride) should be calculated
+            //         from frame->pitches and frame->width ?
         }
         else
             memcpy(data->pixelData, frame->buf, frame->size);
