@@ -18,6 +18,8 @@
 #include <qfile.h>
 #include <qmap.h>
 #include <qdir.h>
+#include <qprocess.h>
+#include <qdatetime.h>
 
 // MythTV headers
 #include "mythconfig.h"
@@ -103,7 +105,7 @@ class XMLTVGrabber: public ComboBoxSetting, public VSSetting
     XMLTVGrabber(const VideoSource& parent)
       : VSSetting(parent, "xmltvgrabber")
     {
-        setLabel(QObject::tr("XMLTV listings grabber"));
+        setLabel(QObject::tr("Listings grabber"));
     };
 };
 
@@ -147,7 +149,9 @@ class DataDirectUserID: public LineEditSetting, public VSSetting
 class DataDirectPassword: public LineEditSetting, public VSSetting {
   public:
     DataDirectPassword(const VideoSource& parent):
-        VSSetting(parent, "password") {
+        VSSetting(parent, "password")
+    {
+        SetPasswordEcho(true);
         setLabel(QObject::tr("Password"));
     };
 };
@@ -158,7 +162,6 @@ void DataDirectLineupSelector::fillSelections(const QString &uid,
 {
     (void) uid;
     (void) pwd;
-    (void) _source;
 #ifdef USING_BACKEND
     if (uid.isEmpty() || pwd.isEmpty())
         return;
@@ -199,8 +202,11 @@ void DataDirectLineupSelector::fillSelections(const QString &uid,
 void DataDirect_config::load() 
 {
     VerticalConfigurationGroup::load();
-    if ((userid->getValue() != lastloadeduserid) || 
-        (password->getValue() != lastloadedpassword)) 
+    bool is_sd_userid = userid->getValue().contains("@") > 0;
+    bool match = ((is_sd_userid  && (source == DD_SCHEDULES_DIRECT)) ||
+                  (!is_sd_userid && (source == DD_ZAP2IT)));
+    if (((userid->getValue() != lastloadeduserid) ||
+         (password->getValue() != lastloadedpassword)) && match)
     {
         lineupselector->fillSelections(userid->getValue(), 
                                        password->getValue(),
@@ -295,15 +301,7 @@ void XMLTV_generic_config::save()
         "instead of just 'mythfilldatabase'.\nYour grabber does not provide "
         "channel numbers, so you have to set them manually.");
 
-    if (grabber == "tv_grab_de_tvtoday" || grabber == "tv_grab_se_swedb" || 
-        grabber == "tv_grab_fi" || grabber == "tv_grab_es" ||
-        grabber == "tv_grab_es_laguiatv" ||
-        grabber == "tv_grab_nl" || grabber == "tv_grab_jp" ||
-        grabber == "tv_grab_no" || grabber == "tv_grab_pt" ||
-        grabber == "tv_grab_ee" || grabber == "tv_grab_be_tvb" ||
-        grabber == "tv_grab_be_tlm" || grabber == "tv_grab_is" ||
-        grabber == "tv_grab_br" || grabber == "tv_grab_cz" ||
-        grabber == "tv_grab_il" || grabber == "tv_grab_ru")
+    if (is_grabber_external(grabber))
     {
         VERBOSE(VB_IMPORTANT, "\n" << err_msg);
         MythPopupBox::showOkPopup(
@@ -370,75 +368,61 @@ XMLTVConfig::XMLTVConfig(const VideoSource& parent) :
 
     // only save settings for the selected grabber
     setSaveAll(false);
- 
-    addTarget("datadirect", new DataDirect_config(parent));
-    grabber->addSelection("North America (DataDirect)", "datadirect");
-    
+
+    addTarget("schedulesdirect1",
+              new DataDirect_config(parent, DD_SCHEDULES_DIRECT));
+    grabber->addSelection("North America (SchedulesDirect.org) "
+                          "(Internal)", "schedulesdirect1");
+
+#if 1
+    addTarget("datadirect", new DataDirect_config(parent, DD_ZAP2IT));
+    grabber->addSelection(
+        "North America (TMS Labs) (Internal)", "datadirect");
+#endif
+
     addTarget("eitonly", new EITOnly_config(parent));
     grabber->addSelection("Transmitted guide only (EIT)", "eitonly");
 
-    addTarget("tv_grab_de_tvtoday", new XMLTV_generic_config(parent, "tv_grab_de_tvtoday"));
-    grabber->addSelection("Germany (tvtoday)", "tv_grab_de_tvtoday");
+    QProcess find_grabber_proc( QString("tv_find_grabbers"), this );
+    find_grabber_proc.addArgument("baseline");
+    find_grabber_proc.addArgument("manualconfig");
+    if ( find_grabber_proc.start() )
+    {
 
-    addTarget("tv_grab_se_swedb", new XMLTV_generic_config(parent, "tv_grab_se_swedb"));
-    grabber->addSelection("Sweden (tv.swedb.se)","tv_grab_se_swedb");
+        int i=0;
+        // Assume it shouldn't take more than 25 seconds
+        // Broken versions of QT cause QProcess::start
+        // and QProcess::isRunning to return true even
+        // when the executable doesn't exist
+        while (find_grabber_proc.isRunning() && i < 250)
+        {
+            usleep(100000);
+            ++i;
+        }
 
-    addTarget("tv_grab_no", new XMLTV_generic_config(parent, "tv_grab_no"));
-    grabber->addSelection("Norway","tv_grab_no");
-
-    addTarget("tv_grab_uk_rt", new XMLTV_generic_config(parent, "tv_grab_uk_rt"));
-    grabber->addSelection("United Kingdom (alternative)","tv_grab_uk_rt");
-
-    addTarget("tv_grab_au", new XMLTV_generic_config(parent, "tv_grab_au"));
-    grabber->addSelection("Australia", "tv_grab_au");
-
-    addTarget("tv_grab_fi", new XMLTV_generic_config(parent, "tv_grab_fi"));
-    grabber->addSelection("Finland", "tv_grab_fi");
-
-    addTarget("tv_grab_es", new XMLTV_generic_config(parent, "tv_grab_es"));
-    grabber->addSelection("Spain", "tv_grab_es");
-
-    addTarget("tv_grab_es_laguiatv", new XMLTV_generic_config(parent, "tv_grab_es_laguiatv"));
-    grabber->addSelection("Spain (Alt)", "tv_grab_es_laguiatv");
-
-    addTarget("tv_grab_nl", new XMLTV_generic_config(parent, "tv_grab_nl"));
-    grabber->addSelection("Holland", "tv_grab_nl");
-
-    addTarget("tv_grab_dk", new XMLTV_generic_config(parent, "tv_grab_dk"));
-    grabber->addSelection("Denmark", "tv_grab_dk");
-
-    addTarget("tv_grab_fr", new XMLTV_generic_config(parent, "tv_grab_fr"));
-    grabber->addSelection("France", "tv_grab_fr");
-
-    addTarget("tv_grab_jp", new XMLTV_generic_config(parent, "tv_grab_jp"));
-    grabber->addSelection("Japan", "tv_grab_jp");
-
-    addTarget("tv_grab_pt", new XMLTV_generic_config(parent, "tv_grab_pt"));
-    grabber->addSelection("Portugal", "tv_grab_pt");
-
-    addTarget("tv_grab_ee", new XMLTV_generic_config(parent, "tv_grab_ee"));
-    grabber->addSelection("Estonia", "tv_grab_ee");
-
-    addTarget("tv_grab_be_tvb", new XMLTV_generic_config(parent, "tv_grab_be_tvb"));
-    grabber->addSelection("Belgium (Dutch)", "tv_grab_be_tvb");
-
-    addTarget("tv_grab_be_tlm", new XMLTV_generic_config(parent, "tv_grab_be_tlm"));
-    grabber->addSelection("Belgium (French)", "tv_grab_be_tlm");
-
-    addTarget("tv_grab_is", new XMLTV_generic_config(parent, "tv_grab_is"));
-    grabber->addSelection("Iceland", "tv_grab_is");
-
-    addTarget("tv_grab_br", new XMLTV_generic_config(parent, "tv_grab_br"));
-    grabber->addSelection("Brazil", "tv_grab_br");
-
-    addTarget("tv_grab_cz", new XMLTV_generic_config(parent, "tv_grab_cz"));
-    grabber->addSelection("Czech Republic", "tv_grab_cz");
-
-    addTarget("tv_grab_il", new XMLTV_generic_config(parent, "tv_grab_il"));
-    grabber->addSelection("Israel", "tv_grab_il");
-
-    addTarget("tv_grab_ru", new XMLTV_generic_config(parent, "tv_grab_ru"));
-    grabber->addSelection("Russia", "tv_grab_ru");
+        if (find_grabber_proc.normalExit())
+        {
+            while (find_grabber_proc.canReadLineStdout())
+            {
+                QString grabber_list = find_grabber_proc.readLineStdout();
+                QStringList grabber_split = QStringList::split("|",
+                    grabber_list);
+                QString grabber_name = grabber_split[1] + " (xmltv)";
+                QFileInfo grabber_file(grabber_split[0]);
+                addTarget(grabber_file.fileName(),
+                    new XMLTV_generic_config(parent, grabber_file.fileName()));
+                grabber->addSelection(grabber_name, grabber_file.fileName());
+            }
+        }
+        else
+        {
+            VERBOSE(VB_IMPORTANT, "tv_find_grabbers exited early or we timed out waiting");
+        }
+    }
+    else
+    {
+        VERBOSE(VB_IMPORTANT, "Failed to run tv_find_grabbers");
+    }
 
     addTarget("/bin/true", new NoGrabber_config(parent));
     grabber->addSelection("No grabber", "/bin/true");
@@ -1477,26 +1461,32 @@ class InputName: public LabelSetting, public CISetting {
     };
 };
 
-class FreeToAir: public CheckBoxSetting, public CISetting {
+class FreeToAir: public CheckBoxSetting, public CISetting
+{
   public:
     FreeToAir(const CardInput& parent):
         CISetting(parent, "freetoaironly")
     {
         setValue(true);
-        setLabel(QObject::tr("Free to air channels only."));
-        setHelpText(QObject::tr("If set, only free to air channels will be "
-                    "used."));
+        setLabel(QObject::tr("Unencrypted channels only"));
+        setHelpText(QObject::tr(
+                        "If set, only unencrypted channels will be tuned to "
+                        "by MythTV or not be ignored by the MythTV channel "
+                        "scanner."));
     };
 };
 
-class RadioServices: public CheckBoxSetting, public CISetting {
-public:
+class RadioServices: public CheckBoxSetting, public CISetting
+{
+  public:
     RadioServices(const CardInput& parent):
         CISetting(parent, "radioservices")
     {
         setValue(true);
-        setLabel(QObject::tr("Radio channels."));
-        setHelpText(QObject::tr("If set, radio channels will also be included.")); 
+        setLabel(QObject::tr("Allow audio only channels"));
+        setHelpText(QObject::tr(
+                        "If set, audio only channels will not be ignored "
+                        "by the MythTV channel scanner.")); 
     };
 };
 
@@ -1597,7 +1587,7 @@ class DishNetEIT: public CheckBoxSetting, public CISetting
     };
 };
 
-CardInput::CardInput(bool isDVBcard, int _cardid)
+CardInput::CardInput(bool isDTVcard, bool isDVBcard, int _cardid)
 {
     (void) _cardid;
 
@@ -1621,28 +1611,30 @@ CardInput::CardInput(bool isDVBcard, int _cardid)
         group->addChild(new PresetTuner(*this));
     }
 
+    if (isDTVcard)
+    {
+        // we place this in a group just so the margins match the DVB ones.
+        ConfigurationGroup *chgroup = 
+            new HorizontalConfigurationGroup(false, false, true, true);
+        chgroup->addChild(new FreeToAir(*this));
+        group->addChild(chgroup);
+    }
+
 #ifdef USING_DVB
     if (isDVBcard)
     {
-        ConfigurationGroup *dvbgroup =
-            new HorizontalConfigurationGroup();
-        dvbgroup->setLabel(QObject::tr("DVB options"));
-
-        ConfigurationGroup *chgroup = 
-            new VerticalConfigurationGroup(false, false, true, true);
-
         TransButtonSetting *diseqc = new TransButtonSetting();
         diseqc->setLabel(tr("DVB-S"));
         diseqc->setHelpText(tr("Input and satellite settings."));
         diseqc->setVisible(DTVDeviceNeedsConfiguration(_cardid));
-        dvbgroup->addChild(diseqc);
+        group->addChild(diseqc);
         connect(diseqc, SIGNAL(pressed()), SLOT(diseqcConfig()));
    
-        chgroup->addChild(new FreeToAir(*this));
+        ConfigurationGroup *chgroup = 
+            new HorizontalConfigurationGroup(false, false, true, true);
         chgroup->addChild(new RadioServices(*this));
         chgroup->addChild(new DishNetEIT(*this));
-        dvbgroup->addChild(chgroup);
-        group->addChild(dvbgroup);
+        group->addChild(chgroup);
     }
 #endif
 
@@ -2079,6 +2071,30 @@ void VideoSourceEditor::del()
     if (val == 0)
     {
         MSqlQuery query(MSqlQuery::InitCon());
+
+        // Delete the channels associated with the source
+        query.prepare("DELETE FROM channel "
+                      "WHERE sourceid = :SOURCEID");
+        query.bindValue(":SOURCEID", getValue());
+
+        if (!query.exec() || !query.isActive())
+        {
+            MythContext::DBError("Deleting Channels", query);
+            return;
+        }
+
+        // Delete the inputs associated with the source
+        query.prepare("DELETE FROM cardinput "
+                      "WHERE sourceid = :SOURCEID");
+        query.bindValue(":SOURCEID", getValue());
+
+        if (!query.exec() || !query.isActive())
+        {
+            MythContext::DBError("Deleting cardinputs", query);
+            return;
+        }
+
+        // Delete the source itself
         query.prepare("DELETE FROM videosource "
                       "WHERE sourceid = :SOURCEID");
         query.bindValue(":SOURCEID", getValue());
