@@ -1,40 +1,36 @@
 #ifndef VIDEOSOURCE_H
 #define VIDEOSOURCE_H
 
-#include "libmyth/settings.h"
-#include <qregexp.h>
 #include <vector>
-#include <qdir.h>
-#include <qstringlist.h>
+using namespace std;
 
-#include "channelsettings.h"
+#include "settings.h"
 #include "datadirect.h"
+#include "diseqcsettings.h"
 
-#ifdef USING_DVB
-#include "dvbchannel.h"
-#endif
-
-class CardUtil
-{
-  public:
-    enum DVB_TYPES {ERROR_OPEN, ERROR_PROBE, QPSK, QAM, OFDM, ATSC};
-    enum DISEQC_TYPES { SINGLE=0, MINI_2, SWITCH_2_1_0, SWITCH_2_1_1,
-                        SWITCH_4_1_0, SWITCH_4_1_1, POSITIONER_1_2,
-                        POSITIONER_X, POSITIONER_1_2_SWITCH_2,
-                        POSITIONER_X_SWITCH_2 };
-    static const QString DVB;
-
-    static bool isCardPresent(QSqlDatabase *db, const QString &strType);
-    static DVB_TYPES cardDVBType(unsigned nVideoDev);
-    static DVB_TYPES cardDVBType(unsigned nVideoDev, QString &name);
-    static int CardUtil::videoDeviceFromCardID(QSqlDatabase* db,
-                                               unsigned nCardID);
-    static bool CardUtil::isDVB(QSqlDatabase *db, unsigned nCardID);
-    static DISEQC_TYPES CardUtil::diseqcType(QSqlDatabase *db,
-                                             unsigned nCardID);
-};
-
+class SignalTimeout;
+class ChannelTimeout;
+class UseEIT;
 class VideoSource;
+
+static inline bool is_grabber_external(const QString &grabber)
+{
+    return !(grabber == "datadirect" ||
+             grabber == "eitonly" ||
+             grabber == "schedulesdirect1" ||
+             grabber == "/bin/true");
+}
+
+static inline bool is_grabber_datadirect(const QString &grabber)
+{
+    return (grabber == "datadirect") || (grabber == "schedulesdirect1");
+}
+
+static inline bool is_grabber_labs(const QString &grabber)
+{
+    return grabber == "datadirect";
+}
+
 class VSSetting: public SimpleDBStorage {
 protected:
     VSSetting(const VideoSource& _parent, QString name):
@@ -43,37 +39,12 @@ protected:
         setName(name);
     };
 
-    virtual QString setClause(void);
-    virtual QString whereClause(void);
+    virtual QString setClause(MSqlBindings& bindings);
+    virtual QString whereClause(MSqlBindings& bindings);
 
     const VideoSource& parent;
 };
 
-class RegionSelector: public ComboBoxSetting, public TransientStorage {
-    Q_OBJECT
-public:
-    RegionSelector() {
-        setLabel(QObject::tr("Region"));
-        fillSelections();
-    };
-
-public slots:
-    void fillSelections();
-};
-
-class ProviderSelector: public ComboBoxSetting, public TransientStorage {
-    Q_OBJECT
-public:
-    ProviderSelector(const QString& _grabber) :
-        grabber(_grabber) { setLabel(QObject::tr("Provider")); };
-
-public slots:
-    void fillSelections(const QString& location);
-
-protected:
-    QString grabber;
-};
-
 class FreqTableSelector: public ComboBoxSetting, public VSSetting {
     Q_OBJECT
 public:
@@ -91,7 +62,7 @@ public:
    };
 
  public slots:
-    void fillSelections(const QString& uid, const QString& pwd);
+    void fillSelections(const QString& uid, const QString& pwd, int source);
 };
 
 class DataDirectButton: public ButtonSetting, public TransientStorage {
@@ -105,52 +76,66 @@ public:
 class DataDirectUserID;
 class DataDirectPassword;
 
-class DataDirect_config: public VerticalConfigurationGroup {
+class DataDirect_config: public VerticalConfigurationGroup
+{
     Q_OBJECT
-public:
-    DataDirect_config(const VideoSource& _parent); 
+  public:
+    DataDirect_config(const VideoSource& _parent, int _ddsource); 
 
-    virtual void load(QSqlDatabase* db);
+    virtual void load(void);
 
     QString getLineupID(void) const { return lineupselector->getValue(); };
 
-protected slots:
-    void fillDataDirectLineupSelector();
+  protected slots:
+    void fillDataDirectLineupSelector(void);
 
-protected:
-    const VideoSource& parent;
-    DataDirectUserID* userid;
-    DataDirectPassword* password;
-    DataDirectLineupSelector* lineupselector;
-    DataDirectButton* button;
+  protected:
+    const VideoSource        &parent;
+    DataDirectUserID         *userid;
+    DataDirectPassword       *password;
+    DataDirectButton         *button;
+    DataDirectLineupSelector *lineupselector;
     QString lastloadeduserid;
     QString lastloadedpassword;
+    int source;
 };
 
-class XMLTV_uk_config: public VerticalConfigurationGroup {
-public:
-    XMLTV_uk_config(const VideoSource& _parent);
-
-    virtual void save(QSqlDatabase* db);
-
-protected:
-    const VideoSource& parent;
-    RegionSelector* region;
-    ProviderSelector* provider;
-};
-
-class XMLTV_generic_config: public LabelSetting {
+class XMLTV_generic_config: public VerticalConfigurationGroup 
+{
 public:
     XMLTV_generic_config(const VideoSource& _parent, QString _grabber);
 
-    virtual void load(QSqlDatabase* db) { (void)db; };
-    virtual void save(QSqlDatabase* db);
+    virtual void save();
+    virtual void save(QString) { save(); }
 
 protected:
     const VideoSource& parent;
     QString grabber;
 };
 
+class EITOnly_config: public VerticalConfigurationGroup
+{
+public:
+    EITOnly_config(const VideoSource& _parent);
+    virtual void save();
+    virtual void save(QString) { save(); }
+
+protected:
+    UseEIT *useeit;
+};
+
+class NoGrabber_config: public VerticalConfigurationGroup
+{
+public:
+    NoGrabber_config(const VideoSource& _parent);
+
+    virtual void save();
+    virtual void save(QString) { save(); }
+
+protected:
+    UseEIT *useeit;
+};
+
 class XMLTVConfig: public VerticalConfigurationGroup, 
                    public TriggeredConfigurationGroup {
 public:
@@ -163,16 +148,20 @@ public:
         
     int getSourceID(void) const { return id->intValue(); };
 
-    void loadByID(QSqlDatabase* db, int id);
+    void loadByID(int id);
 
-    static void fillSelections(QSqlDatabase* db, SelectSetting* setting);
-    static QString idToName(QSqlDatabase* db, int id);
+    static void fillSelections(SelectSetting* setting);
+    static QString idToName(int id);
 
     QString getSourceName(void) const { return name->getValue(); };
 
-    virtual void save(QSqlDatabase* db) {
+    virtual void save() {
         if (name)
-            ConfigurationWizard::save(db);
+            ConfigurationWizard::save();
+    };
+    virtual void save(QString destination) {
+        if (name)
+            ConfigurationWizard::save(destination);
     };
 
 private:
@@ -201,11 +190,8 @@ private:
     };
 
 private:
-    ID* id;
-    Name* name;
-
-protected:
-    QSqlDatabase* db;
+    ID   *id;
+    Name *name;
 };
 
 class CaptureCard;
@@ -220,21 +206,52 @@ protected:
     int getCardID(void) const;
 
 protected:
-    virtual QString setClause(void);
-    virtual QString whereClause(void);
+    virtual QString setClause(MSqlBindings& bindings);
+    virtual QString whereClause(MSqlBindings& bindings);
 private:
     const CaptureCard& parent;
 };
 
-class TunerCardInput: public ComboBoxSetting, public CCSetting {
+class TunerCardInput: public ComboBoxSetting, public CCSetting
+{
     Q_OBJECT
-public:
-    TunerCardInput(const CaptureCard& parent):
-        CCSetting(parent, "defaultinput") {
-        setLabel(QObject::tr("Default input"));
-    };
+  public:
+    TunerCardInput(const CaptureCard &parent,
+                   QString dev  = QString::null,
+                   QString type = QString::null);
 
-public slots:
+  public slots:
+    void fillSelections(const QString &device);
+
+  private:
+    QString last_device;
+    QString last_cardtype;
+    int     last_diseqct;
+};
+
+class HDHRCardInput: public TunerCardInput
+{
+    Q_OBJECT
+  public:
+    HDHRCardInput(const CaptureCard& parent) : TunerCardInput(parent)
+    {
+        addSelection("MPEG2TS");
+    }
+
+  public slots:
+    void fillSelections(const QString& device);
+};
+
+class CRCIpNetworkRecorderInput: public TunerCardInput
+{
+    Q_OBJECT
+  public:
+    CRCIpNetworkRecorderInput(const CaptureCard& parent) : TunerCardInput(parent)
+    {
+        addSelection("MPEG2TS");
+    }
+
+  public slots:
     void fillSelections(const QString& device);
 };
 
@@ -246,10 +263,15 @@ public:
         setVisible(false);
     };
 
-    void save(QSqlDatabase* db) {
+    void save() {
         changed = true;
         settingValue = "";
-        SimpleDBStorage::save(db);
+        SimpleDBStorage::save();
+    };
+    void save(QString destination) {
+        changed = true;
+        settingValue = "";
+        SimpleDBStorage::save(destination);
     };
 };
 
@@ -260,44 +282,16 @@ public:
         CCSetting(parent,"vbidevice") {
         setVisible(false);
     };
-    void save(QSqlDatabase* db) {
+    void save() {
         changed = true;
         settingValue = "";
-        SimpleDBStorage::save(db);
+        SimpleDBStorage::save();
     };
-};
-
-class DVBDiseqcInputList
-{
-  public:
-    DVBDiseqcInputList() { clearValues(); }
-    DVBDiseqcInputList(const QString& in, const QString& prt, 
-                       const QString& pos) : input(in), port(prt), position(pos)
-    {}
-
-    void clearValues() 
-    {
-        input = "";
-        port = "";
-        position = "";
-    }
-
-    QString input;
-    QString port;
-    QString position;
-};
-
-class DVBDefaultInput: public ComboBoxSetting, public CCSetting {
-    Q_OBJECT
-public:
-
-    DVBDefaultInput(const CaptureCard& parent):
-        CCSetting(parent,"defaultinput") {
-        setLabel(QObject::tr("Default input"));
+    void save(QString destination) {
+        changed = true;
+        settingValue = "";
+        SimpleDBStorage::save(destination);
     };
-
-public slots:
-    void fillSelections(const QString& type);
 };
 
 class CardType: public ComboBoxSetting, public CCSetting {
@@ -306,32 +300,85 @@ public:
     static void fillSelections(SelectSetting* setting);
 };
 
+class VBIDevice;
+
+class V4LConfigurationGroup : public VerticalConfigurationGroup
+{
+   Q_OBJECT
+
+  public:
+     V4LConfigurationGroup(CaptureCard &parent);
+
+  public slots:
+    void probeCard(const QString &device);
+
+  private:
+    CaptureCard       &parent;
+    TransLabelSetting *cardinfo;
+    VBIDevice         *vbidev;
+    TunerCardInput    *input;
+};
+
+class MPEGConfigurationGroup: public VerticalConfigurationGroup
+{
+   Q_OBJECT
+
+  public:
+    MPEGConfigurationGroup(CaptureCard &parent);
+
+  public slots:
+    void probeCard(const QString &device);
+
+  private:
+    CaptureCard       &parent;
+    TransLabelSetting *cardinfo;
+    TunerCardInput    *input;
+};
+
+class pcHDTVConfigurationGroup: public VerticalConfigurationGroup
+{
+    Q_OBJECT
+
+  public:
+    pcHDTVConfigurationGroup(CaptureCard& a_parent);
+
+  public slots:
+    void probeCard(const QString &device);
+
+  private:
+    CaptureCard       &parent;
+    TransLabelSetting *cardinfo;
+    TunerCardInput    *input;
+};
+
+class DVBInput;
 class DVBCardName;
 class DVBCardType;
-class DVBDiseqcType;
+class DVBTuningDelay;
 
 class DVBConfigurationGroup: public VerticalConfigurationGroup {
     Q_OBJECT
 public:
     DVBConfigurationGroup(CaptureCard& a_parent);
 
-    void load(QSqlDatabase* _db) {
-        db = _db;
-        VerticalConfigurationGroup::load(db);
-    };
-
+    virtual void load(void);
+    virtual void save(void);
+    
 public slots:
     void probeCard(const QString& cardNumber);
-
+    void DiSEqCPanel(void);
+    
 private:
-    CaptureCard& parent;
+    CaptureCard        &parent;
 
-    DVBDefaultInput* defaultinput;
-    QSqlDatabase *db;
-    DVBCardName* cardname;
-    DVBCardType* cardtype;
-    DVBDiseqcType* diseqctype;
-    TransButtonSetting *advcfg;
+    DVBInput           *defaultinput;
+    DVBCardName        *cardname;
+    DVBCardType        *cardtype;
+    SignalTimeout      *signal_timeout;
+    ChannelTimeout     *channel_timeout;
+    TransButtonSetting *buttonAnalog;
+    DVBTuningDelay     *tuning_delay;
+    DiSEqCDevTree       tree;
 };
 
 class CaptureCardGroup: public VerticalConfigurationGroup,
@@ -347,30 +394,24 @@ protected slots:
 class CaptureCard: public ConfigurationWizard {
     Q_OBJECT
 public:
-    CaptureCard();
+    CaptureCard(bool use_card_group = true);
 
-    int getCardID(void) const {
-        return id->intValue();
-    };
+    int  getCardID(void) const { return id->intValue(); }
 
-    QString getDvbCard() { return dvbCard; };
+    void loadByID(int id);
+    void setParentID(int id);
 
-    void loadByID(QSqlDatabase* db, int id);
+    static void fillSelections(SelectSetting* setting);
+    static void fillSelections(SelectSetting* setting, bool no_children);
 
-    static void fillSelections(QSqlDatabase* db, SelectSetting* setting);
-
-    void load(QSqlDatabase* _db) {
-        db = _db;
-        ConfigurationWizard::load(db);
-    };
+    void reload(void);
 
 public slots:
-    void execDVBConfigMenu();
-    void execDiseqcWiz();
-    void execAdvConfigWiz();
-    void setDvbCard(const QString& card) { dvbCard = card; };
+    void analogPanel();
+    void recorderOptionsPanel();
 
 private:
+
     class ID: virtual public IntegerSetting,
               public AutoIncrementStorage {
     public:
@@ -381,15 +422,24 @@ private:
         };
     };
 
+    class ParentID: public CCSetting
+    {
+      public:
+        ParentID(const CaptureCard &parent) : CCSetting(parent, "parentid")
+        {
+            setValue("0");
+            setVisible(false);
+        }
+    };
+
     class Hostname: public HostnameSetting, public CCSetting {
     public:
         Hostname(const CaptureCard& parent): CCSetting(parent, "hostname") {};
     };
 
 private:
-    ID* id;
-    QSqlDatabase* db;
-    QString dvbCard;
+    ID       *id;
+    ParentID *parentid;
 };
 
 class CardInput;
@@ -403,11 +453,11 @@ protected:
 
     int getInputID(void) const;
 
-    void fillSelections(QSqlDatabase* db);
+    void fillSelections();
 
 protected:
-    virtual QString setClause(void);
-    virtual QString whereClause(void);
+    virtual QString setClause(MSqlBindings& bindings);
+    virtual QString whereClause(MSqlBindings& bindings);
 private:
     const CardInput& parent;
 };
@@ -415,16 +465,15 @@ private:
 class CaptureCardEditor: public ListBoxSetting, public ConfigurationDialog {
     Q_OBJECT
 public:
-    CaptureCardEditor(QSqlDatabase* _db):
-        db(_db) {
+    CaptureCardEditor() {
         setLabel(tr("Capture cards"));
     };
 
     virtual MythDialog* dialogWidget(MythMainWindow* parent,
                                      const char* widgetName=0);
-    virtual int exec(QSqlDatabase* db);
-    virtual void load(QSqlDatabase* db);
-    virtual void save(QSqlDatabase* db) { (void)db; };
+    virtual int exec();
+    virtual void load();
+    virtual void save() { };
 
 public slots:
     void menu();
@@ -432,26 +481,24 @@ public slots:
     void del();
 
 protected:
-    QSqlDatabase* db;
 };
 
 class VideoSourceEditor: public ListBoxSetting, public ConfigurationDialog {
     Q_OBJECT
 public:
-    VideoSourceEditor(QSqlDatabase* _db):
-        db(_db) {
+    VideoSourceEditor() {
         setLabel(tr("Video sources"));
     };
 
     virtual MythDialog* dialogWidget(MythMainWindow* parent,
                                      const char* widgetName=0);
 
-    bool cardTypesInclude(QSqlDatabase* db, const int& SourceID, 
+    bool cardTypesInclude(const int& SourceID, 
                           const QString& thecardtype);
 
-    virtual int exec(QSqlDatabase* db);
-    virtual void load(QSqlDatabase* db);
-    virtual void save(QSqlDatabase* db) { (void)db; };
+    virtual int exec();
+    virtual void load();
+    virtual void save() { };
 
 public slots:
     void menu(); 
@@ -459,24 +506,93 @@ public slots:
     void del();
 
 protected:
-    QSqlDatabase* db;
 };
 
 class CardInputEditor: public ListBoxSetting, public ConfigurationDialog {
 public:
-    CardInputEditor(QSqlDatabase* _db):
-        db(_db) {
+    CardInputEditor() {
         setLabel(QObject::tr("Input connections"));
     };
     virtual ~CardInputEditor();
 
-    virtual int exec(QSqlDatabase* db);
-    virtual void load(QSqlDatabase* db);
-    virtual void save(QSqlDatabase* db) { (void)db; };
+    virtual int exec();
+    virtual void load();
+    virtual void save() { };
 
 protected:
     vector<CardInput*> cardinputs;
-    QSqlDatabase* db;
+};
+
+class StartingChannel : public ComboBoxSetting, public CISetting
+{
+    Q_OBJECT
+  public:
+    StartingChannel(const CardInput &parent) :
+        ComboBoxSetting(false, 1), CISetting(parent, "startchan")
+    {
+        setLabel(QObject::tr("Starting channel"));
+        setHelpText(QObject::tr("Starting LiveTV channel.") + " " +
+                    QObject::tr("This is updated on every successful "
+                                "channel change."));
+    }
+    void fillSelections(void) {;}
+  public slots:
+    void SetSourceID(const QString &sourceid);
+};
+
+class CardID;
+class ChildID;
+class InputName;
+class SourceID;
+
+class CardInput: public ConfigurationWizard
+{
+    Q_OBJECT
+  public:
+    CardInput(bool is_dtv_card, bool is_dvb_card, int cardid);
+
+    int getInputID(void) const { return id->intValue(); };
+
+    void loadByID(int id);
+    void loadByInput(int cardid, QString input);
+    QString getSourceName(void) const;
+
+    void SetChildCardID(uint);
+
+    virtual void save();
+    virtual void save(QString /*destination*/) { save(); }
+
+  public slots:
+    void channelScanner();
+    void sourceFetch();
+    void diseqcConfig();
+
+  private:
+    class ID: virtual public IntegerSetting,
+              public AutoIncrementStorage
+    {
+      public:
+        ID():
+            AutoIncrementStorage("cardinput", "cardid")
+        {
+            setVisible(false);
+            setName("CardInputID");
+        };
+        virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent,
+                                      const char* widgetName = 0)
+        {
+            (void)cg; (void)parent; (void)widgetName;
+            return NULL;
+        };
+    };
+
+    ID              *id;
+    CardID          *cardid;
+    ChildID         *childid;
+    InputName       *inputname;
+    SourceID        *sourceid;
+    StartingChannel *startchan;
+    DiSEqCDevSettings  settings;
 };
 
 #endif
