@@ -25,15 +25,18 @@
 
 #include <cstdio>
 #include <cstdlib>
+#include <algorithm>
 #include <inttypes.h>
 #include "config.h"
 
 #ifdef MMX
-#include "mmx.h"
+#include "libavcodec/i386/mmx.h"
 #define CPU_MMXEXT 0
 #define CPU_MMX 1
 #endif
+
 #ifdef HAVE_ALTIVEC
+extern "C" int has_altivec(void);    // in libavcodec/ppc/dsputil_altivec.c
 #ifdef HAVE_ALTIVEC_H
 #include <altivec.h>
 #else
@@ -42,6 +45,11 @@
 #endif
 #include "yuv2rgb.h"
 
+/** \file yuv2rgb.cpp
+ *  \brief Contains various YUV, VUY and RGBA colorspace conversion rutines.
+ *
+ */
+
 static void yuv420_argb32_non_mmx(unsigned char *image, unsigned char *py,
                            unsigned char *pu, unsigned char *pv,
                            int h_size, int v_size, int rgb_stride,
@@ -325,6 +333,15 @@ static void mmx_argb32 (uint8_t * image,
 }
 #endif
 
+/** \fn yuv2rgb_init_mmxext(int bpp, int mode)
+ *  \brief This returns a yuv to rgba converter, using
+ *          mmxext if MMX was compiled in.
+ *
+ *  \param mode must be MODE_RGB
+ *  \param bpp must be 32
+ *
+ *  \return function pointer or NULL if converter could not be found.
+ */
 yuv2rgb_fun yuv2rgb_init_mmxext (int bpp, int mode)
 {
 #ifdef MMX
@@ -340,6 +357,15 @@ yuv2rgb_fun yuv2rgb_init_mmxext (int bpp, int mode)
     return NULL; /* Fallback to C */
 }
 
+/** \fn yuv2rgb_init_mmx (int bpp, int mode)
+ *  \brief This returns a yuv to rgba converter, using
+ *         mmx if MMX was compiled in.
+ *
+ *  \param mode must be MODE_RGB
+ *  \param bpp must be 32
+ *
+ *  \return function pointer or NULL if converter could not be found.
+ */
 yuv2rgb_fun yuv2rgb_init_mmx (int bpp, int mode)
 {
 #ifdef MMX
@@ -366,9 +392,9 @@ yuv2rgb_fun yuv2rgb_init_mmx (int bpp, int mode)
 #define RGBOUT(r, g, b, y1)\
 {\
     y = (y1 - 16) * C_Y;\
-    r = (y + r_add) >> SCALE_BITS;\
-    g = (y + g_add) >> SCALE_BITS;\
-    b = (y + b_add) >> SCALE_BITS;\
+    r = std::min(UCHAR_MAX, std::max(0, (y + r_add) >> SCALE_BITS));\
+    g = std::min(UCHAR_MAX, std::max(0, (y + g_add) >> SCALE_BITS));\
+    b = std::min(UCHAR_MAX, std::max(0, (y + b_add) >> SCALE_BITS));\
 }
 
 static void yuv420_argb32_non_mmx(unsigned char *image, unsigned char *py,
@@ -463,11 +489,11 @@ void rgb32_to_yuv420p(unsigned char *lum, unsigned char *cb, unsigned char *cr,
 #define A_II  3
 #endif
 
-    wrap = width;
+    wrap = (width + 1) & ~1;
     wrap4 = srcwidth * 4;
     p = src;
-    for(y=0;y<height;y+=2) {
-        for(x=0;x<width;x+=2) {
+    for(y=0;y+1<height;y+=2) {
+        for(x=0;x+1<width;x+=2) {
             r = p[R_II];
             g = p[G_II];
             b = p[B_II];
@@ -525,12 +551,250 @@ void rgb32_to_yuv420p(unsigned char *lum, unsigned char *cb, unsigned char *cr,
             lum += -wrap + 2;
             alpha += -wrap + 2;
         }
-        p += wrap4;
+        if (width & 1) {
+            r = p[R_II];
+            g = p[G_II];
+            b = p[B_II];
+            r1 = r;
+            g1 = g;
+            b1 = b;
+            lum[0] = (FIX(0.29900) * r + FIX(0.58700) * g +
+                      FIX(0.11400) * b + ONE_HALF) >> SCALEBITS;
+            alpha[0] = p[A_II];
+
+            lum[1] = 16;
+            alpha[1] = 0;
+
+            p += wrap4;
+            lum += wrap;
+            alpha += wrap;
+
+            r = p[R_II];
+            g = p[G_II];
+            b = p[B_II];
+            r1 += r;
+            g1 += g;
+            b1 += b;
+            lum[0] = (FIX(0.29900) * r + FIX(0.58700) * g +
+                      FIX(0.11400) * b + ONE_HALF) >> SCALEBITS;
+            alpha[0] = p[A_II];
+
+            lum[1] = 16;
+            alpha[1] = 0;
+
+            cr[0] = ((- FIX(0.16874) * r1 - FIX(0.33126) * g1 +
+                    FIX(0.50000) * b1 + 2 * ONE_HALF - 1) >> (SCALEBITS + 1)) +
+                    128;
+            cb[0] = ((FIX(0.50000) * r1 - FIX(0.41869) * g1 -
+                    FIX(0.08131) * b1 + 2 * ONE_HALF - 1) >> (SCALEBITS + 1)) +
+                    128;
+
+            cb++;
+            cr++;
+            p += -wrap4 + 4;
+            lum += -wrap + 2;
+            alpha += -wrap + 2;
+        }
+        p += wrap4 * 2 - width * 4;
         lum += wrap;
         alpha += wrap;
     }
+    if (height & 1) {
+        for(x=0;x+1<width;x+=2) {
+            r = p[R_II];
+            g = p[G_II];
+            b = p[B_II];
+            r1 = r;
+            g1 = g;
+            b1 = b;
+            lum[0] = (FIX(0.29900) * r + FIX(0.58700) * g +
+                      FIX(0.11400) * b + ONE_HALF) >> SCALEBITS;
+            alpha[0] = p[A_II];
+
+            r = p[R_II+4];
+            g = p[G_II+4];
+            b = p[B_II+4];
+            r1 += r;
+            g1 += g;
+            b1 += b;
+            lum[1] = (FIX(0.29900) * r + FIX(0.58700) * g +
+                      FIX(0.11400) * b + ONE_HALF) >> SCALEBITS;
+            alpha[1] = p[A_II+4];
+
+            lum += wrap;
+            alpha += wrap;
+
+            lum[0] = 16;
+            alpha[0] = 0;
+
+            lum[1] = 16;
+            alpha[1] = 0;
+
+            cr[0] = ((- FIX(0.16874) * r1 - FIX(0.33126) * g1 +
+                    FIX(0.50000) * b1 + 2 * ONE_HALF - 1) >> (SCALEBITS + 1)) +
+                    128;
+            cb[0] = ((FIX(0.50000) * r1 - FIX(0.41869) * g1 -
+                    FIX(0.08131) * b1 + 2 * ONE_HALF - 1) >> (SCALEBITS + 1)) +
+                    128;
+
+            cb++;
+            cr++;
+            p += 2 * 4;
+            lum += -wrap + 2;
+            alpha += -wrap + 2;
+        }
+        if (width & 1) {
+            r = p[R_II];
+            g = p[G_II];
+            b = p[B_II];
+            r1 = r;
+            g1 = g;
+            b1 = b;
+            lum[0] = (FIX(0.29900) * r + FIX(0.58700) * g +
+                      FIX(0.11400) * b + ONE_HALF) >> SCALEBITS;
+            alpha[0] = p[A_II];
+
+            lum[1] = 16;
+            alpha[1] = 0;
+
+            lum += wrap;
+            alpha += wrap;
+
+            lum[0] = 16;
+            alpha[0] = 0;
+
+            lum[1] = 16;
+            alpha[1] = 0;
+
+            cr[0] = ((- FIX(0.16874) * r1 - FIX(0.33126) * g1 +
+                    FIX(0.50000) * b1 + ONE_HALF - 1) >> SCALEBITS) +
+                    128;
+            cb[0] = ((FIX(0.50000) * r1 - FIX(0.41869) * g1 -
+                    FIX(0.08131) * b1 + ONE_HALF - 1) >> SCALEBITS) +
+                    128;
+
+            cb++;
+            cr++;
+            p += 4;
+            lum += -wrap + 2;
+            alpha += -wrap + 2;
+       }
+    }
 }
 
+/* YUV420 to VUY2 colorspace conversion routines.
+ *
+ * In the early days of the OS X port of MythTV, Paul Jara noticed that
+ * QuickTime spent a lot of time converting from YUV420 to YUV422.
+ * He found some sample code on the Ars Technica forum by a
+ * Frenchman called Titer which used Altivec to speed this up.
+ * Jeremiah Morris took that code and added it into MythTV.
+ *
+ * All was well until the Intel Macs came along,
+ * which seem to crash when fed YUV420 from MythTV.
+ *
+ * Fortunately, Mino Taoyama has provided an MMX optimised version too.
+ */
+
+// Plain C function which is used for non-Vector-compatible dimensions
+static void non_vec_yuv420_2vuy (uint8_t *, uint8_t *, uint8_t *,
+                                 uint8_t *, int, int, int, int, int);
+
+#ifdef MMX
+static void mmx_yuv420_2vuy (uint8_t * image, uint8_t * py,
+                             uint8_t * pu, uint8_t * pv,
+                             int h_size, int v_size,
+                             int vuy_stride, int y_stride, int uv_stride)
+{
+    uint8_t *ibuf1 = image;
+    uint8_t *ibuf2 = image;
+    uint8_t *ybuf1 = py;
+    uint8_t *ybuf2 = py;
+    uint8_t *ubuf = pu;
+    uint8_t *vbuf = pv;
+    
+    int x,y;
+    
+
+    if ((h_size % 16) || (v_size % 2))
+        return non_vec_yuv420_2vuy(image, py, pu, pv, h_size, v_size,
+                                   vuy_stride, y_stride, uv_stride);
+
+    for(y = v_size/2; y--; )
+    {
+        ibuf1 = ibuf2;
+        ibuf2 += h_size * 2;
+        ybuf1 = ybuf2;
+        ybuf2 += h_size;
+        
+        for(x = 0; x < h_size / 16; x++)
+        {
+            
+            movq_m2r (*ybuf1, mm0);   // y data
+            movq_m2r (*ybuf2, mm1);   // y data
+            movq_m2r (*ubuf, mm2);    // u data
+            movq_m2r (*vbuf, mm3);    // v data
+            
+            movq_r2r (mm2, mm4);      // Copy U
+            
+            punpcklbw_r2r (mm3, mm2); // Combine low U & V  mm2 = uv low
+            punpckhbw_r2r (mm3, mm4); // Combine high U & V mm4 = uv high
+            
+            movq_r2r (mm2, mm5);      // Copy low UV  mm5 = uv low
+            movq_r2r (mm2, mm6);      // Copy low UV  mm6 = uv low
+            punpcklbw_r2r (mm0, mm5); // mm5 = y1 low uv low
+            punpckhbw_r2r (mm0, mm6); // mm6 = y1 high uv high
+            
+            movntq_r2m (mm5, *(ibuf1));
+            movntq_r2m (mm6, *(ibuf1+8));
+    
+            movq_r2r (mm2, mm5);      // Copy low UV mm5 = uv low
+            movq_r2r (mm2, mm6);      // Copy low UV mm6 = uv low
+	    punpcklbw_r2r (mm1, mm5); // mm5 = y2 low uv low
+            punpckhbw_r2r (mm1, mm6); // mm6 = y2 high uv high
+    
+            movntq_r2m (mm5, *(ibuf2));
+            movntq_r2m (mm6, *(ibuf2+8));
+    
+    
+            movq_m2r (*(ybuf1+8), mm0); // y data
+            movq_m2r (*(ybuf2+8), mm1); // y data
+    
+            movq_r2r (mm4, mm5);      // Copy high UV mm5 = uv high
+            movq_r2r (mm4, mm6);      // Copy high UV mm6 = uv high
+            punpcklbw_r2r (mm0, mm5); // mm5 = y1 low uv high
+            punpckhbw_r2r (mm0, mm6); // mm6 = y1 high uv high
+            
+            movntq_r2m (mm5, *(ibuf1+16));
+            movntq_r2m (mm6, *(ibuf1+24));
+            
+            movq_r2r (mm4, mm5);      // Copy high UV mm5 = uv high
+            movq_r2r (mm4, mm6);      // Copy high UV mm6 = uv high     
+            punpcklbw_r2r (mm1, mm5); // mm5 = y2 low uv low
+            punpckhbw_r2r (mm1, mm6); // mm6 = y2 high uv high
+    
+            movntq_r2m (mm5, *(ibuf2+16));
+            movntq_r2m (mm6, *(ibuf2+24));
+            
+            ibuf1 += 32;
+            ibuf2 += 32;
+            ybuf1 += 16;
+            ybuf2 += 16;
+            ubuf += 8;
+            vbuf += 8;
+        }
+        ibuf2 += vuy_stride;
+        ybuf2 += y_stride;
+        ubuf += uv_stride;
+        vbuf += uv_stride;
+    }
+
+    emms();
+}
+
+#endif // MMX
+
+
 static void non_vec_yuv420_2vuy (uint8_t * image, uint8_t * py,
                                  uint8_t * pu, uint8_t * pv,
                                  int h_size, int v_size,
@@ -565,7 +829,9 @@ static void non_vec_yuv420_2vuy (uint8_t * image, uint8_t * py,
     }
 }
 
+
 #ifdef HAVE_ALTIVEC
+
 // Altivec code adapted from VLC's i420_yuv2.c (thanks to Titer and Paul Jara) 
 
 #define VEC_NEXT_LINES()                                                    \
@@ -602,7 +868,7 @@ static void altivec_yuv420_2vuy (uint8_t * image, uint8_t * py,
     vector unsigned char uv_vec;
     vector unsigned char y_vec;
 
-    if (!((h_size % 32) | (v_size % 2)))
+    if (!((h_size % 32) || (v_size % 2)))
     {
         // Width is a multiple of 32, process 2 lines at a time
         for (y = v_size / 2; y--; )
@@ -617,7 +883,7 @@ static void altivec_yuv420_2vuy (uint8_t * image, uint8_t * py,
         }
     
     }
-    else if (!((h_size % 16) | (v_size % 4)))
+    else if (!((h_size % 16) || (v_size % 4)))
     {
         // Width is a multiple of 16, process 4 lines at a time
         for (y = v_size / 4; y--; )
@@ -655,16 +921,37 @@ static void altivec_yuv420_2vuy (uint8_t * image, uint8_t * py,
                             vuy_stride, y_stride, uv_stride);
     }
 }
-#endif
 
-yuv2vuy_fun yuv2vuy_init_altivec (void)
+#endif // HAVE_ALTIVEC
+
+
+/** \fn     get_vuy2yuv_conv(void)
+ *  \return A pointer to a YUV to VUY conversion function,
+ *          which uses Altivec on appropriate machines,
+ *          or MMX if it was compiled in.
+ */
+yuv2vuy_fun get_yuv2vuy_conv(void)
 {
 #ifdef HAVE_ALTIVEC
-    return altivec_yuv420_2vuy;
+    if (has_altivec())
+        return altivec_yuv420_2vuy;
 #endif
+
+#ifdef MMX
+    return mmx_yuv420_2vuy;
+#endif
+
     return non_vec_yuv420_2vuy; /* Fallback to C */
 }
 
+
+
+/* 2VUY to YUV420 conversion routines
+ *
+ * For completeness, and as another Altivec assembler example.
+ * Note that we have no MMX version of this.
+ */
+
 static void non_vec_2vuy_yuv420 (uint8_t * image, uint8_t * py,
                                  uint8_t * pu, uint8_t * pv,
                                  int h_size, int v_size,
@@ -700,6 +987,7 @@ static void non_vec_2vuy_yuv420 (uint8_t * image, uint8_t * py,
 }
 
 #ifdef HAVE_ALTIVEC
+
 // Altivec code adapted from VLC's i420_yuv2.c (thanks to Titer and Paul Jara) 
 
 #define VEC_READ_LINE(ptr, y, uv)                                           \
@@ -740,7 +1028,7 @@ static void altivec_2vuy_yuv420 (uint8_t * image, uint8_t * py,
                          uv1_vec, uv2_vec,
                          uva_vec, uvb_vec;
 
-    if (!((h_size % 32) | (v_size % 2)))
+    if (!((h_size % 32) || (v_size % 2)))
     {
         // Width is a multiple of 32, process 2 lines at a time
         for (y = v_size / 2; y--; )
@@ -755,7 +1043,7 @@ static void altivec_2vuy_yuv420 (uint8_t * image, uint8_t * py,
         }
     
     }
-    else if (!((h_size % 16) | (v_size % 4)))
+    else if (!((h_size % 16) || (v_size % 4)))
     {
         // Width is a multiple of 16, process 4 lines at a time
         for (y = v_size / 4; y--; )
@@ -793,12 +1081,19 @@ static void altivec_2vuy_yuv420 (uint8_t * image, uint8_t * py,
                             vuy_stride, y_stride, uv_stride);
     }
 }
-#endif
 
-vuy2yuv_fun vuy2yuv_init_altivec (void)
+#endif // HAVE_ALTIVEC
+
+
+/** \fn     get_vuy2yuv_conv(void)
+ *  \return A pointer to a VUY to YUV conversion function,
+ *          which uses Altivec on appropriate machines.
+ */
+vuy2yuv_fun get_vuy2yuv_conv(void)
 {
 #ifdef HAVE_ALTIVEC
-    return altivec_2vuy_yuv420;
+    if (has_altivec())
+        return altivec_2vuy_yuv420;
 #endif
     return non_vec_2vuy_yuv420; /* Fallback to C */
 }
