@@ -34,10 +34,8 @@ using namespace std;
 #include "libmythtv/programinfo.h"
 #include "libmythtv/dbcheck.h"
 #include "libmythtv/jobqueue.h"
-#include "libmythupnp/upnp.h"
 
-#include "upnpcdstv.h"
-#include "upnpcdsmusic.h"
+#include "mediaserver.h"
 #include "httpstatus.h"
 
 QMap<int, EncoderLink *> tvList;
@@ -49,8 +47,7 @@ QString lockfile_location;
 HouseKeeper *housekeeping = NULL;
 QString logfile = "";
 
-HttpServer *g_pHttpServer = NULL;
-UPnp       *g_pUPnp       = NULL;
+MediaServer *g_pUPnp       = NULL;
 
 bool setupTVs(bool ismaster, bool &error)
 {
@@ -210,9 +207,6 @@ void cleanup(void)
     if (g_pUPnp)
         delete g_pUPnp;
 
-    if (g_pHttpServer)
-        delete g_pHttpServer;
-
     if (pidfile != "")
         unlink(pidfile.ascii());
 
@@ -268,6 +262,7 @@ int main(int argc, char **argv)
     bool testsched = false;
     bool resched = false;
     bool nosched = false;
+    bool noupnp = false;
     bool nojobqueue = false;
     bool nohousekeeper = false;
     bool noexpirer = false;
@@ -378,6 +373,10 @@ int main(int argc, char **argv)
         {
             nosched = true;
         } 
+        else if (!strcmp(a.argv()[argpos],"--noupnp"))
+        {
+            noupnp = true;
+        } 
         else if (!strcmp(a.argv()[argpos],"--nojobqueue"))
         {
             nojobqueue = true;
@@ -397,10 +396,12 @@ int main(int argc, char **argv)
         else if (!strcmp(a.argv()[argpos],"--version"))
         {
             extern const char *myth_source_version;
-            cout << "Library API version: " << MYTH_BINARY_VERSION << endl;
-            cout << "Source code version: " << myth_source_version << endl;
+            extern const char *myth_source_path;
+            cout << "Library API version     : " << MYTH_BINARY_VERSION << endl;
+            cout << "Source code version     : " << myth_source_version << endl;
+            cout << "SVN Branch              : " << myth_source_path << endl;
 #ifdef MYTH_BUILD_CONFIG
-            cout << "Options compiled in:" <<endl;
+            cout << "Options compiled in     :" <<endl;
             cout << MYTH_BUILD_CONFIG << endl;
 #endif
             return BACKEND_EXIT_OK;
@@ -422,6 +423,7 @@ int main(int argc, char **argv)
                     "--testsched                    Test run scheduler (ignore existing schedule)" << endl <<
                     "--resched                      Force the scheduler to update" << endl <<
                     "--nosched                      Do not perform any scheduling" << endl <<
+                    "--noupnp                       Do not enable the UPNP server" << endl <<
                     "--nojobqueue                   Do not start the JobQueue" << endl <<
                     "--nohousekeeper                Do not start the Housekeeper" << endl <<
                     "--noautoexpire                 Do not start the AutoExpire thread" << endl <<
@@ -548,7 +550,6 @@ int main(int argc, char **argv)
     }
 
     int port = gContext->GetNumSetting("BackendServerPort", 6543);
-    int statusport = gContext->GetNumSetting("BackendStatusPort", 6544);
 
     QString myip = gContext->GetSetting("BackendServerIP");
     QString masterip = gContext->GetSetting("MasterServerIP");
@@ -610,35 +611,13 @@ int main(int argc, char **argv)
     else
         jobqueue = new JobQueue(ismaster);
 
-    // Initialize & Start the Mini HttpServer
-    VERBOSE(VB_IMPORTANT, "Main::Starting HttpServer");
+    // Start UPnP Services 
 
-    g_pHttpServer = new HttpServer(statusport);
-
-    if (!g_pHttpServer->ok())
-    { 
-        VERBOSE(VB_IMPORTANT, "Main::HttpServer Create Error");
-        // exit(BACKEND_BUGGY_EXIT_NO_BIND_STATUS);
-    }
+    g_pUPnp = new MediaServer( ismaster, noupnp );
 
     VERBOSE(VB_IMPORTANT, "Main::Registering HttpStatus Extension");
 
-    g_pHttpServer->RegisterExtension(new HttpStatus(&tvList, sched, expirer, ismaster ));
-
-    // Start UPnP Services For Master Backends Only
-    if (ismaster)
-    {
-        g_pUPnp = new UPnp(ismaster, g_pHttpServer);
-
-        VERBOSE(VB_UPNP, "Main::Registering UPnpCDSTv Extension");
-
-        g_pUPnp->RegisterExtension(new UPnpCDSTv());
-
-        VERBOSE(VB_UPNP, "Main::Registering UPnpCDSMusic Extension");
-
-        g_pUPnp->RegisterExtension(new UPnpCDSMusic());
-    }
-    // End uPnP &  Mini HttpServer Initialization
+    g_pUPnp->GetHttpServer()->RegisterExtension(new HttpStatus(&tvList, sched, expirer, ismaster ));
 
     VERBOSE(VB_IMPORTANT, QString("%1 version: %2 www.mythtv.org")
                             .arg(binname).arg(MYTH_BINARY_VERSION));
@@ -666,7 +645,7 @@ int main(int argc, char **argv)
         }
     }
 
-    new MainServer(ismaster, port, statusport, &tvList, sched, expirer);
+    new MainServer(ismaster, port, &tvList, sched, expirer);
 
     if (ismaster)
     {
