@@ -9,25 +9,29 @@
 #include <qwidget.h>
 #include <fcntl.h>
 #include <signal.h>
+#include <pthread.h>
 
 #include <iostream>
 using namespace std;
 
+#include "mythconfig.h"
 #include "tv.h"
 #include "proglist.h"
 #include "progfind.h"
 #include "manualbox.h"
 #include "manualschedule.h"
 #include "playbackbox.h"
-#include "previousbox.h"
-#include "customrecord.h"
+#include "previouslist.h"
+#include "customedit.h"
 #include "viewscheduled.h"
 #include "programrecpriority.h"
 #include "channelrecpriority.h"
 #include "globalsettings.h"
 #include "profilegroup.h"
+#include "playgroup.h"
+#include "networkcontrol.h"
 
-#include "themedmenu.h"
+#include "exitcodes.h"
 #include "programinfo.h"
 #include "mythcontext.h"
 #include "mythdbcon.h"
@@ -42,19 +46,23 @@ using namespace std;
 #include "lcddevice.h"
 #include "langsettings.h"
 
+#include "libmythui/myththemedmenu.h"
+#include "libmythui/myththemebase.h"
+
+#define NO_EXIT  0
 #define QUIT     1
 #define HALT     2
 
-ThemedMenu *menu;
+static MythThemedMenu *menu;
+static MythThemeBase *themeBase;
 XBox *xbox = NULL;
 
 void startGuide(void)
 {
-    QString startchannel = gContext->GetSetting("DefaultTVChannel");
-    if (startchannel == "")
-        startchannel = "3";
-
-    RunProgramGuide(startchannel);
+    uint chanid = 0;
+    QString channum = gContext->GetSetting("DefaultTVChannel");
+    channum = (channum.isEmpty()) ? "3" : channum;
+    RunProgramGuide(chanid, channum);
 }
 
 void startFinder(void)
@@ -64,7 +72,7 @@ void startFinder(void)
 
 void startSearchTitle(void)
 {
-  ProgLister searchTitle(plTitleSearch, "",
+    ProgLister searchTitle(plTitleSearch, "", "",
                          gContext->GetMainWindow(), "proglist");
 
     qApp->unlock();
@@ -74,7 +82,7 @@ void startSearchTitle(void)
 
 void startSearchKeyword(void)
 {
-  ProgLister searchKeyword(plKeywordSearch, "",
+    ProgLister searchKeyword(plKeywordSearch, "", "",
                         gContext->GetMainWindow(), "proglist");
 
     qApp->unlock();
@@ -84,7 +92,7 @@ void startSearchKeyword(void)
 
 void startSearchPeople(void)
 {
-  ProgLister searchPeople(plPeopleSearch, "",
+    ProgLister searchPeople(plPeopleSearch, "", "",
                          gContext->GetMainWindow(), "proglist");
 
     qApp->unlock();
@@ -94,7 +102,7 @@ void startSearchPeople(void)
 
 void startSearchPower(void)
 {
-  ProgLister searchPower(plPowerSearch, "",
+    ProgLister searchPower(plPowerSearch, "", "",
                          gContext->GetMainWindow(), "proglist");
 
     qApp->unlock();
@@ -104,7 +112,7 @@ void startSearchPower(void)
 
 void startSearchChannel(void)
 {
-    ProgLister searchChannel(plChannel, "",
+    ProgLister searchChannel(plChannel, "", "",
                              gContext->GetMainWindow(), "proglist");
 
     qApp->unlock();
@@ -114,7 +122,7 @@ void startSearchChannel(void)
 
 void startSearchCategory(void)
 {
-  ProgLister searchCategory(plCategory, "",
+    ProgLister searchCategory(plCategory, "", "",
                             gContext->GetMainWindow(), "proglist");
 
     qApp->unlock();
@@ -124,7 +132,7 @@ void startSearchCategory(void)
 
 void startSearchMovie(void)
 {
-    ProgLister searchMovie(plMovies, "",
+    ProgLister searchMovie(plMovies, "", "",
                            gContext->GetMainWindow(), "proglist");
 
     qApp->unlock();
@@ -134,7 +142,7 @@ void startSearchMovie(void)
 
 void startSearchNew(void)
 {
-    ProgLister searchNew(plNewListings, "",
+    ProgLister searchNew(plNewListings, "", "",
                          gContext->GetMainWindow(), "proglist");
 
     qApp->unlock();
@@ -144,7 +152,7 @@ void startSearchNew(void)
 
 void startSearchTime(void)
 {
-  ProgLister searchTime(plTime, "",
+    ProgLister searchTime(plTime, "", "",
                          gContext->GetMainWindow(), "proglist");
 
     qApp->unlock();
@@ -201,16 +209,16 @@ void startDelete(void)
 
 void startPrevious(void)
 {
-    PreviousBox previous(gContext->GetMainWindow(), "previous box");
+    PreviousList previous(gContext->GetMainWindow(), "previous list");
 
     qApp->unlock();
     previous.exec();
     qApp->lock();
 }
 
-void startCustomRecord(void)
+void startCustomEdit(void)
 {
-    CustomRecord custom(gContext->GetMainWindow(), "custom record");
+    CustomEdit custom(gContext->GetMainWindow(), "custom record");
 
     qApp->unlock();
     custom.exec();
@@ -235,13 +243,10 @@ void startManualSchedule(void)
     qApp->lock();
 }
 
-void startTV(void)
+void startTV(bool startInGuide)
 {
     TV *tv = new TV();
 
-    QTime timer;
-    timer.start();
-
     if (!tv->Init())
     {
         VERBOSE(VB_IMPORTANT, "Experienced fatal error "
@@ -250,12 +255,10 @@ void startTV(void)
         return;
     }
 
-    bool tryTV = false;
-    bool tryRecorder = false;
     bool quitAll = false;
-
     bool showDialogs = true;
-    if (!tv->LiveTV(showDialogs))
+
+    if (!tv->LiveTV(showDialogs, startInGuide))
     {
         tv->StopLiveTV();
         quitAll = true;
@@ -263,46 +266,6 @@ void startTV(void)
 
     while (!quitAll)
     {
-        timer.restart();
-        while (tryRecorder)
-        { //try to play recording, if filenotfound retry until timeout
-
-            if (tv->PlayFromRecorder(tv->GetLastRecorderNum()) == 1)
-                tryRecorder = false;
-            else if (timer.elapsed() > 2000)
-            {
-                tryRecorder = false;
-                tryTV = true;
-            }
-            else
-            {
-                qApp->unlock();
-                qApp->processEvents();
-                usleep(100000);
-                qApp->lock();
-             }
-        }
-
-        timer.restart();
-        while (tryTV)
-        {//try to start livetv
-            bool showDialogs = false;
-            if (tv->LiveTV(showDialogs) == 1) //1 == livetv ok
-                tryTV = false;
-            else if (timer.elapsed() > 2000)
-            {
-                tryTV = false;
-                quitAll = true;
-            }
-            else
-            {
-                qApp->unlock();
-                qApp->processEvents();
-                usleep(100000);
-                qApp->lock();
-            }
-        }
-
         while (tv->GetState() != kState_None)
         {
             qApp->unlock();
@@ -311,32 +274,48 @@ void startTV(void)
             qApp->lock();
         }
 
-        if (tv->WantsToQuit() && !tv->IsSwitchingCards())
+        if (tv->WantsToQuit())
             quitAll = true;
-        else
-        {
-            if (tv->IsSwitchingCards())
-            {
-                tryRecorder = false;
-                tryTV = true;
-            }
-            else
-            {
-                tryRecorder = true;
-                tryTV = false;
-            }
-        }
+    }
+
+    while (tv->IsMenuRunning()) 
+    {
+        qApp->unlock();
+        qApp->processEvents();
+        usleep(10000);
+        qApp->lock();
     }
 
     delete tv;
 }
 
+void startTVInGuide(void)
+{
+    startTV(true);
+}
+
+void startTVNormal(void)
+{
+    startTV(false);
+}
+
 void showStatus(void)
 {
     StatusBox statusbox(gContext->GetMainWindow(), "status box");
-    qApp->unlock();
-    statusbox.exec();
-    qApp->lock();
+    if (statusbox.IsErrored())
+    {
+        MythPopupBox::showOkPopup(
+            gContext->GetMainWindow(), QObject::tr("Theme Error"),
+            QString(QObject::tr(
+                        "Your theme does not contain elements required "
+                        "to display the status screen.")));
+    }
+    else
+    {
+        qApp->unlock();
+        statusbox.exec();
+        qApp->lock();
+    }
 }
 
 void TVMenuCallback(void *data, QString &selection)
@@ -344,8 +323,16 @@ void TVMenuCallback(void *data, QString &selection)
     (void)data;
     QString sel = selection.lower();
 
+    if (sel.left(9) == "settings ")
+    {
+        gContext->addCurrentLocation("Setup");
+        gContext->ActivateSettingsCache(false);
+    }
+
     if (sel == "tv_watch_live")
-        startTV();
+        startTVNormal();
+    else if (sel == "tv_watch_live_epg")
+        startTVInGuide();
     else if (sel == "tv_watch_recording")
         startPlayback();
     else if (sel == "tv_schedule")
@@ -357,7 +344,7 @@ void TVMenuCallback(void *data, QString &selection)
     else if (sel == "tv_manualschedule")
         startManualSchedule();
     else if (sel == "tv_custom_record")
-        startCustomRecord();
+        startCustomEdit();
     else if (sel == "tv_fix_conflicts")
         startManaged();
     else if (sel == "tv_set_recpriorities")
@@ -389,18 +376,18 @@ void TVMenuCallback(void *data, QString &selection)
         AppearanceSettings settings;
         settings.exec();
 
-        LanguageSettings::reload();
-
-        gContext->LoadQtConfig();
-        gContext->GetMainWindow()->Init();
-        gContext->UpdateImageCache();
-        menu->ReloadTheme();
+        GetMythMainWindow()->JumpTo("Reload Theme");
     } 
     else if (sel == "settings recording") 
     {
         ProfileGroupEditor editor;
         editor.exec();
     } 
+    else if (sel == "settings playgroup") 
+    {
+        PlayGroupEditor editor;
+        editor.exec();
+    } 
     else if (sel == "settings general") 
     {
         GeneralSettings settings;
@@ -428,7 +415,6 @@ void TVMenuCallback(void *data, QString &selection)
     {
         GeneralRecPrioritiesSettings settings;
         settings.exec();
-        ScheduledRecording::signalChange(0);
     } 
     else if (sel == "settings channelrecpriorities") 
     {
@@ -443,6 +429,18 @@ void TVMenuCallback(void *data, QString &selection)
     }
     else if (sel == "tv_status")
         showStatus();
+
+    if (sel.left(9) == "settings ")
+    {
+        gContext->removeCurrentLocation();
+
+        gContext->ActivateSettingsCache(true);
+        RemoteSendMessage("CLEAR_SETTINGS_CACHE");
+
+        if (sel == "settings general" ||
+            sel == "settings generalrecpriorities")
+            ScheduledRecording::signalChange(0);
+    }
 }
 
 int handleExit(void)
@@ -450,36 +448,25 @@ int handleExit(void)
     if (gContext->GetNumSetting("NoPromptOnExit", 1) == 0)
         return QUIT;
 
-    // first of all find out, if a backend runs on this host...
-    bool backendOnLocalhost = false;
-
-    QStringList strlist = "QUERY_IS_ACTIVE_BACKEND";
-    strlist << gContext->GetHostName();
-
-    gContext->SendReceiveStringList(strlist);
-
-    if (QString(strlist[0]) == "FALSE")
-        backendOnLocalhost = false;
-    else //if (QString(strlist[0]) == "TRUE")
-        backendOnLocalhost = true;
+    // first of all find out, if this is a frontend only host...
+    bool frontendOnly = gContext->IsFrontendOnly();
 
     QString title = QObject::tr("Do you really want to exit MythTV?");
 
     DialogBox diag(gContext->GetMainWindow(), title);
     diag.AddButton(QObject::tr("No"));
     diag.AddButton(QObject::tr("Yes, Exit now"));
-    if (!backendOnLocalhost)
+    if (frontendOnly)
         diag.AddButton(QObject::tr("Yes, Exit and Shutdown"));
 
     int result = diag.exec();
     switch (result)
     {
+        case 1: return NO_EXIT;
         case 2: return QUIT;
         case 3: return HALT;
-        default: return 0;
+        default: return NO_EXIT;
     }
-
-    return 0;
 }
 
 void haltnow()
@@ -490,28 +477,20 @@ void haltnow()
         system(halt_cmd.ascii());
 }
 
-int RunMenu(QString themedir)
+bool RunMenu(QString themedir)
 {
-    menu = new ThemedMenu(themedir.ascii(), "mainmenu.xml", 
-                          gContext->GetMainWindow(), "mainmenu");
+    menu = new MythThemedMenu(themedir.ascii(), "mainmenu.xml", 
+                              GetMythMainWindow()->GetMainStack(), "mainmenu");
     menu->setCallback(TVMenuCallback, gContext);
    
-    int exitstatus = 0;
- 
     if (menu->foundTheme())
     {
-        do {
-            menu->exec();
-        } while (!(exitstatus = handleExit()));
-    }
-    else
-    {
-        cerr << "Couldn't find theme " << themedir << endl;
+        GetMythMainWindow()->GetMainStack()->AddScreen(menu);
+        return true;
     }
 
-    delete menu;
-    menu = NULL;
-    return exitstatus;
+    cerr << "Couldn't find theme " << themedir << endl;
+    return false;
 }   
 
 // If any settings are missing from the database, this will write
@@ -578,12 +557,25 @@ QString RandTheme(QString &themename)
     return themename;
 }
 
-int internal_play_media(const char *mrl, const char* plot, const char* title, 
-                        const char* director, int lenMins, const char* year) 
+int internal_play_media(const QString &mrl, const QString &plot, 
+                        const QString &title, const QString &director, 
+                        int lenMins, const QString &year) 
 {
     int res = -1; 
+  
+    QFile checkFile(mrl);
+    if (!checkFile.exists() && !mrl.startsWith("dvd:"))
+    {
+        QString errorText = QObject::tr("Failed to open \n '%1' in %2 \n"
+                                        "Check if the video exists")
+                                        .arg(mrl.section("/", -1))
+                                        .arg(mrl.section("/", 0, -2));
+        MythPopupBox::showOkPopup(gContext->GetMainWindow(),
+                                    "Open Failed",
+                                    errorText);
+        return res;
+    }
     
-   
     TV *tv = new TV();
 
     if (!tv->Init())
@@ -601,16 +593,22 @@ int internal_play_media(const char *mrl, const char* plot, const char* title,
     pginfo->lenMins = lenMins;
     pginfo->isVideo = true;
     pginfo->pathname = mrl;
-    pginfo->description = plot;
     
+    QDir d(mrl + "/VIDEO_TS");
+    if (mrl.findRev(".iso", -1, false) == (int)mrl.length() - 4 ||
+        mrl.findRev(".img", -1, false) == (int)mrl.length() - 4 ||
+        d.exists())
+    {
+        pginfo->pathname = QString("dvd:%1").arg(mrl);
+    }
+    
+    pginfo->description = plot;
     
     if (strlen(director))
         pginfo->subtitle = QString( "%1: %2" ).arg(QObject::tr("Directed By")).arg(director);
     
     pginfo->title = title;
 
-    tv->Playback(pginfo);
-    
     if (tv->Playback(pginfo))
     {
         while (tv->GetState() != kState_None)
@@ -619,10 +617,10 @@ int internal_play_media(const char *mrl, const char* plot, const char* title,
             qApp->processEvents();
             usleep(10000);
             qApp->lock();
+            
         }
     }
     
-
     res = 0;
     
     sleep(1);
@@ -632,8 +630,41 @@ int internal_play_media(const char *mrl, const char* plot, const char* title,
     return res;
 }
 
+void gotoMainMenu(void)
+{
+    // If we got to this callback, we're back on the menu.  So, send a CTRL-L
+    // to cause the menu to reload
+    QKeyEvent *event =
+        new QKeyEvent(QEvent::KeyPress, Qt::Key_L, 0, Qt::ControlButton);
+    QApplication::postEvent((QObject*)(gContext->GetMainWindow()), event);
+}
+
+void reloadTheme(void)
+{
+    LanguageSettings::reload();
+
+    gContext->LoadQtConfig();
+    gContext->GetMainWindow()->Init();
+    gContext->UpdateImageCache();
+
+    themeBase->Reload();
+    menu->ReloadTheme();
+
+    if (!menu->foundTheme())
+        exit(FRONTEND_BUGGY_EXIT_NO_THEME);
+
+    LCD::SetupLCD();
+    if (LCD *lcd = LCD::Get()) 
+    {
+        lcd->setupLEDs(RemoteGetRecordingMask);
+        lcd->resetServer();
+    }
+}
+
 void InitJumpPoints(void)
 {
+    REG_JUMP("Reload Theme", "", "", reloadTheme);
+    REG_JUMP("Main Menu", "", "", gotoMainMenu);
     REG_JUMP("Program Guide", "", "", startGuide);
     REG_JUMP("Program Finder", "", "", startFinder);
     //REG_JUMP("Search Listings", "", "", startSearch);
@@ -642,8 +673,11 @@ void InitJumpPoints(void)
     REG_JUMP("Channel Recording Priorities", "", "", startChannelRecPriorities);
     REG_JUMP("TV Recording Playback", "", "", startPlayback);
     REG_JUMP("TV Recording Deletion", "", "", startDelete);
-    REG_JUMP("Live TV", "", "", startTV);
+    REG_JUMP("Live TV", "", "", startTVNormal);
+    REG_JUMP("Live TV In Guide", "", "", startTVInGuide);
     REG_JUMP("Manual Record Scheduling", "", "", startManual);
+    REG_JUMP("Status Screen", "", "", showStatus);
+    REG_JUMP("Previously Recorded", "", "", startPrevious);
 
     TV::InitKeys();
 }
@@ -657,6 +691,9 @@ int internal_media_init()
 
 static void *run_priv_thread(void *data)
 {
+    VERBOSE(VB_PLAYBACK, QString("user: %1 effective user: %2 run_priv_thread")
+                            .arg(getuid()).arg(geteuid()));
+
     (void)data;
     while (true) 
     {
@@ -708,18 +745,46 @@ static void *run_priv_thread(void *data)
     }
     return NULL; // will never happen
 }
-   
+
+void CleanupMyOldInUsePrograms(void)
+{
+    MSqlQuery query(MSqlQuery::InitCon());
+
+    query.prepare("DELETE FROM inuseprograms "
+                  "WHERE hostname = :HOSTNAME and recusage = 'player' ;");
+    query.bindValue(":HOSTNAME", gContext->GetHostName());
+    query.exec();
+}
+
 int main(int argc, char **argv)
 {
-    QString lcd_host;
-    int lcd_port;
-    QString geometry = "";
-#ifdef Q_WS_X11
-    // Remember any -geometry argument which QApplication init will remove
-    for(int argpos = 1; argpos + 1 < argc; ++argpos)
-        if (!strcmp(argv[argpos],"-geometry"))
-            geometry = argv[argpos+1];
+    QString geometry = QString::null;
+    QString display  = QString::null;
+
+    for(int argpos = 0; argpos < argc; ++argpos)
+    {
+        if (!strcmp(argv[argpos],"--version"))
+        {
+            extern const char *myth_source_version;
+            extern const char *myth_source_path;
+            cout << "Library API version     : " << MYTH_BINARY_VERSION << endl;
+            cout << "Source code version     : " << myth_source_version << endl;
+            cout << "SVN Branch              : " << myth_source_path << endl;
+#ifdef MYTH_BUILD_CONFIG
+            cout << "Options compiled in:" <<endl;
+            cout << MYTH_BUILD_CONFIG << endl;
 #endif
+            return FRONTEND_EXIT_OK;
+        }
+#ifdef Q_WS_X11
+    // Remember any -display or -geometry argument
+    // which QApplication init will remove.
+        else if (!strcmp(argv[argpos],"-geometry"))
+            geometry = argv[argpos+1];
+        else if (!strcmp(argv[argpos],"-display"))
+            display = argv[argpos+1];
+#endif
+    }
 
 #ifdef Q_WS_MACX
     // Without this, we can't set focus to any of the CheckBoxSetting, and most
@@ -729,17 +794,26 @@ int main(int argc, char **argv)
     QApplication a(argc, argv);
 
     QString logfile = "";
-    QString verboseString = QString(" important general");
 
     QString pluginname = "";
+    QMap<QString, QString> settingsOverride;
 
     QFileInfo finfo(a.argv()[0]);
 
     QString binname = finfo.baseName();
 
+    bool ResetSettings = false;
+
     if (binname != "mythfrontend")
         pluginname = binname;
 
+    gContext = new MythContext(MYTH_BINARY_VERSION);
+    if (!gContext->Init())
+    {   
+        VERBOSE(VB_IMPORTANT, "Failed to init MythContext, exiting.");
+        return FRONTEND_EXIT_NO_MYTHCONTEXT;
+    }
+
     for(int argpos = 1; argpos < a.argc(); ++argpos)
     {
         if (!strcmp(a.argv()[argpos],"-l") ||
@@ -751,7 +825,7 @@ int main(int argc, char **argv)
                 if (logfile.startsWith("-"))
                 {
                     cerr << "Invalid or missing argument to -l/--logfile option\n";
-                    return -1;
+                    return FRONTEND_EXIT_INVALID_CMDLINE;
                 }
                 else
                 {
@@ -761,108 +835,119 @@ int main(int argc, char **argv)
             else
             {
                 cerr << "Missing argument to -l/--logfile option\n";
-                return -1;
+                return FRONTEND_EXIT_INVALID_CMDLINE;
             }
         } else if (!strcmp(a.argv()[argpos],"-v") ||
             !strcmp(a.argv()[argpos],"--verbose"))
         {
             if (a.argc()-1 > argpos)
             {
-                QString temp = a.argv()[argpos+1];
-                if (temp.startsWith("-"))
-                {
-                    cerr << "Invalid or missing argument to -v/--verbose option\n";
-                    return -1;
-                } else
-                {
-                    QStringList verboseOpts;
-                    verboseOpts = QStringList::split(',', temp);
-                    ++argpos;
-                    for (QStringList::Iterator it = verboseOpts.begin(); 
-                         it != verboseOpts.end(); ++it )
-                    {
-                        if (!strcmp(*it,"none"))
-                        {
-                            print_verbose_messages = VB_NONE;
-                            verboseString = "";
-                        }
-                        else if (!strcmp(*it,"all"))
-                        {
-                            print_verbose_messages = VB_ALL;
-                            verboseString = "all";
-                        }
-                        else if (!strcmp(*it,"quiet"))
-                        {
-                            print_verbose_messages = VB_IMPORTANT;
-                            verboseString = "important";
-                        }
-                        else if (!strcmp(*it,"record"))
-                        {
-                            print_verbose_messages |= VB_RECORD;
-                            verboseString += " " + *it;
-                        }
-                        else if (!strcmp(*it,"playback"))
-                        {
-                            print_verbose_messages |= VB_PLAYBACK;
-                            verboseString += " " + *it;
-                        }
-                        else if (!strcmp(*it,"channel"))
-                        {
-                            print_verbose_messages |= VB_CHANNEL;
-                            verboseString += " " + *it;
-                        }
-                        else if (!strcmp(*it,"osd"))
-                        {
-                            print_verbose_messages |= VB_OSD;
-                            verboseString += " " + *it;
-                        }
-                        else if (!strcmp(*it,"file"))
-                        {
-                            print_verbose_messages |= VB_FILE;
-                            verboseString += " " + *it;
-                        }
-                        else if (!strcmp(*it,"schedule"))
-                        {
-                            print_verbose_messages |= VB_SCHEDULE;
-                            verboseString += " " + *it;
-                        }
-                        else if (!strcmp(*it,"network"))
-                        {
-                            print_verbose_messages |= VB_NETWORK;
-                            verboseString += " " + *it;
-                        }
-                        else if (!strcmp(*it,"commflag"))
-                        {
-                            print_verbose_messages |= VB_COMMFLAG;
-                            verboseString += " " + *it;
-                        }
-                        else if (!strcmp(*it,"audio"))
-                        {
-                            print_verbose_messages |= VB_AUDIO;
-                            verboseString += " " + *it;
-                        }
-                        else if (!strcmp(*it,"libav"))
-                        {
-                            print_verbose_messages |= VB_LIBAV;
-                            verboseString += " " + *it;
-                        }
-                        else
-                        {
-                            cerr << "Unknown argument for -v/--verbose: "
-                                 << *it << endl;;
-                        }
-                    }
-                }
+                if (parse_verbose_arg(a.argv()[argpos+1]) ==
+                        GENERIC_EXIT_INVALID_CMDLINE)
+                    return FRONTEND_EXIT_INVALID_CMDLINE;
+
+                ++argpos;
             } else
             {
                 cerr << "Missing argument to -v/--verbose option\n";
-                return -1;
+                return FRONTEND_EXIT_INVALID_CMDLINE;
             }
         }
-        else if (!strcmp(a.argv()[argpos],"--version"))
+        else if (!strcmp(a.argv()[argpos],"-r") ||
+                 !strcmp(a.argv()[argpos],"--reset"))
         {
-            cout << MYTH_BINARY_VERSION << endl;
-            exit(0);
+            ResetSettings = true;
+        }
+        else if (!strcmp(a.argv()[argpos],"-w") ||
+                 !strcmp(a.argv()[argpos],"--windowed"))
+        {
+            settingsOverride["RunFrontendInWindow"] = "1";
+        }
+        else if (!strcmp(a.argv()[argpos],"-nw") ||
+                 !strcmp(a.argv()[argpos],"--no-windowed"))
+        {
+            settingsOverride["RunFrontendInWindow"] = "0";
+        }
+        else if (!strcmp(a.argv()[argpos],"-O") ||
+                 !strcmp(a.argv()[argpos],"--override-setting"))
+        {
+            if (a.argc()-1 > argpos)
+            {
+                QString tmpArg = a.argv()[argpos+1];
+                if (tmpArg.startsWith("-"))
+                {
+                    cerr << "Invalid or missing argument to -O/--override-setting option\n";
+                    return BACKEND_EXIT_INVALID_CMDLINE;
+                } 
+ 
+                QStringList pairs = QStringList::split(",", tmpArg);
+                for (unsigned int index = 0; index < pairs.size(); ++index)
+                {
+                    QStringList tokens = QStringList::split("=", pairs[index]);
+                    tokens[0].replace(QRegExp("^[\"']"), "");
+                    tokens[0].replace(QRegExp("[\"']$"), "");
+                    tokens[1].replace(QRegExp("^[\"']"), "");
+                    tokens[1].replace(QRegExp("[\"']$"), "");
+                    settingsOverride[tokens[0]] = tokens[1];
+                }
+            }
+            else
+            {
+                cerr << "Invalid or missing argument to -O/--override-setting option\n";
+                return BACKEND_EXIT_INVALID_CMDLINE;
+            }
+
+            ++argpos;
+        }
+        else if (!strcmp(a.argv()[argpos],"-G") ||
+                 !strcmp(a.argv()[argpos],"--get-setting"))
+        {
+            if (a.argc()-1 > argpos)
+            {
+                QString tmpArg = a.argv()[argpos+1];
+                if (tmpArg.startsWith("-"))
+                {
+                    cerr << "Invalid or missing argument to -G/--get-setting option\n";
+                    return FRONTEND_EXIT_INVALID_CMDLINE;
+                } 
+ 
+                QStringList pairs = QStringList::split(",", tmpArg);
+                QString value;
+                for (unsigned int index = 0; index < pairs.size(); ++index)
+                {
+                    value = gContext->GetSetting(pairs[index]);
+                    cerr << "\tSettings Value : " << pairs[index];
+                    cerr <<  " = " << value << endl;
+                }
+                return FRONTEND_EXIT_OK;
+            }
+            else
+            {
+                cerr << "Invalid or missing argument to -G/--get-setting option\n";
+                return FRONTEND_EXIT_INVALID_CMDLINE;
+            }
+
+            ++argpos;
+        }
+        else if (!strcmp(a.argv()[argpos],"-display") ||
+                 !strcmp(a.argv()[argpos],"--display"))
+        {
+            if (a.argc()-1 > argpos)
+            {
+                display = a.argv()[argpos+1];
+                if (display.startsWith("-"))
+                {
+                    cerr << "Invalid or missing argument to -display option\n";
+                    return FRONTEND_EXIT_INVALID_CMDLINE;
+                }
+                else
+                    ++argpos;
+            }
+            else
+            {
+                cerr << "Missing argument to -display option\n";
+                return FRONTEND_EXIT_INVALID_CMDLINE;
+            }
         }
         else if (!strcmp(a.argv()[argpos],"-geometry") ||
                  !strcmp(a.argv()[argpos],"--geometry"))
@@ -873,7 +958,7 @@ int main(int argc, char **argv)
                 if (geometry.startsWith("-"))
                 {
                     cerr << "Invalid or missing argument to -geometry option\n";
-                    return -1;
+                    return FRONTEND_EXIT_INVALID_CMDLINE;
                 }
                 else
                     ++argpos;
@@ -881,7 +966,7 @@ int main(int argc, char **argv)
             else
             {
                 cerr << "Missing argument to -geometry option\n";
-                return -1;
+                return FRONTEND_EXIT_INVALID_CMDLINE;
             }
         }
         else if ((argpos + 1 == a.argc()) &&
@@ -902,17 +987,24 @@ int main(int argc, char **argv)
                     "-geometry or --geometry WxH    Override window size settings\n" <<
                     "--geometry WxH+X+Y             Override window size and position\n" <<
                     "-l or --logfile filename       Writes STDERR and STDOUT messages to filename" << endl <<
-                    "-v or --verbose debug-level    Prints more information" << endl <<
-                    "                               Accepts any combination (separated by comma)" << endl << 
-                    "                               of all,none,quiet,record,playback,channel," << endl <<
-                    "                               osd,file,schedule,network,commflag,audio,libav" << endl <<
+                    "-r or --reset                  Resets frontend appearance settings and language" << endl <<
+                    "-w or --windowed               Run in windowed mode" << endl <<
+                    "-nw or --no-windowed           Run in non-windowed mode " << endl <<
+                    "-O or " << endl <<
+                    "  --override-setting KEY=VALUE Force the setting named 'KEY' to value 'VALUE'" << endl <<
+                    "                               This option may be repeated multiple times" << endl <<
+                    "-G or " << endl <<
+                    "  --get-setting KEY[,KEY2,etc] Returns the current database setting for 'KEY'" << endl <<
+                    "                               Use a comma seperated list to return multiple values" << endl <<
+                    "-v or --verbose debug-level    Use '-v help' for level info" << endl <<
+
                     "--version                      Version information" << endl <<
                     "<plugin>                       Initialize and run this plugin" << endl <<
                     endl <<
                     "Environment Variables:" << endl <<
                     "$MYTHTVDIR                     Set the installation prefix" << endl <<
                     "$MYTHCONFDIR                   Set the config dir (instead of ~/.mythtv)" << endl;
-            return -1;
+            return FRONTEND_EXIT_INVALID_CMDLINE;
         }
     }
 
@@ -925,7 +1017,7 @@ int main(int argc, char **argv)
         if (logfd < 0)
         {
             perror("open(logfile)");
-            return -1;
+            return FRONTEND_EXIT_OPENING_LOGFILE_ERROR;
         }
     }
 
@@ -949,24 +1041,51 @@ int main(int argc, char **argv)
     if (!dir.exists())
         dir.mkdir(fileprefix);
 
-    gContext = NULL;
-    gContext = new MythContext(MYTH_BINARY_VERSION);
-
-    if (geometry != "")
-        if (!gContext->ParseGeometryOverride(geometry))
-            cerr << "Illegal -geometry argument '"
-                 << geometry <<"' (ignored)\n";
-
-    if(!gContext->Init())
+    if (ResetSettings)
     {
-        VERBOSE(VB_IMPORTANT, "Failed to init MythContext, exiting.");
-        return -1;
+       AppearanceSettings as;
+       as.save();
+
+       MSqlQuery query(MSqlQuery::InitCon());
+       query.prepare("update settings set data='EN' "
+                     "WHERE hostname = :HOSTNAME and value='Language' ;");
+       query.bindValue(":HOSTNAME", gContext->GetHostName());
+       query.exec();
+
+       return FRONTEND_EXIT_OK;
+    }
+
+    if (!display.isEmpty())
+    {
+        gContext->SetX11Display(display);
+    }
+
+    if (!geometry.isEmpty() && !gContext->ParseGeometryOverride(geometry))
+    {
+        VERBOSE(VB_IMPORTANT,
+                QString("Illegal -geometry argument '%1' (ignored)")
+                .arg(geometry));
+    }
+
+    if (settingsOverride.size())
+    {
+        QMap<QString, QString>::iterator it;
+        for (it = settingsOverride.begin(); it != settingsOverride.end(); ++it)
+        {
+            VERBOSE(VB_IMPORTANT, QString("Setting '%1' being forced to '%2'")
+                                          .arg(it.key()).arg(it.data()));
+            gContext->OverrideSettingForSession(it.key(), it.data());
+        }
     }
 
     // Create priveleged thread, then drop privs
     pthread_t priv_thread;
 
+    VERBOSE(VB_PLAYBACK, QString("user: %1 effective user: %2 before "
+                            "privileged thread").arg(getuid()).arg(geteuid()));
     int status = pthread_create(&priv_thread, NULL, run_priv_thread, NULL);
+    VERBOSE(VB_PLAYBACK, QString("user: %1 effective user: %2 after "
+                            "privileged thread").arg(getuid()).arg(geteuid()));
     if (status) 
     {
         perror("pthread_create");
@@ -974,34 +1093,29 @@ int main(int argc, char **argv)
     }
     setuid(getuid());
 
+    gContext->ActivateSettingsCache(false);
     if (!UpgradeTVDatabaseSchema())
     {
         VERBOSE(VB_IMPORTANT,
                 "Couldn't upgrade database to new schema, exiting.");
-        return -1;
-    }    
+        return FRONTEND_EXIT_DB_OUTOFDATE;
+    }
+    gContext->ActivateSettingsCache(true);
 
-    VERBOSE(VB_ALL, QString("%1 version: %2 www.mythtv.org")
+    VERBOSE(VB_IMPORTANT, QString("%1 version: %2 www.mythtv.org")
                             .arg(binname).arg(MYTH_BINARY_VERSION));
 
-    VERBOSE(VB_ALL, QString("Enabled verbose msgs :%1").arg(verboseString));
+    VERBOSE(VB_IMPORTANT, QString("Enabled verbose msgs: %1").arg(verboseString));
 
-    lcd_host = gContext->GetSetting("LCDHost", "localhost");
-    lcd_port = gContext->GetNumSetting("LCDPort", 13666);
-    if (lcd_host.length() > 0 && lcd_port > 1024)
-    {
-        class LCD * lcd = LCD::Get();
-        if (lcd->connectToHost(lcd_host, lcd_port) == false)
-            delete lcd;
-        else
-            lcd->setupLEDs(RemoteGetRecordingMask);
-    }
+    LCD::SetupLCD();
+    if (LCD *lcd = LCD::Get())
+        lcd->setupLEDs(RemoteGetRecordingMask);
 
     LanguageSettings::load("mythfrontend");
 
     WriteDefaults();
 
-    QString themename = gContext->GetSetting("Theme", "blue");
+    QString themename = gContext->GetSetting("Theme", "G.A.N.T.");
     bool randomtheme = gContext->GetNumSetting("RandomTheme", 0);
 
     if (randomtheme)
@@ -1011,24 +1125,28 @@ int main(int argc, char **argv)
     if (themedir == "")
     {
         cerr << "Couldn't find theme " << themename << endl;
-        exit(21);
+        return FRONTEND_EXIT_NO_THEME;
     }
 
     gContext->LoadQtConfig();
 
-    MythMainWindow *mainWindow = new MythMainWindow();
+    MythMainWindow *mainWindow = GetMythMainWindow();
     gContext->SetMainWindow(mainWindow);
+
+    gContext->UpdateImageCache();
+    themeBase = new MythThemeBase();
+
     LanguageSettings::prompt();
 
     InitJumpPoints();
 
     internal_media_init();
 
+    CleanupMyOldInUsePrograms();
+
     MythPluginManager *pmanager = new MythPluginManager();
     gContext->SetPluginManager(pmanager);
 
-    gContext->UpdateImageCache();
-
     if (gContext->GetNumSetting("EnableXbox") == 1)
     {
         xbox = new XBox();
@@ -1048,16 +1166,22 @@ int main(int argc, char **argv)
     {
         if (pmanager->run_plugin(pluginname))
         {
+            qApp->setMainWidget(mainWindow);
+            qApp->exec();
+
             qApp->unlock();
-            return 0;
+            return FRONTEND_EXIT_OK;
         }
-        else
+        else 
         {
             pluginname = "myth" + pluginname;
             if (pmanager->run_plugin(pluginname))
             {
+                qApp->setMainWidget(mainWindow);
+                qApp->exec();
+
                 qApp->unlock();
-                return 0;
+                return FRONTEND_EXIT_OK;
             }
         }
     }
@@ -1065,24 +1189,55 @@ int main(int argc, char **argv)
     qApp->unlock();
 
 #ifndef _WIN32
-    MediaMonitor *mon = NULL;
-    mon = MediaMonitor::getMediaMonitor();
+    MediaMonitor *mon = MediaMonitor::GetMediaMonitor();
     if (mon)
     {
-        VERBOSE(VB_ALL, QString("Starting media monitor."));
-        mon->startMonitoring();
+        VERBOSE(VB_IMPORTANT, QString("Starting media monitor."));
+        mon->StartMonitoring();
     }
 #endif
 
-    int exitstatus = RunMenu(themedir);
+    NetworkControl *networkControl = NULL;
+    if (gContext->GetNumSetting("NetworkControlEnabled", 0))
+    {
+        int networkPort = gContext->GetNumSetting("NetworkControlPort", 6545);
+        networkControl = new NetworkControl(networkPort);
+        if (!networkControl->ok())
+            VERBOSE(VB_IMPORTANT,
+                    QString("NetworkControl failed to bind to port %1.")
+                    .arg(networkPort));
+    }
+
+    gContext->addCurrentLocation("MainMenu");
+
+    int exitstatus = NO_EXIT;
+
+    do
+    {
+        themename = gContext->GetSetting("Theme", "blue");
+        themedir = gContext->FindThemeDir(themename);
+        if (themedir == "")
+        {
+            cerr << "Couldn't find theme " << themename << endl;
+            return FRONTEND_EXIT_NO_THEME;
+        }
+
+        if (!RunMenu(themedir))
+            break;
+
+        qApp->setMainWidget(mainWindow);
+        qApp->exec();
+    } while (!(exitstatus = handleExit()));
 
     if (exitstatus == HALT)
         haltnow();
 
+    pmanager->DestroyAllPlugins();
+
 #ifndef _WIN32
     if (mon)
     {
-        mon->stopMonitoring();
+        mon->StopMonitoring();
         delete mon;
     }
 #endif
@@ -1094,7 +1249,13 @@ int main(int argc, char **argv)
         pthread_join(priv_thread, &value);
     }
 
-    delete mainWindow;
+    if (networkControl)
+        delete networkControl;
+
+    DestroyMythMainWindow();
+    delete themeBase;
     delete gContext;
-    return exitstatus;
+    return FRONTEND_EXIT_OK;
 }
+
+/* vim: set expandtab tabstop=4 shiftwidth=4: */
