@@ -14,10 +14,11 @@ using namespace std;
 
 #include "viewscheduled.h"
 #include "scheduledrecording.h"
-#include "infodialog.h"
+#include "customedit.h"
 #include "proglist.h"
 #include "tv.h"
 
+#include "exitcodes.h"
 #include "dialogbox.h"
 #include "mythcontext.h"
 #include "remoteutil.h"
@@ -52,8 +53,9 @@ ViewScheduled::ViewScheduled(MythMainWindow *parent, const char *name)
     }
     else
     {
-        cerr << "MythFrontEnd: ViewSchedule - Failed to get selector object.\n";
-        exit(29);
+        VERBOSE(VB_IMPORTANT, "ViewScheduled::ViewScheduled(): "
+                "Failed to get selector object.");
+        exit(FRONTEND_BUGGY_EXIT_NO_SELECTOR);
     }
 
     updateBackground();
@@ -62,18 +64,21 @@ ViewScheduled::ViewScheduled(MythMainWindow *parent, const char *name)
     inFill = false;
     needFill = false;
 
+    curcard = 0;
     listPos = 0;
     FillList();
  
     setNoErase();
 
     gContext->addListener(this);
+    gContext->addCurrentLocation("ViewScheduled");
 }
 
 ViewScheduled::~ViewScheduled()
 {
     gContext->SaveSetting("ViewSchedShowLevel", !showAll);
     gContext->removeListener(this);
+    gContext->removeCurrentLocation();
     delete theme;
 }
 
@@ -97,6 +102,8 @@ void ViewScheduled::keyPressEvent(QKeyEvent *e)
                 selected();
             else if (action == "MENU" || action == "INFO")
                 edit();
+            else if (action == "CUSTOMEDIT")
+                customEdit();
             else if (action == "UPCOMING")
                 upcoming();
             else if (action == "DETAILS")
@@ -117,6 +124,8 @@ void ViewScheduled::keyPressEvent(QKeyEvent *e)
                 setShowAll(false);
             else if (action == "PREVVIEW" || action == "NEXTVIEW")
                 setShowAll(!showAll);
+            else if (action == "VIEWCARD")
+                viewCards();
             else
                 handled = false;
         }
@@ -153,8 +162,9 @@ void ViewScheduled::LoadWindow(QDomElement &element)
                 parseContainer(e);
             else
             {
-                cerr << "Unknown element: " << e.tagName() << endl;
-                exit(30);
+                VERBOSE(VB_IMPORTANT,
+                        QString("ViewScheduled: Unknown child element: %1. "
+                                "Ignoring.").arg(e.tagName()));
             }
         }
     }
@@ -255,13 +265,25 @@ void ViewScheduled::FillList(void)
 
     QDateTime now = QDateTime::currentDateTime();
 
+    QMap<int, int> toomanycounts;
+
     ProgramInfo *p = recList.first();
     while (p)
     {
-        if (p->recendts >= now && (showAll || p->recstatus <= rsWillRecord || 
-                                   (p->recstatus > rsEarlierShowing && 
-                                    p->recstatus != rsRepeat)))
+        if ((p->recendts >= now || p->endts >= now) && 
+            (showAll || p->recstatus <= rsWillRecord || 
+             p->recstatus == rsDontRecord ||
+             (p->recstatus == rsTooManyRecordings && 
+              ++toomanycounts[p->recordid] <= 1) ||
+             (p->recstatus > rsTooManyRecordings && 
+              p->recstatus != rsRepeat)))
+        {
+            cardref[p->cardid]++;
+            if (p->cardid > maxcard)
+                maxcard = p->cardid;
+
             p = recList.next();
+        }
         else
         {
             recList.remove();
@@ -347,12 +369,18 @@ void ViewScheduled::updateList(QPainter *p)
 
                 if (p->recstatus == rsRecording)
                     ltype->EnableForcedFont(i, "recording");
-                else if (p->recstatus == rsConflict)
+                else if (p->recstatus == rsConflict ||
+                         p->recstatus == rsOffLine ||
+                         p->recstatus == rsAborted)
                     ltype->EnableForcedFont(i, "conflictingrecording");
                 else if (p->recstatus == rsWillRecord)
-                    ltype->EnableForcedFont(i, "record");
-                else if (p->recstatus <= rsEarlierShowing ||
-                         p->recstatus == rsRepeat)
+                    {
+                    if (curcard == 0 || p->cardid == curcard)
+                        ltype->EnableForcedFont(i, "record");
+                    }
+                else if (p->recstatus == rsRepeat ||
+                         (p->recstatus != rsDontRecord &&
+                          p->recstatus <= rsEarlierShowing))
                     ltype->EnableForcedFont(i, "disabledrecording"); 
             }
         }
@@ -522,6 +550,18 @@ void ViewScheduled::edit()
     p->EditScheduled();
 }
 
+void ViewScheduled::customEdit()
+{
+    ProgramInfo *p = recList[listPos];
+    if (!p)
+        return;
+
+    CustomEdit *ce = new CustomEdit(gContext->GetMainWindow(),
+                                    "customedit", p);
+    ce->exec();
+    delete ce;
+}
+
 void ViewScheduled::upcoming()
 {
     ProgramInfo *p = recList[listPos];
@@ -529,7 +569,7 @@ void ViewScheduled::upcoming()
     if (!p)
         return;
 
-    ProgLister *pl = new ProgLister(plTitle, p->title,
+    ProgLister *pl = new ProgLister(plTitle, p->title, "",
                                    gContext->GetMainWindow(), "proglist");
     pl->exec();
     delete pl;
@@ -587,3 +627,17 @@ void ViewScheduled::setShowAll(bool all)
 
     needFill = true;
 }
+
+void ViewScheduled::viewCards()
+{
+    needFill = true;
+
+    curcard++;
+    while (curcard <= maxcard)
+    {
+        if (cardref[curcard] > 0)
+            return;
+        curcard++;
+    }
+    curcard = 0;
+}
